Package: NOISeq
Type: Package
Title: Exploratory analysis and differential expression for RNA-seq
        data
Version: 2.56.0
Date: 2014-02-24
Author: Sonia Tarazona, Pedro Furio-Tari, Maria Jose Nueda, Alberto Ferrer and Ana Conesa
Maintainer: Sonia Tarazona <sotacam@eio.upv.es>
Depends: R (>= 2.13.0), methods, Biobase (>= 2.13.11), splines (>=
        3.0.1), Matrix (>= 1.2)
Description: Analysis of RNA-seq expression data or other similar kind
        of data. Exploratory plots to evualuate saturation, count
        distribution, expression per chromosome, type of detected
        features, features length, etc. Differential expression between
        two experimental conditions with no parametric assumptions.
License: Artistic-2.0
LazyLoad: yes
biocViews: ImmunoOncology, RNASeq, DifferentialExpression,
        Visualization, Sequencing
git_url: https://git.bioconductor.org/packages/NOISeq
git_branch: RELEASE_3_23
git_last_commit: a85acf6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:00:10 UTC; biocbuild
Built: R 4.6.0; ; 2026-04-29 04:53:03 UTC; unix

import(Biobase)
import(methods)
import(splines)
import(Matrix)

export("noiseq","noiseqbio","readData","addData","dat",
	"rpkm","tmm","uqua","degenes",
	"DE.plot", "filtered.data","QCreport",
	"ARSyNseq","PCA.GENES")

exportClasses("Output","Biodetection","CD","CountsBio","GCbias","lengthbias","Saturation","PCA")
exportMethods("show", "explo.plot","dat2save")

make.ASCA.design <- function(x)
{
x<-as.factor(x)
levels<-unique(x)
n<-length(x)
p<-length(levels)

Design<-matrix(0,nrow=n,ncol=p)

for (i in 1:n){
	for (j in 1:p){
	
	if (x[i]==levels[j])
	{
	Design[i,j]=1
	}
	}
}
colnames(Design)<-levels

output<-Design
output
}

####################################################################################################
#########  Algorithm to share information across genes when few replicates are available  ##########
####################################################################################################


## By Sonia Tarazona
## Created: 11-mar-2013




## Function to compute Z for noise when few replicates are available

share.info = function (mydata, n1, n2, r, nclust)   {
  
  # clustering data by k-means algorithm
  # 1.a) Normalized data
  gc()
  cl = suppressWarnings(kmeans(mydata, nclust, nstart = 25, iter.max = nclust + 30))

  cat("...k-means clustering done\n")
  cat(paste("Size of", nclust, "clusters:\n"))
  print(cl$size)
            
  # Creating pseudo-data
  cluster.data = lapply(1:nclust, function (k) { mydata[cl$cluster == k,] })
    
  # Resampling
  npermu = cl$size * r
  npermu = sapply(npermu, function (x) min(x, 1000))  ## modified to reduce the number of permutations to be done
  
  cat("Resampling cluster...")
  
  myres = vector("list", length = nclust)
  
  for (i in 1:nclust) {
    
    print(i)
    
#       if (cl$size[i] > 1) {   # OPTION 2.A
    if (cl$size[i] > 1 && cl$size[i] < 1000) {   # OPTION 2.C: small clusters
      
      myres[[i]] = t(sapply(1:npermu[i], function (j) {
        
        permu = sample(cluster.data[[i]])
        
        nn1 = n1*cl$size[i]
        nn2 = n2*cl$size[i]
        
        mean1 = mean(permu[1:nn1])
        mean2 = mean(permu[(nn1+1):(nn1+nn2)])
        
        sd1 = sd(as.numeric(permu[1:nn1]))
        sd2 = sd(as.numeric(permu[(nn1+1):(nn1+nn2)]))
        
        data.frame("M" = log2(mean1/mean2), "D" = mean1-mean2, 
                   "M.sd" = sqrt(sd1^2 / (mean1^2 * log(2)^2 * nn1) + 
                     sd2^2 / (mean2^2 * log(2)^2 * nn2)),
                   "D.sd" = sqrt(sd1^2/sqrt(nn1) + sd2^2/sqrt(nn2)))                
        
      }))      
    }
    
    
    if (cl$size[i] >= 1000) {  # OPTION 2.C & 2.D: big clusters
      
#       Option 2.D: clustering big clusters
      cl2 = kmeans(cluster.data[[i]], nclust, nstart = 25, iter.max = nclust + 20)
      
      cat(paste("Size of", nclust, "subclusters of cluster:", i)); cat("\n")
      print(cl2$size)
      subcluster.data = lapply(1:nclust, function (k) { cluster.data[[i]][cl2$cluster == k,] })
      
      npermu2 = cl2$size * r
      npermu2 = sapply(npermu2, function (x) min(x, 1000))  ## modified to reduce the number of permutations to be done
      myres2 = vector("list", length = nclust)
      
      for (h in 1:nclust) {
        if (cl2$size[h] > 1) {   
          
          myres2[[h]] = t(sapply(1:npermu2[h], function (j) {
            
            permu = sample(subcluster.data[[h]])
            
            nn1 = n1*cl2$size[h]
            nn2 = n2*cl2$size[h]       
            
            mean1 = mean(permu[1:nn1])
            mean2 = mean(permu[(nn1+1):(nn1+nn2)])
            
            sd1 = sd(as.numeric(permu[1:nn1]))
            sd2 = sd(as.numeric(permu[(nn1+1):(nn1+nn2)]))
            
            data.frame("M" = log2(mean1/mean2), "D" = mean1-mean2, 
                       "M.sd" = sqrt(sd1^2 / (mean1^2 * log(2)^2 * nn1) + 
                                       sd2^2 / (mean2^2 * log(2)^2 * nn2)),
                       "D.sd" = sqrt(sd1^2/sqrt(nn1) + sd2^2/sqrt(nn2)))              
          }))      
        }
      }
      myres[[i]] = do.call("rbind", myres2)
    }
  }                    

  
  # Computing Zr for noise 
  cat("Computing Z for noise...\n")
  
  # 4.A) a0: Global for all R*G permutations
  myres = do.call("rbind", myres)
  
  
  a0.M <- quantile(as.numeric(myres[,"M.sd"]), probs = 0.9, na.rm = TRUE)
  a0.D <- quantile(as.numeric(myres[,"D.sd"]), probs = 0.9, na.rm = TRUE)
  
  M <- as.numeric(myres[,"M"]) / (a0.M + as.numeric(myres[,"M.sd"]))
  D <- as.numeric(myres[,"D"]) / (a0.D + as.numeric(myres[,"D.sd"]))
  (M + D) / 2


}

ASCAfun.triple<-function (X,Desa,Desb,Desc,Fac) {

n <- ncol(X) # number of genes
I <- ncol(Desa) # number of levels in the factor TIME
J <- ncol(Desb) # number of levels in the other factor
H <- ncol(Desc) # number of levels in the other factor

#Matrices con medias efectos individuales
XK1<-matrix(NA,nrow=I,ncol=n)
for (i in 1:I) {
     sub<-X[Desa[,i]==1,]
     XK1[i,]<-apply(sub,2,mean)
}


XK2<-matrix(NA,nrow=J,ncol=n)
for (j in 1:J) {
     sub<-X[Desb[,j]==1,]
     XK2[j,]<-apply(sub,2,mean)
}

XK3<-matrix(NA,nrow=H,ncol=n)
for (h in 1:H) {
     sub<-X[Desc[,h]==1,]
     XK3[h,]<-apply(sub,2,mean)
}

#Matrices con medias de efectos simples

XK12<-matrix(NA,nrow=I*J,ncol=n)
k=1
for (j in 1:J){
  for (i in 1:I){
    sub<-X[(Desa[,i]+Desb[,j])==2,]
    XK12[k,]<-apply(sub,2,mean)
    k=k+1
  }
}

XK13<-matrix(NA,nrow=I*H,ncol=n)
k=1
for (h in 1:H){
  for (i in 1:I){
    sub<-X[(Desa[,i]+Desc[,h])==2,]
    XK13[k,]<-apply(sub,2,mean)
    k=k+1
  }
}


XK23<-matrix(NA,nrow=J*H,ncol=n)
k=1
for (h in 1:H){
  for (j in 1:J){
    sub<-X[(Desb[,j]+Desc[,h])==2,]
    XK23[k,]<-apply(sub,2,mean)
    k=k+1
  }
}


NK<-matrix(NA,nrow=I,ncol=J*H)
XK<-matrix(NA,nrow=I*J*H,ncol=n)

k=1
for (h in 1:H){
 for (j in 1:J){
  for (i in 1:I){
    sub<-as.matrix(rbind(X[(Desa[,i]+Desb[,j]+Desc[,h])==3,]))
    NK[i,(h-1)*J+j]<-sqrt(nrow(sub))
    XK[k,]<-apply(sub,2,mean)+XK1[i,]+XK2[j,]+XK3[h,]-XK12[(j-1)*I+i,]-XK13[(h-1)*I+i,]-XK23[(h-1)*J+j,]
    k=k+1
  }
 }
}

XKw<-XK*(as.numeric(NK))

PCA<-PCA.GENES(XKw)
      scw<-PCA$scores[,1:Fac]
      ld<-PCA$loadings[,1:Fac]
      ssq<-PCA$var.exp
      if(Fac==1) {
      scw<-as.matrix(scw)
      ld<-as.matrix(ld) 
      }      
  if(Fac==0) {
      scw<-as.matrix(rep(0,I*J*H))
      ld<-as.matrix(rep(0,n))
	}        
# Re-weigth the scores
sc<-scw/(as.numeric(NK))
    
XKrec<-sc%*%t(ld)

Xabc<-NULL
TPabc<-NULL
for (i in 1:nrow(X)){
     position1<-which(Desa[i,]==1)
     position2<-which(Desb[i,]==1)
     position3<-which(Desc[i,]==1)
     Xabc<-rbind(Xabc,XK[I*(position2-1)+I*J*(position3-1)+position1,])
     TPabc<-rbind(TPabc,XKrec[I*(position2-1)+I*J*(position3-1)+position1,])
}
Eabc<-Xabc-TPabc

 #Leverage & SPE 
    leverage<-apply(ld^2,1,sum)
    SPE<-apply(Eabc^2,2,sum)

output<-list(XK,sc,ld,ssq,Xabc,TPabc,Eabc,leverage,SPE)
names(output)<-c("data","scores","loadings","var.exp","X","TP","E","leverage","SPE")
output


}

ARSyNseq <- function(data, factor = NULL, batch = FALSE, norm = "rpkm", logtransf = FALSE, Variability = 0.75, beta = 2)
{

	#   data:     A Biobase's eSet object created with the readData function
	#   factor:   Column name choosen from factors argument of data. 
	#             When it is NULL, all the factors (1,2 or 3) specified in data are considered.
	#   batch:    TRUE when the factor is an identified batch effect. This option can be run only with 1 factor.
	#   norm:     Normalization method. It can be one of "rpkm" (default), "uqua"
	#             upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).

	#------- Parameters for PCs selection: ----------------------
	#Variability: Parameter for PCs selection of the ANOVA models effects.
	#    beta:    Parameter for PCs selection of the residual model. 

	#------- Only used for 2 or 3 factors:---------------------
	#    Join:    Logical to indicate whether interaction Factor1xFactor2 must be analysed jointly with the second factor. 
	#Interaction: Logical to indicate whether interaction/s between factors should be analyzed.

	#------- Arguments for normalization: ----------------------
	#    k:       Counts equal to 0 are replaced by k. By default, no replacement, k=0.  
	#    lc:      Length correction is done by dividing expression by length^lc. By default, no correction, lc = 1.

	#----------------------------------
	# --- Compute Inputs for ARSyN
	#----------------------------------


	Join = TRUE
	Interaction = TRUE


	 dat <- as.matrix(assayData(data)$exprs)
	 long <- featureData(data)@data[rownames(dat), "Length"]
	 if (is.null(long)) long = 1000

	 if (norm == "rpkm") {     
	   dat <- rpkm(dat, long = long, k = 0, lc = 1)
	  }

	 if (norm == "uqua") {
	   dat <- uqua(dat, long = long, lc = 1, k = 0)
	    }
	   
	 if (norm == "tmm") {
	    dat <- tmm(dat, long = long, lc = 1, k = 0)      
	    }

	 if (!logtransf)   dat <- log(dat + 1)
	 
	X <- t(dat) 		#conditions x genes


	#----------------------------------

	 if(is.null(factor))
	{
	 Covariates <- t(pData(data))
	 Num.factors <- nrow(Covariates)
	 labels.factors <- rownames(Covariates)
	 Design <- list(NULL,NULL,NULL)
	 for (i in 1:Num.factors)
	 {
		x <- as.character(Covariates[i,])
		Design[[i]] <- make.ASCA.design(x)
	 } 
	}else{
	 Covariates <- pData(data)[,factor]
	 Num.factors <- 1
	 labels.factors <- factor
	 Design <- list(NULL,NULL,NULL)
		x <- as.character(Covariates)
		Design[[1]] <- make.ASCA.design(x)
	 }

	####################################
	### --- Execute ASCAmodel 
	####################################

	 my.asca <- ARSyNmodel(Factors=Num.factors,X=X,Designa=Design[[1]],Designb=Design[[2]],Designc=Design[[3]],Join=Join,Interaction=Interaction,Variability=Variability,beta=beta)

	#################################### 
	### --- Writing filtered matrix 
	####################################

	 X.filtered <- X
	 M<-length(my.asca)-1

	if(!batch)
	{
	# for (i in 1:(M-1))
	#  {
	#	X.filtered <- X.filtered-my.asca[[i]]$E 
	#  }
		X.filtered <- X.filtered-my.asca[[M]]$TP
	}

	if(batch)
	{
		X.filtered <- X.filtered-my.asca[[1]]$TP 
	}

	data.filtered <- t(X.filtered)

	 if (!logtransf)  data.filtered <- exp(data.filtered)+1
	                     
	exprs(data) = data.filtered

	return(data)
}

ASCAfun1<-function (X,Design,Fac) {

n <- ncol(X) # number of genes
I <- ncol(Design) # number of levels in the factor

NK<-NULL
XK<-matrix(NA,nrow=I,ncol=n)

for (i in 1:I) {
     sub<-X[Design[,i]==1,]
  if(is.null(nrow(sub))){ #when there isn't replicates
    NK[i] <- 1
    XK[i,] <- sub 
}else{
     NK[i]<-nrow(sub)
     XK[i,]<-apply(sub,2,mean) }
}
  NK<-sqrt(NK)

# Weigh the data of the Submodel with the corresponding number of measurement occasions

  XKw<- NK*XK

  PCA<-PCA.GENES(XKw)
      scw<-PCA$scores[,1:Fac]
      ld<-PCA$loadings[,1:Fac]
      ssq<-PCA$var.exp
      if(Fac==1) {
      scw<-as.matrix(scw)
      ld<-as.matrix(ld)
}      
 if(Fac==0) {
      scw<-as.matrix(rep(0,I))
      ld<-as.matrix(rep(0,n))
	}        
# Re-weigth the scores
    sc<-scw/NK

XKrec<-sc%*%t(ld)

Xa<-NULL
TPa<-NULL
for (i in 1:nrow(X)){
    position<-which(Design[i,]==1)
    Xa<-rbind(Xa,XK[position,])
    TPa<-rbind(TPa,XKrec[position,])
}

Ea<-Xa-TPa

#Leverage & SPE 
    leverage<-apply(ld^2,1,sum)
    SPE<-apply(Ea^2,2,sum)

output<-list(XK,sc,ld,ssq,Xa,TPa,Ea,leverage,SPE)
names(output)<-c("data","scores","loadings","var.exp","X","TP","E","leverage","SPE")
output



}

##### Plot to compare count distributions for two or more samples


### Generating data

cd.dat <- function (input, norm = FALSE, refColumn = 1) {
  
  if (inherits(input,"eSet") == FALSE)
    stop("ERROR: The input data must be an eSet object.\n")
    
  
  if (!is.null(assayData(input)$exprs)) {
    if (ncol( assayData(input)$exprs) < 2)
      stop("ERROR: The input object should have at least two samples.\n")
  
    datos <- assayData(input)$exprs
    
  } else {
    if (ncol( assayData(input)$counts) < 2)
      stop("ERROR: The input object should have at least two samples.\n")
    
    datos <- assayData(input)$counts
    
  }
  
  
  ceros = which(rowSums(datos) == 0)
  hayceros = (length(ceros) > 0)
  
  if (hayceros) {
    print(paste("Warning:", length(ceros), 
                "features with 0 counts in all samples are to be removed for this analysis."))
    datos = datos[-ceros,]
  }  
  
  
  ## scaling data and/or changing 0 to k
  if (norm) {
    datos = sinceros(datos, k = NULL)
  } else { 
    datos = rpkm(datos, long = 1000, lc = 1, k = 0.5)
  }
  
  
  
  ## to plot
  data2plot = log2(datos / datos[,refColumn])
  
  if (is.numeric(refColumn)) refColumn = colnames(datos)[refColumn]
  print(paste("Reference sample is:", refColumn))
  
  
  #### Diagnostic test
    
  MsinRef = as.matrix(data2plot[,-match(refColumn, colnames(data2plot))])
  colnames(MsinRef) = colnames(data2plot)[-match(refColumn, colnames(data2plot))]
  alpha = 0.05
  alpha = alpha/ncol(MsinRef)
  nperm = 10^3
    
  bootmed = sapply(1:nperm, function(k) {
    permut = sample(1:nrow(MsinRef), replace = TRUE, nrow(MsinRef))
    permut = as.matrix(MsinRef[permut,])
    permut = apply(permut, 2, median)   
    permut
  })
  
  if (is.null(dim(bootmed))) bootmed = t(as.matrix(bootmed))
  
  bootmed = t(apply(bootmed, 1, quantile, probs = round(c(alpha/2, 1 - alpha/2), 4)))
  diagno = apply(bootmed, 1, 
                 function (x) { 
                   ddd =  (x[1] <= 0) * (0 <= x[2])
                   if (ddd == 1) { ddd = "PASSED" } else { ddd = "FAILED"}
                   ddd
                   })
  bootmed = cbind(bootmed, diagno)
    
  rownames(bootmed) = colnames(MsinRef)
  colnames(bootmed)[3] = "Diagnostic Test"
  print("Confidence intervals for median of M:")
  print(bootmed)
    
  if ("FAILED" %in% bootmed[,3]) {
    print("Diagnostic test: FAILED. Normalization is required to correct this bias.")
  }  else {
    print("Diagnostic test: PASSED.")
  }
  
  
  #### Results
  
  list("data2plot" = data2plot, "refColumn" = refColumn, "DiagnosticTest" = bootmed)
   
}


###########################################################################
###########################################################################
###########################################################################




### Generating plot

cd.plot <- function (dat, samples = NULL,...) {
  
  refColumn = dat$refColumn
  dat = dat$data2plot
  
  if (is.null(samples)) samples <- 1:ncol(dat)
  
  if (is.numeric(samples)) { samples = colnames(dat)[samples] }
  
  samples = setdiff(samples, refColumn)
  
  if(length(samples) > 12) stop("Please select 12 samples or less to be plotted (excluding reference).")
      
  dat = as.matrix(dat[,samples])
  
  dat.dens = apply(dat, 2, density, adjust = 1.5)
  limY = c(0,max(sapply(dat.dens, function (x) max(x$y, na.rm = TRUE))))
  
  plot(dat.dens[[1]], xlab = "M = log2(sample/refsample)", ylab = "Density", lwd = 2, ylim = limY,
       type = "l", col = miscolores[1], main = paste("Reference sample:", refColumn), ...)
  abline(v = median(dat[,1], na.rm = TRUE), col = miscolores[1], lty = 2)
  
  if (length(samples) > 1) {
    for (i in 2:length(samples)) {
      lines(dat.dens[[i]], col = miscolores[i], lwd = 2)
      abline(v = median(dat[,i], na.rm = TRUE), col = miscolores[i], lty = i+1)
    }
  }  

  legend("topleft", legend = samples, text.col = miscolores[1:length(samples)], bty = "n",
         lty = 1, lwd = 2, col = miscolores[1:length(samples)])

}

##### Function to generate data for exploratory plots #####

# By Sonia & Pedro
# Modified: 2-jun-15

dat = function (input, type = c("biodetection","cd","countsbio","GCbias","lengthbias","saturation","PCA"), 
                k = 0, ndepth = 6, factor = NULL, norm = FALSE, refColumn = 1, logtransf = FALSE) {
  
  type <- match.arg(type)
  
  if (type == "biodetection") {
    
    output = new("Biodetection", dat = biodetection.dat(input, factor = factor, k = k))    
  }
  
  if (type == "cd") {
    
    output = new("CD", dat = cd.dat(input, norm = norm, refColumn = refColumn))
  }
  
  if (type == "countsbio") {
    
    output =  new("CountsBio", dat = countsbio.dat(input, factor = factor, norm = norm))
  }


  if (type == "GCbias") {
    
    output =  new("GCbias", dat = GC.dat(input, factor = factor, norm = norm))
  }

  
  if (type == "lengthbias") {
    
    output =  new("lengthbias", dat = length.dat(input, factor = factor, norm = norm))
  }
  
  if (type == "saturation") {
    
    output = new("Saturation", dat = saturation.dat(input, k = k, ndepth = ndepth))
  }
  
  if (type == "PCA") {
    
    output = new("PCA", dat = PCA.dat(input, norm = norm, logtransf = logtransf))
  }
  
  
  output  
  
}

probdeg <-
function (Mg, Dg, Mn, Dn, prec = 2) {

  # Mg, Dg -> signal
  # Mn, Dn -> noise
  # prec = precission (number of digits to round M and D)

  tot <- length(Mn)   # number of points in noise distribution

  gens <- names(Mg)

  Mruido <- abs(round(Mn, prec))
  Druido <- round(Dn, prec)
  Mgen <- abs(round(Mg, prec))
  Dgen <- round(Dg, prec)

  MDgen <- na.omit(cbind(Mgen, Dgen))

  MDunic <- unique(MDgen)  

  Nres <- apply(MDunic, 1, n.menor, S1 = Mruido, S2 = Druido)
  
  lugares <- apply(MDgen, 1, busca, S = MDunic)

  Nconj <- Nres[lugares]

  names(Nconj) <- names(lugares)

  laprob <- Nconj / tot

  laprob <- laprob[gens]
  names(laprob) <- gens

  Nconj <- Nconj[gens]
  names(Nconj) <- gens

  laprob <- list("prob" = laprob, "numDE" = Nconj, "numNOISE" = tot)

  laprob

}

#######################

tmm = function (datos, long = 1000, lc = 0, k = 0, refColumn = 1,
                logratioTrim = .3, sumTrim = 0.05, doWeighting = TRUE,
                Acutoff = -1e10) {
  
  # lc: Length correction. Expression is divided by long^lc. lc can be any real number.
  
  if (!is.null(ncol(long))) {
    mynames = long[,1]
    long = long[,2]
    names(long) = mynames
  }
  
  L <- (long/1000)^lc
  datos = datos/L
  
  total <- colSums(as.matrix(datos))
  
  datos0 <- sinceros(datos, k)
  
  if (ncol(as.matrix(datos)) > 1) {
    
    fk <- .calcNormFactors(as.matrix(datos), refColumn = refColumn, method = "TMM",
                           logratioTrim = logratioTrim, sumTrim = sumTrim,
                           doWeighting = doWeighting, Acutoff = Acutoff)
          
    fk = fk * (total/mean(total))
    
    datos.norm <- t(t(datos0)/fk)
    
  } else {
    
    datos.norm <- datos0/L
    
  }
  
  na.omit(datos.norm)  
  
}


#######################


rpkm <- function (datos, long = 1000, lc = 1, k = 0) {
  
  if (!is.null(ncol(long))) {
    mynames = long[,1]
    long = long[,2]
    names(long) = mynames
  }

  total <- colSums(as.matrix(datos))

  datos0 <- sinceros(datos, k)                   

  datos.norm <- (t(t(datos0)/total)*10^6)/((long/1000)^lc)
  
  na.omit(datos.norm)   
}


##################################


uqua <- function (datos, long = 1000, lc = 0, k = 0) {
  
# lc: Length correction. Expression is divided by long^lc. lc can be any real number.
  
  if (!is.null(ncol(long))) {
    mynames = long[,1]
    long = long[,2]
    names(long) = mynames
  }

  L <- (long/1000)^lc
  datos = datos/L

  datos0 <- sinceros(datos, k)

  if (ncol(as.matrix(datos)) > 1) {
    sumatot <- rowSums(datos)
    supertot <- sum(sumatot)
    counts0 <- which(sumatot == 0)

    if (length(counts0) > 0) {
      datitos <- datos[-counts0,]
    } else {
      datitos <- datos
    }

    q3 <- apply(datitos, 2, quantile, probs = 0.75)
    d <- q3*supertot/sum(q3)

    datos.norm <- t(t(datos0)/d)*10^6

  } else {

    datos.norm <- datos0/L

  }
  
  na.omit(datos.norm)  
  
}



##################################

## Taken from the edgeR package with minor modifications

.calcNormFactors <- function(object, method=c("TMM","quantile"), refColumn=NULL,
                            logratioTrim=.3, sumTrim=0.05, doWeighting=TRUE, Acutoff=-1e10, 
                            quantile=0.75) {
                            
  method <- match.arg(method)
                            
  if( is.matrix(object) ) {
    if(is.null(refColumn))
      refColumn <- 1
    data <- object
    libsize <- colSums(data)
  } else {
    stop("calcNormFactors() only operates on 'matrix' objects")
  }

  f <- switch(method,
              TMM = apply(data,2,.calcFactorWeighted,ref=data[,refColumn], 
                          logratioTrim=logratioTrim, sumTrim=sumTrim, doWeighting=doWeighting, 
                          Acutoff=Acutoff),              
              quantile = .calcFactorQuantile(data, libsize, q=quantile))
              
  f <- f/exp(mean(log(f)))

  return(f)
  
}

.calcFactorQuantile <- function (data, lib.size, q=0.75) {
    y <- t(t(data)/lib.size)
    f <- apply(y,2,function(x) quantile(x,p=q))
    f/exp(mean(log(f)))
}


.calcFactorWeighted <- function(obs, ref, logratioTrim=.3, sumTrim=0.05, doWeighting=TRUE, Acutoff=-1e10) {

  if( all(obs==ref) )
    return(1)

  obs <- as.numeric(obs)
  ref <- as.numeric(ref)

  nO <- sum(obs)
  nR <- sum(ref)
  logR <- log2((obs/nO)/(ref/nR))          # log ratio of expression, accounting for library size
  absE <- (log2(obs/nO) + log2(ref/nR))/2  # absolute expression
  v <- (nO-obs)/nO/obs + (nR-ref)/nR/ref   # estimated asymptotic variance
  
  # remove infinite values, cutoff based on A
  fin <- is.finite(logR) & is.finite(absE) & (absE > Acutoff)
  
  logR <- logR[fin]
  absE <- absE[fin]
  v <- v[fin]
  
  # taken from the original mean() function
  n <- sum(fin)
  loL <- floor(n * logratioTrim) + 1
  hiL <- n + 1 - loL
  loS <- floor(n * sumTrim) + 1
  hiS <- n + 1 - loS
  
  #keep <- (rank(logR) %in% loL:hiL) & (rank(absE) %in% loS:hiS)
  # a fix from leonardo ivan almonacid cardenas, since rank() can return
  # non-integer values when there are a lot of ties
  keep <- (rank(logR)>=loL & rank(logR)<=hiL) & (rank(absE)>=loS & rank(absE)<=hiS)
  
  if (doWeighting) 
    2^( sum(logR[keep]/v[keep], na.rm=TRUE) / sum(1/v[keep], na.rm=TRUE) )
  else
    2^( mean(logR[keep], na.rm=TRUE) )
}

ASCA.2f<-function(X = X,Designa = Designa, Designb = Designb, Designc=NULL, Fac=c(1,2,2,2),Join = TRUE,Interaction=TRUE)

{
#--------------------------------------------------------------------------------------
#  Dimensions of the matrices: 
#  X (p x n) contains expression values of n genes (in columns) and p conditions (in rows)
#  Designa (p x I) contains 0's and 1's for the TIME-POINTS in the experiment
#  Designb (p x J) EXPERIMENTAL GROUP  
#  Designres (p x H) INDIVIDUALS

#  Join = TRUE if the analyses of the model b and ab is studied jointly
#  Interaction = TRUE to consider interaction "ab" in the separated model

n<-ncol(X)
p<-nrow(X)
I<-ncol(Designa)
J<-ncol(Designb)

Faca=Fac[1] # number components Model a (time)
Facb=Fac[2] # number components Model b  (second factor)
Facab=Fac[3] # number components Model ab (interaction) 
Facres=Fac[4] # number components Residues

#----------------------- Calculate Overall Mean --------------------------------------

offset<-apply(X,2,mean)
Xoff<-X-(cbind(matrix(1,nrow=p,ncol=1))%*%rbind(offset))


#-----------------------  PART I: Submodel a (TIME) -----------------------------------

	Model.a<-ASCAfun1(Xoff,Designa,Faca)
	Xres<-Xoff-Model.a$X

#-------------------------- PART II: Submodel b and ab -------------------------------

if(!Join) 
{
	Model.b<-ASCAfun1(Xoff,Designb,Facb)
	if (Interaction) {
		Model.ab<-ASCAfun2(Xoff,Designa,Designb,Facab)
	}
}

if(Join)
{
	Model.bab<-ASCAfun12(Xoff,Designa,Designb,Facab)
}

# ------------------------Collecting models ------------------------------------------

models <- ls(pattern="Model")
output <- vector(mode="list")
Xres <- Xoff
for (i in 1: length(models)) {
	mymodel <- get(models[i], envir=environment())
	output <- c(output, list(mymodel))
	Xres <- Xres - mymodel$X
	rm(mymodel)
	gc()
	
}
names(output) <- models

#------------------------- PART III: Submodel res -----------------------------------

Model.res<-ASCAfun.res(Xres,Facres)

Model.res<-list(Model.res)
names(Model.res)<-c("Model.res")
output<-c(output,Model.res)


#------------------------- Add Input data to the Output ----------------------------

Input<-list(X, Designa, Designb, Designc, Fac, Join,Interaction,Xoff)
names(Input)<-c("X", "Designa", "Designb", "Designc", "Fac", "Join","Interaction","Xoff")
Input<-list(Input)
names(Input)<-"Input"
output<-c(output,Input)

output 
}

#### SATURATION PLOTS

## Data for saturation plot (with or without biotypes)

saturation.dat <- function (input, k = 0, biotypes = NULL, ndepth = 6) {

  # input: input object. 
  # k: A feature is considered to be detected if the corresponding number of counts is > k.
  # biotypes: List containing groups of biotypes to be studied. 
  #           If biotypes = NULL, all biotypes are plotted independently.
  # ndepth: Number of different depths to be plotted.


  if (inherits(input,"eSet") == FALSE)
    stop("Error. You must give an eSet object\n")

  if (!is.null(assayData(input)$exprs))
    datos <- assayData(input)$exprs
  else
    datos <- assayData(input)$counts
  
  if (!is.null(featureData(input)$Biotype)) {  # read biotypes if they are provided
    infobio <- featureData(input)$Biotype    
  } else { infobio = NULL }
  
    
  nsam <- NCOL(datos)

  if (!is.null(infobio)) {    
    if(is.null(biotypes)) {
      biotypes <- unique(infobio)
      names(biotypes) <- biotypes         
    }   
  } else { biotypes = NULL }   

  
  
  satura <- vector("list", length = length(biotypes)+1)
  names(satura) <- c("global", names(biotypes))
  
  ndepth1 = ceiling(ndepth/2)
  
  datos = round(datos, 0)
#   datos0 = datos
#   datos0[datos0 == 0] = 0.05
  datos0 = datos + 0.2
    
  
  # Random subsamples for each sample
  submuestras <- seq.depth <- vector("list", length = nsam)
  names(submuestras) <- names(seq.depth) <- colnames(datos)

  for (n in 1:nsam) {  # simulating subsamples for each sample

    total <- sum(datos[,n]) # total counts in sample n

    varias <- vector("list", length = ndepth+1) # simulation for each depth and real depth    

    for (i in 1:(ndepth1)) {    # simulating depths < real depth

      muestra <- rmultinom(10, size = round(total/(ndepth1+1),0), prob = datos[,n])
      
      if (i == 1) { 
        varias[[i]] <- muestra 
      } else { varias[[i]] <- varias[[i-1]] + muestra }
    }

    varias[[ndepth1+1]] <- as.matrix(datos[,n])
    
    for (i in (ndepth1+2):(ndepth+1)) {    # simulating depths < real depth
      
      muestra <- rmultinom(10, size = round(total/(ndepth1+1),0), prob = datos0[,n])
      
      if (i == ndepth1+2) { 
        varias[[i]] <- matrix(varias[[i-1]], ncol = 10, nrow = nrow(varias[[i-1]])) + muestra 
      } else { varias[[i]] <- varias[[i-1]] + muestra }
    }    

    submuestras[[n]] <- varias

    seq.depth[[n]] <- c(round(total/(ndepth1+1),0)*(1:ndepth1), total, 
                        round(total/(ndepth1+1),0)*((ndepth1+2):(ndepth+1)))
  }


  
  # Global saturation

  satura[[1]] <- vector("list", length = nsam)
  names(satura[[1]]) <- colnames(datos)

  for (n in 1:nsam) { # for each sample

    satura[[1]][[n]] <- sapply(submuestras[[n]],
                               function(x) { mean(apply(x, 2, noceros, k = k)) })                               
  }
  

  # Per biotypes
  if (!is.null(infobio)) { # if biotypes available

    biog <- lapply(biotypes, function(x) { which(is.element(infobio, x)) })
    names(biog) = names(biotypes)

    for (j in 2:length(satura))  { # for each biotype

      satura[[j]] <- vector("list", length = nsam)
      names(satura[[j]]) <- colnames(datos)

      for (n in 1:nsam) { # for each sample

        conbio <- lapply(submuestras[[n]], function(x) { as.matrix(x[biog[[j-1]],]) })

        satura[[j]][[n]] <- sapply(conbio, function(x) { mean(apply(x, 2, noceros, k = k)) })
      }
    }
  } else { biog <- NULL }



  # computing detection increasing per million reads
  newdet <- vector("list", length = 1+length(biotypes))
  names(newdet) <- c("global", names(biotypes))
  
  for (j in 1:length(newdet))  {
    
    newdet[[j]] <- vector("list", length = nsam)
    names(newdet[[j]]) <- colnames(datos)
    
    for (n in 1:nsam) {      
      puntos <- data.frame("x" = seq.depth[[n]], "y" = satura[[j]][[n]])      
      pendi <- NULL
      
      for(i in 2:nrow(puntos)) {        
        pendi <- c(pendi,
                   (puntos$y[i]-puntos$y[i-1])/(puntos$x[i]-puntos$x[i-1]))
      }
      
      newdet[[j]][[n]] <- c(NA,pendi*1000000)
      
    }
  }
  



  bionum <- c(NROW(datos), sapply(biog, length))
  names(bionum) <- c("global", names(biog))
  
  
  # Results at real sequencing depth
  real = vector("list", length = length(satura))
  names(real) = names(satura)
  realdepth = sapply(seq.depth, function (x) x[ndepth1+1])/10^6
  
  for (i in 1:length(real)) {
    real[[i]] = data.frame("depth" = realdepth,
                           "detec" = sapply(satura[[i]], function (x) x[ndepth1+1]))
    rownames(real[[i]]) = colnames(datos)
  }
  

  # Results

  satura <- list("saturation" = satura, "bionum" = bionum,
                 "depth" = seq.depth, "newdet" = newdet, "real" = real)
  
  satura

}






##**************************************************************************#
##**************************************************************************#
##**************************************************************************#



#### Saturation plot

saturation.plot <- function(satdat, samples = NULL, toplot = 1, 
                            yrightlim = NULL, toreport = FALSE, yleftlim = NULL, ...) {

  # satdat: Data coming from saturation.dat function
  # samples: Samples to be plotted. If NULL, all samples are plotted (Maximum = 12).
  # toplot: Number or name of biotype (including "global") to be plotted.
  # colL, colR, mybg: A vector with as many colors as different samples to be plotted.
  #                   If NULL, default colors are used.
  
  mypar = par(no.readonly = TRUE)
  
  # Parameters
  lwdL = 2
  lwdR = 10
  xlab = "Sequencing depth (million reads)"
  ylabL = "Number of detected features"
  ylabR = "New detections per million reads"
  cex.main = cex.lab = cex.axis = 1
  cex = 0.8
  
  
  if (is.null(samples)) {
    samples <- 1:length(sat)
  }  


  # Preparing data
  sat <- satdat$saturation[[toplot]]
  depth <- satdat$depth
  num <- satdat$bionum[[toplot]]
  nuevo <- satdat$newdet[[toplot]]
  real = satdat$real[[toplot]][samples,]
    
  
  if (is.numeric(toplot)) {
      main <- paste(toupper(names(satdat[[1]])[toplot]), " (", num, ")", sep = "")
  } else {
      main <- paste(toupper(toplot), " (", num, ")", sep = "")
  }

  
  legend = names(satdat$saturation[[1]])[samples]
  if (toreport) legend = samples

  
  # colors
  miscolores <- colors()[c(554,89,111,512,17,586,132,428,601,568,86,390)] 
  
  if (length(samples) > 2) {
      colL <- miscolores
  }  else {
      colL <- miscolores[c(4,2)]
  }  

  colR <- miscolores[c(12,11)] 

  mybg <- colL

  

  # xlim for plot
  xlim <- range(unlist(depth[samples])/10^6)
  
  # yleftlim
  if (is.null(yleftlim)) {
    yleftlim <- range(unlist(sat[samples]))
  } else { yleftlim = yleftlim }


  # Percentage of detections at real depth
  percen <- round(100*real[,"detec"]/num, 1)


  # Drawing new detections bars    
  if (length(samples) <= 2) {
    bars <- TRUE
  } else {
    bars <-  FALSE
  } 


  
  ## PLOTS
  
  if(!bars) {  # PLOT for detections without bars for new detections

    plot(depth[[samples[1]]]/10^6, sat[[samples[1]]], pch = 21, col = colL[1], #bg = mybg[1],
         lwd = lwdL, ylim = yleftlim,
         xlim = xlim, main = main, type = "b", xlab = xlab,
         ylab = ylabL, cex.main = cex.main, cex.lab = cex.lab,
         cex.axis = cex.axis, ...)

    if (length(samples) > 1) {  # for more than 1 sample

      j <- 2

      for (i in samples[-1]) {
        lines(depth[[i]]/10^6, sat[[i]], pch = 21, col = colL[j], #bg = mybg[j], 
              lwd = lwdL, type = "b")
        j <- j+1
      }
    }
    
    points(real, pch = 21, col = colL[1:length(samples)], bg = mybg[1:length(samples)])

    legend("bottom", legend = paste(legend, ": ", percen, "% detected", sep = ""),
           pch = 21, pt.bg = mybg, text.col = colL, bty = "n", ncol = 2,
           lwd = lwdL, col = colL, cex = cex)

    

  } else {   # PLOT for detections and new detections

    # yrightlim for plot.y2
    if (is.null(yrightlim)) {
      yrightlim <- c(0, max(10,max(na.omit(unlist(nuevo[samples])))))
    }
    
    
        
    if (!toreport) nf <- layout(matrix(c(1,2),2,1,byrow=TRUE),heights=c(0.8,0.2))
    
      
    par(mar = c(5, 4, 4, 4) + 0.1)


    # PLOT with 2 axis
    plot.y2(x = depth[[samples[1]]]/10^6, yright = nuevo[[samples[1]]],
            yleft = sat[[samples[1]]], type = c("h", "o"),
            lwd = c(lwdR, lwdL), xlab = xlab, xlim = xlim,
            yrightlim = yrightlim, yleftlim = yleftlim,
            yylab = c(ylabR, ylabL), pch = c(1,21), col = c(colR[1],colL[1]),
            main = main, x2 = depth[[samples[2]]]/10^6,
            yright2 = nuevo[[samples[2]]], yleft2 = sat[[samples[2]]],
            col2 = c(colR[2],colL[2]), cex.main = cex.main, #bg = mybg,
            cex.lab = cex.lab, cex.axis = cex.axis, cex = cex, ...)
    
    points(real, pch = 21, col = colL, bg = mybg)

    par(mar = c(0,0,0,0))
    plot(0,axes=FALSE,type="n")
    
    #HEADERS
    rect(0.7,-0.7, 1.3, 1, col = "grey90", border = "grey90") 
    text(0.93, 0.7,"Left axis", font = 3,cex=1.2)
    text(1.07, 0.68, "Right axis", font = 3, cex = 1.2)
    text(1.22,0.7,"%detected", font = 3, cex = 1.2)
    
    # The rest of the legend arguments
    text(0.72,0.15,legend[1], font = 2, adj=0)
    points(0.93, 0.15, lty = 1, pch = 21,
           col = colL[1], bg = mybg[1])
    points(1.07, 0.15, pch = "-",
           col = colR[1], cex = lwdR)
    text(1.24, 0.15, percen[1],adj=1)

    if (length(samples) == 2) {

      text(x = 0.72,-0.25, legend[2], font = 2, adj=0)
      points(0.93,-0.25, lty = 1, pch = 21,
             col = colL[2], bg = mybg[2])
      points(1.07, -0.25, pch = "-",
             col = colR[2], cex = lwdR)
      text(1.24, -0.25, percen[2],adj=1)
    }
    # Reset with the default values    
    if (!toreport) par(mypar); layout(1) 
    
  }
}

ARSyNmodel<-function(X = X, Factors = 2, Designa = Designa, Designb = Designb, Designc = Designc, Variability="average",
Join =TRUE, Interaction=TRUE, beta=2)
{

if(Factors==1)
{
  Fac0<-c(1,2)
  names(Fac0)<-c("Model.a","Model.res")
  asca0<- ASCA.1f(X=X, Designa=Designa, Fac=Fac0)
  Fac<-ARSyNcomponents(asca0,Variability=Variability,beta=beta)
  for (i in 1:length(Fac)){
	Fac0[names(Fac[i])]<-Fac[names(Fac[i])]
	}
	asca<- ASCA.1f(X=X, Designa=Designa, Fac=Fac0)
}



if(Factors==2)
{
  Fac0<-c(1,2,2,2)
  names(Fac0)<-c("Model.a","Model.b","Model.ab","Model.res")
  if(Join[1]){ names(Fac0)[3]<-c("Model.bab")}
  asca0<- ASCA.2f(X=X, Designa=Designa, Designb=Designb,Fac=Fac0,Join=Join,Interaction=Interaction)
  Fac<-ARSyNcomponents(asca0,Variability=Variability,beta=beta)
  for (i in 1:length(Fac)){
	Fac0[names(Fac[i])]<-Fac[names(Fac[i])]
	}
	asca<- ASCA.2f(X=X, Designa=Designa, Designb=Designb,Fac=Fac0,Join=Join,Interaction=Interaction)
}

if(Factors==3)
{
  Fac0= c(0,2,2,2,2,2,2,2)
  names(Fac0)<-c("Model.a","Model.b","Model.c","Model.ab","Model.ac","Model.bc","Model.abc","Model.res")
  if(Join[1]){ names(Fac0)[4]<-c("Model.bab")}
  if(Join[2]){ names(Fac0)[5]<-c("Model.cac")}
  asca0<- ASCA.3f(X=X, Designa=Designa, Designb=Designb,Designc = Designc,Fac=Fac0,Join=Join,Interaction=Interaction)
  Fac<-ARSyNcomponents(asca0,Variability=Variability,beta=beta)
  for (i in 1:length(Fac)){
	Fac0[names(Fac[i])]<-Fac[names(Fac[i])]
	}
	asca<- ASCA.3f(X=X, Designa=Designa, Designb=Designb,Designc = Designc,Fac=Fac0,Join=Join,Interaction=Interaction)
}

Input<-asca$Input
asca$Input<-c(Input,Factors)
names(asca$Input)<-c(names(Input),"Factors")

output<-asca
output
}

#### GC CONTENT PLOT


## Data for GC plot 

GC.dat <- function (input, factor = NULL, norm = FALSE)  {
  
  # This plot shows the mean expression for each GC content bin, globally or for each biotype (if available).
  
  # datos: Count data matrix. Each column is a different biological sample.

  if (inherits(input,"eSet") == FALSE)    
    stop("Error. You must give an eSet object\n")

  if (!is.null(assayData(input)$exprs))
    datos <- assayData(input)$exprs
  else
    datos <- assayData(input)$counts
  
  ceros = which(rowSums(datos) == 0)
  
  if (length(ceros) > 0) {
    print(paste("Warning:", length(ceros), 
                "features with 0 counts in all samples are to be removed for this analysis."))
    datos = datos[-ceros,]
  }
      
  nsam <- NCOL(datos)
  if (nsam == 1) datos <- as.matrix(datos)
    
  
  # Per condition
  if (is.null(factor)) {  # per sample  
    print("GC content bias detection is to be computed for:")
    print(colnames(datos))
    
  } else {  # per condition
    mifactor = as.factor(pData(input)[,factor])
    niveles = levels(mifactor)
    print("GC content bias detection is to be computed for:")
    print(niveles)
    
    if (norm) {
      datos = sapply(niveles, 
                     function (k) {
                       rowMeans(as.matrix(datos[, mifactor == k]))
                     })    
    } else {
      datos = sapply(niveles, 
                     function (k) {
                       rowMeans(t(10^6*t(datos[, mifactor == k])/colSums(as.matrix(datos[, mifactor == k]))))                     
                     })    
    } 
    colnames(datos) = niveles
  }
  
  
  
  # GC content
  if (any(!is.na(featureData(input)$GC)) == FALSE)
    stop ("GC content was not provided.\nPlease run addData() function to add 
          this information\n")
  
  GC <- as.numeric(as.character(featureData(input)$GC))
  if (length(ceros) > 0) GC = GC[-ceros] 
  
  
  
  # Biotypes
  if (!is.null(featureData(input)$Biotype)) {  # read biotypes if they are provided
    infobio <- as.character(featureData(input)$Biotype)
    if (length(ceros) > 0) infobio = infobio[-ceros] 
    biotypes <- unique(infobio)
    names(biotypes) <- biotypes 
    # which genes belong to each biotype
    biog <- lapply(biotypes, function(x) { which(is.element(infobio, x)) })
    names(biog) = biotypes
    bionum <- c(NROW(datos), sapply(biog, length))
    names(bionum) <- c("global", names(biotypes))   
    
  } else { infobio = NULL; biotypes = NULL; bionum = NULL }  
  
  
  
  ## Calculations for plot
  
  GCexpr = vector("list", length = 1 + length(biotypes))
  names(GCexpr) = c("global", names(biotypes))
  
  numXbin = 200  
  
  for (i in 1:length(GCexpr))  {
    
    if (i == 1) {  # GLOBAL
      
      numdatos = length(GC)      
      numbins = floor(numdatos / numXbin)
      misbins = quantile(GC, probs = seq(0,1,1/numbins), na.rm = TRUE)
      
      if (length(misbins) != length(unique(misbins))) {
        repes = names(table(misbins))[which(table(misbins) > 1)]
        for (rr in repes) {
          cuantos = length(which(misbins == rr))
          cuales = which(misbins == rr)
          sumo = (misbins[cuales[1]+cuantos] - misbins[cuales[1]])/cuantos
          for (j in cuales[-1]) misbins[j] = misbins[j-1] + sumo            
        }
      }
      
      miclasi = cut(GC, breaks = misbins, labels = FALSE)
      misbins = sapply(1:numbins, function (i) mean(misbins[i:(i+1)]))
      miclasi = misbins[miclasi]
      
      GCexpr[[i]] = aggregate(datos, by = list("GCbin" = miclasi), mean, trim = 0.025)      
      
    } else {  # PER BIOTYPE
      
      datos2 = datos[biog[[i-1]],]
      GC2 = GC[biog[[i-1]]]
      
      if (bionum[i] >= numXbin*10) {  # more than numXbin*10 genes in the biotype
        
        numdatos = length(GC2)      
        numbins = floor(numdatos / numXbin)
        misbins = quantile(GC2, probs = seq(0,1,1/numbins), na.rm = TRUE)
        
        if (length(misbins) != length(unique(misbins))) {
          repes = names(table(misbins))[which(table(misbins) > 1)]
          for (rr in repes) {
            cuantos = length(which(misbins == rr))
            cuales = which(misbins == rr)
            sumo = (misbins[cuales[1]+cuantos] - misbins[cuales[1]])/cuantos
            for (j in cuales[-1]) misbins[j] = misbins[j-1] + sumo            
          }
        }
        
        miclasi = cut(GC2, breaks = misbins, labels = FALSE)
        misbins = sapply(1:numbins, function (i) mean(misbins[i:(i+1)]))
        miclasi = misbins[miclasi]        
        GCexpr[[i]] = aggregate(datos2, by = list("GCbin" = miclasi), mean, trim = 0.025, na.rm = TRUE)        
        
      } else {   # less than numXbin*10 genes in the biotype
        
        GCexpr[[i]] = cbind(GC2, datos2)      
        
      }      
      
    }
  }
  
  
  
  ## SPLINES REGRESSION MODEL   
  #library(splines)
  
  datos = GCexpr[[1]]
  GCcont = datos[,1]  
  knots =  c(rep(GCcont[1],3), seq(GCcont[1], GCcont[length(GCcont)-1], 
                                   length.out=round(length(GCcont)/10, 0)), 
             rep(GCcont[length(GCcont)], 4))
  bx = splineDesign (knots, GCcont, outer.ok = TRUE)
  
  
  mismodelos = vector("list", length = ncol(datos)-1)
  names(mismodelos) = colnames(datos)[-1]
  
  for (i in 2:ncol(datos)) {
    
    print(colnames(datos)[i])
    
    mismodelos[[i-1]] = lm(datos[,i] ~ bx)
        
    print(summary(mismodelos[[i-1]]))
    
  }
  

  
  
  ## Results
  
  list("data2plot" = GCexpr, "RegressionModels" = mismodelos)
}





#**************************************************************************#
#**************************************************************************#
#**************************************************************************#





## PLOT: Median expression for each length bin

GC.plot <- function (dat, samples = NULL, toplot = "global", toreport = FALSE,...)  {
  
  datos = dat[["data2plot"]]
  mismodelos = dat[["RegressionModels"]]
  

  if (is.null(samples)) samples <- 1:(ncol(datos[[1]])-1)
  if(length(samples) > 12) stop("Please select 12 samples or less to be plotted.")
  
  if (is.numeric(samples)) { samples = colnames(datos[[1]])[samples+1] }
  
  if (is.numeric(toplot)) {
    if (toplot == 1) { toplot = "global"} else { toplot = names(toplot)[toplot + 1] }
  }
    
  
  if ((toplot == "global") && (length(samples) <= 2)) {   ### DIAGNOSTIC PLOTS
    
    if((!toreport) && (length(samples) == 2)) par(mfrow = c(1,2))
    
    for (i in 1:length(samples)) {
      matplot(datos[[1]][,1], cbind(datos[[1]][,samples[i]], mismodelos[[samples[i]]]$fit), 
               type="pl", main=samples[i], pch=20, lty=1, lwd = 2, col = c(1,4),
               ylab = "Mean expression", xlab = "GC content bins", ylim = c(0,max(datos[[1]][,samples[i]])),...)   
      text(max(datos[[1]][,1]), 0.2*max(datos[[1]][,samples[i]]), col = 4, adj = 1,
           paste("R2 = ", 100*round(summary(mismodelos[[samples[i]]])$"r.squared",4), "%", sep = ""))
      laF = summary(mismodelos[[samples[i]]])$"fstatistic"
      text(max(datos[[1]][,1]), 0.1*max(datos[[1]][,samples[i]]), col = 4, adj = 1,
           paste("p-value:", signif(pf(laF[1], df1 = laF[2], df2 = laF[3], lower.tail = FALSE),2)))
    } 

    
  } else {   ### DESCRIPTIVE PLOTS
    
    matplot(datos[[toplot]][,1], datos[[toplot]][,samples], xlab = "GC content bins", ylab = "Mean expression", 
            type = "l", main = toupper(toplot), col = miscolores, lwd = 2, 
            ylim = range(datos[[toplot]][,-1]),lty = 1,...)
    
    legend("bottomright", samples, col = miscolores[1:length(samples)], lwd = 2, bty = "n")    
    
  }
  
  if((!toreport) && (length(samples) == 2)) layout(1)
  
   
}

ASCA.3f<-function(X = X,Designa = Designa,Designb = Designb,Designc = Designc,Fac = c(1,2,2,2,2,2,2,2),
Join =c(TRUE,TRUE),Interaction=c(TRUE,TRUE,TRUE,TRUE))

{
#--------------------------------------------------------------------------------------
#  Dimensions of the matrices: 
#  X (p x n) contains expression values of n genes (in columns) and p conditions (in rows)
#  Designa (p x I) contains 0's and 1's for the TIME-POINTS in the experiment
#  Designb (p x J) EXPERIMENTAL GROUP FACTOR 1  
#  Designc (p x K) ANOTHER FACTOR  

#  Join = c(TRUE,TRUE) if the analyses of the model b and ab and c and ac is studied jointly
#  Interaction = c(TRUE,TRUE,TRUE,TRUE) to consider interaction "ab", "ac", "bc" and "abc" in the separated model

n<-ncol(X)
p<-nrow(X)
I<-ncol(Designa)
J<-ncol(Designb)
K<-ncol(Designc)

Faca=Fac[1]# number components Model a (time)
Facb=Fac[2] # number components Model b  (second factor)
Facc=Fac[3] # number components Model c  (third factor)
Facab=Fac[4] # number components Model ab (interaction) 
Facac=Fac[5] # number components Model ac (interaction) 
Facbc=Fac[6] # number components Model bc (interaction) 
Facabc=Fac[7]  # number components Model abc (interaction) 
Facres=Fac[8]
#----------------------- Calculate Overall Mean -------------------------------------

offset<-apply(X,2,mean)
Xoff<-X-(cbind(matrix(1,nrow=p,ncol=1))%*%rbind(offset))

#-----------------------  PART I: Submodel a (TIME) ---------------------------------

Model.a<-ASCAfun1(Xoff,Designa,Faca)
Xres<-Xoff-Model.a$X

#-------------------------- PART II.1: Submodel b.ab-----------------------------------
if(!Join[1]) 
{
	Model.b<-ASCAfun1(Xoff,Designb,Facb)
	if (Interaction[1]) {
		Model.ab<-ASCAfun2(Xoff,Designa,Designb,Facab)
		}
}

if(Join[1])
{
	Model.bab<-ASCAfun12(Xoff,Designa,Designb,Facab)
}
#-------------------------- PART II.2: Submodel (c.ac) -------------------------------
if(!Join[2]) 
{
	Model.c<-ASCAfun1(Xoff,Designc,Facc)
	if (Interaction[2]) {
		Model.ac<-ASCAfun2(Xoff,Designa,Designc,Facac)
	}
}

if(Join[2])
{
	Model.cac<-ASCAfun12(Xoff,Designa,Designc,Facac)
}
#-------------------------- PART II.3: Submodel (bc) --------------------------------

	if (Interaction[3]) {
		Model.bc<-ASCAfun2(Xoff,Designb,Designc,Facbc)
	}
#-------------------------- PART II.4: Submodel (abc) --------------------------------

	if (Interaction[4]) {
		Model.abc<-ASCAfun.triple(Xoff,Designa,Designb,Designc,Facabc)
	}
# ------------------------Collecting models ------------------------------------------

models <- ls(pattern="Model")
output <- vector(mode="list")
Xres <- Xoff
for (i in 1: length(models)) {
	mymodel <- get(models[i], envir=environment())
	output <- c(output, list(mymodel))
	Xres <- Xres - mymodel$X
	rm(mymodel)
	gc()
}
names(output) <- models

#------------------------- PART III: Submodel res -----------------------------------

Model.res<-ASCAfun.res(Xres,Facres)

Model.res<-list(Model.res)
names(Model.res)<-c("Model.res")
output<-c(output,Model.res)



#------------------------- Add Input data to the Output ----------------------------

Input<-list(X, Designa, Designb, Designc, Fac, Join,Interaction,Xoff)
names(Input)<-c("X", "Designa", "Designb", "Designc", "Fac", "Join","Interaction","Xoff")
Input<-list(Input)
names(Input)<-"Input"
output<-c(output,Input)

output 

}

MD.plot <-
function (Ms = Ms, Ds = Ds, Mn = Mn, Dn = Dn, 
          xlim = range(Ms,na.rm=TRUE), ylim = c(0,quantile(Ds,0.8,na.rm=TRUE)),
          tit = "") {
  plot(Mn, Dn, pch = ".", main = tit, xlab = "M", ylab = "D",
       xlim = xlim, ylim = ylim)
  points(Ms, Ds, col = 2, pch = ".")

  legend("topright", c("noise", "signal"), col = 1:2, pch = 15,
         bg = "lightgrey")

}

PCA.GENES<-function(X)
{
#PCA.GENES is very useful to obtain principal components to a matrix that has more variables than individuals. 
#R can not apply princomp is such case and when there are a lot of variables eigen(t(X)%*%X) can not be computed.

#X is a matrix that has on columns the genes considered as variables in the PCA analysis.
#First we center the matrix by columns (Xoff) and then we obtain the eigenvalues and the eigenvectors of the matrix Xoff%*%t(Xoff) and we #use the equivalences between the loadings and scores to obtain the solution
#Llamo scores1 y loadings1 a lo que busco y scores2 y loadings2 a los scores y loadings de la traspuesta


n<-ncol(X)
p<-nrow(X)
offset<-apply(X,2,mean)
Xoff<-X-(cbind(matrix(1,p,1))%*%rbind(offset))

#eigen command sorts the eigenvalues in decreasing orden.

eigen<-eigen(Xoff%*%t(Xoff)/(p-1))
var<-cbind(eigen$values/sum(eigen$values),cumsum(eigen$values/sum(eigen$values)))

loadings2<-eigen$vectors
scores2<-t(Xoff)%*%loadings2

normas2<-sqrt(apply(scores2^2,2,sum))

scores1<-loadings2%*%diag(normas2)
loadings1<-scores2%*%diag(1/normas2)

output<-list(eigen,var,scores1,loadings1)
names(output)<-c("eigen","var.exp","scores","loadings")
output
}

ASCAfun.res<-function (X,Fac) {


 PCA<-PCA.GENES(X)
      sc<-PCA$scores[,1:Fac]
      ld<-PCA$loadings[,1:Fac]
      ssq<-PCA$var.exp

      if(Fac==1) {
      sc<-as.matrix(sc)
      ld<-as.matrix(ld)
      }      
      TPres<-sc%*%t(ld)
     
      if(Fac==0){
      sc=0
      ld=0
      TPres<-matrix(0,nrow(X),ncol(X))
      }
      
  Eres<-X-TPres

output<-list(sc,ld,ssq,X,TPres,Eres)
names(output)<-c("scores","loadings","var.exp","X","TP","E")
output


}

ARSyNcomponents<-function(asca=asca,Variability=0.75,beta=2)
{
# This program selects the number of components that explain more than the Variability%
# If Variability="average" the number of components will be those that explain more than 
# the average variation of the principal components
# For residuals model the number of components selected are beta*average-variability.

MODEL<-asca[-length(asca)]
M<-length(MODEL)-1
output<-NULL

if(Variability=="average")
{
	#library(Matrix)
	for (i in 1:M)
	{ 
	lim<-1/rankMatrix(MODEL[[i]]$X)[1]	
	t<-table(MODEL[[i]]$var.exp[,1]>lim)
	if(length(t)==1) {t[2]=0}
	t<-t[2]
	names(t)<-names(MODEL)[i]
	output<-c(output,t)
	}
}
if(Variability!="average")
{
	lim<-Variability
	for (i in 1:M)
	{ 
	t<-which(MODEL[[i]]$var.exp[,2]>lim)[1]
	names(t)<-names(MODEL)[i]
	output<-c(output,t)
	}	
}
### Residuals model
#library(Matrix)
i=M+1 
lim <- beta*1/rankMatrix(MODEL[[i]]$X)[1]	
	t<-table(MODEL[[i]]$var.exp[,1]>lim)
	if(length(t)==1) {t[2]=0}
	t<-t[2]
	names(t)<-names(MODEL)[i]
	output<-c(output,t)

	
output
}

DE.plot <- function (output, q = NULL, graphic = c("MD","expr","chrom","distr"),
                     pch = 20, cex = 0.5, col = 1, pch.sel = 1, cex.sel = 0.6, col.sel = 2,
                     log.scale = TRUE, chromosomes = NULL, join = FALSE,...) {
  
  mypar <- par(no.readonly = TRUE)
  
  if (class(output) != "Output") 
    stop("Error. Output argument must contain an object generated by noiseq or noiseqbio functions.\n")
  
  graphic <- match.arg(graphic)
  
  noiseqbio = "theta" %in% colnames(output@results[[1]])[1:4]
  
  
  ## MD plot
  if (graphic == "MD") {
    
    if (noiseqbio) {      
      
      M = output@results[[1]][,"log2FC"]
      D = abs(output@results[[1]][,1] - output@results[[1]][,2])+1
      names(M) = names(D) = rownames(output@results[[1]])
      
      plot(M, D, pch = pch, xlab = "M", ylab = "D", cex = cex, col = col, log = "y",...)
      
      if(!is.null(q)) {
        mySelection = rownames(degenes(output,q))
        points(M[mySelection], D[mySelection], col = col.sel, pch = pch.sel, cex = cex.sel)        
      }     
      
    } else {
      
      plot(output@results[[1]][,"M"], (1+output@results[[1]][,"D"]), pch = pch, 
           xlab = "M", ylab = "D", cex = cex, col = col, log = "y",...)
      
      if(!is.null(q)) {
        mySelection = rownames(degenes(output,q))
        points(output@results[[1]][mySelection,"M"], output@results[[1]][mySelection,"D"]+1, col = col.sel, pch = pch.sel, cex = cex.sel)
      }      
      
    }          
  } 
  
  
  ## Expression plot: Condition1 vs Condition2
  else if (graphic == "expr") { 
    
    data <- cbind(output@results[[1]][1],output@results[[1]][2])
    rownames(data) <- rownames(output@results[[1]])
    colnames(data) <- colnames(output@results[[1]][c(1,2)])
    
    if (log.scale) {
      
      k <- min(data, na.rm=TRUE)
      escala = logscaling(data, base = 2, k = k)
      
      plot(escala$data, pch = pch, cex = cex, col = col, yaxt = "n", xaxt = "n",...)
      axis(side = 1, at = escala$at, labels = escala$labels)
      axis(side = 2, at = escala$at, labels = escala$labels)
      
      if(!is.null(q)) {
        
        mySelection = rownames(degenes(output,q))  
        points(escala$data[mySelection,], col = col.sel, pch = pch.sel, cex = cex.sel)
      }
      
    } else {
      
      plot(data, pch = pch, cex = cex, col = col,...)
      
      if(!is.null(q)) {
        
        mySelection = rownames(degenes(output,q))  
        points(data[mySelection,], col = col.sel, pch = pch.sel, cex = cex.sel)
        
      }
    }
  }
  
  
  ## MANHATTAN PLOT
  else if (graphic == "chrom") {
    
    mydata <- data.frame(as.character(output@results[[1]][,"Chrom"]),
                         output@results[[1]][,c("GeneStart","GeneEnd")],output@results[[1]][,1:2])
    mydata <- na.omit(mydata)
    
    colnames(mydata) <- c("chr", "start", "end", colnames(mydata)[-c(1:3)])
    
    if (is.null(chromosomes)) {  # todos los cromosomas
      chromosomes <- unique(mydata$chr)
      chromosomes = sort(chromosomes)
    }
    
    print("REMEMBER. You are plotting these chromosomes and in this order:")
    print(chromosomes)
    
    # logarithmic scale
    if (log.scale) {
      if(min(mydata[,-c(1:3)], na.rm = TRUE) < 1) {
        kk <- -min(mydata[,-c(1:3)], na.rm = TRUE)+1
      } else { kk <- 0 }
      mydata[,-c(1:3)] <- log2(mydata[,-c(1:3)]+kk)
    }  
    
    # Selecting chromosomes and ordering positions
    ordenat = NULL
    for (cromo in chromosomes) {
      myselec = mydata[mydata[,"chr"] == cromo,]
      myselec = myselec[order(myselec[,"start"]),]
      ordenat = rbind(ordenat, myselec)
    }  
    
    sel.ord <- NULL
    
    if (!is.null(q)) {
      # up-regulated in first condition
      cond1 <- rownames(degenes(output,q,M="up"))
      # up-regulated in second condition
      cond2 <- rownames(degenes(output,q,M="down"))
      
      cond1 <- (rownames(ordenat) %in% cond1)*1
      cond2 <- (rownames(ordenat) %in% cond2)*1
      
      sel.ord  <- cbind(cond1, cond2)
      rownames(sel.ord) <- rownames(ordenat)
    }
    
    chr.long <- aggregate(ordenat[,"end"],
                          by = list(as.character(ordenat[,"chr"])), max)
    chr.long <- chr.long[match(chromosomes, chr.long[,1]),]
    
    
    if (join) { # si todos los cromosomas van en el mismo plot
      
      total.long <- sum(as.numeric(chr.long$x))
      
      chr.start <- cumsum(c(1,chr.long$x[-length(chr.long$x)]))
      names(chr.start) <- chr.long[,1]
      
      plot(c(1,total.long), c(-max(ordenat[,5]), max(ordenat[,4])),
           type = "n", xlab = "", ylab = "Expression data", xaxt = "n")
      
      axis(side = 1, at = chr.start, labels = chr.long[,1], font = 2)
      
      abline(h = 0, lty = 2, lwd = 0.5)    
      
      for (ch in chromosomes) {
        
        dat.chr <- ordenat[which(ordenat[,"chr"] == ch),]
        dat.chr[,c("start","end")] <- dat.chr[,c("start","end")] +
          chr.start[ch] - 1
        
        rect(xleft = dat.chr[,"start"], ybottom = 0,
             xright = dat.chr[,"end"], ytop = dat.chr[,4],
             col = "grey", border = NA)
        
        rect(xleft = dat.chr[,"start"], ybottom = -dat.chr[,5],
             xright = dat.chr[,"end"], ytop = 0,
             col = "grey", border = NA)
        
        if (!is.null(q)) {
          
          aux <- which(rownames(sel.ord) %in% rownames(dat.chr))          
          
          sel.chr1 <- dat.chr[,4]*sel.ord[aux,1]
          sel.chr2 <- -dat.chr[,5]*sel.ord[aux,2]
          
          rect(xleft = dat.chr[,"start"], ybottom = 0,
               xright = dat.chr[,"end"], ytop = sel.chr1,
               col = 2, border = NA)
          
          rect(xleft = dat.chr[,"start"], ybottom = sel.chr2,
               xright = dat.chr[,"end"], ytop = 0,
               col = 3, border = NA)        
        }
        
        segments(x0 = dat.chr[,"start"], y0 = 0, x1 = dat.chr[,"end"],
                 y1 = 0, col = 4, lwd = 0.5) # annotated genes
        
      }        
      
      text(c(1,1), 0.9*c(max(ordenat[,4]), -max(ordenat[,5])),
           colnames(mydata)[4:5], font = 3, adj = 0, col = 2:3)
      
      
      
      
      
      # a plot for each chromosome
    } else {    
      
      num.chr <- length(chromosomes)
      
      k <- 20
      
      long.prop <- round((chr.long$x / min(chr.long$x))*k, 0)
      
      while (max(long.prop)*num.chr > 500 | max(long.prop) > 50) {
        
        k <- k-1
        
        long.prop <- round((chr.long$x / min(chr.long$x))*k, 0)
        
      }
      
      forlayout <- matrix(0, num.chr, max(long.prop))
      
      #print(dim(forlayout))
      
      for (i in 1:num.chr) {
        
        forlayout[i, 1:long.prop[i]] <- i
        
      }
      
      layout(forlayout)
      
      if (num.chr > 1)	par(mar=c(2,4.1,0.1,0.1))
      
      miylim <- c(-max(ordenat[,5], na.rm=TRUE), max(ordenat[,4],na.rm=TRUE))
      
      for (i in 1:num.chr) {
        
        apintar <- ordenat[which(ordenat[,"chr"] ==  chromosomes[i]), 2:5]
        
        plot(c(1,chr.long[i,2]), miylim, type = "n",
             xlab = "", ylab = chromosomes[i], xaxt = "n",
             font.lab = 2, cex.lab = 1.3)
        
        rect(xleft = apintar[,"start"], ybottom = 0, xright = apintar[,"end"],
             ytop = apintar[,3], col = "grey", border = NA)
        
        rect(xleft = apintar[,"start"], ybottom = -apintar[,4],
             xright = apintar[,"end"], ytop = 0, col = "grey", border = NA)
        
        if (!is.null(q)) {
          
          aux <- which(rownames(sel.ord) %in% rownames(apintar))          
          asel <- sel.ord[aux,]*apintar[,3:4]
          
          rect(xleft = apintar[,"start"], ybottom = 0, xright = apintar[,"end"],
               ytop = asel[,1], col = 2, border = NA)
          
          rect(xleft = apintar[,"start"], ybottom = -asel[,2],
               xright = apintar[,"end"], ytop = 0, col = 3, border = NA)        
          
        }      
        
        abline(h = 0, lty = 2, lwd = 0.5)
        
        segments(x0 = apintar[,"start"], y0 = 0, x1 = apintar[,"end"],
                 y1 = 0, col = 4, lwd = 0.5) # annotated genes
        
        etiq <- mypretty(c(1,chr.long[i,2]), n = 10)
        
        axis(side = 1, at = etiq, labels = etiq)
        
        text(c(1,1), 0.9*miylim, colnames(mydata)[5:4],
             font = 3, adj = 0, col = 3:2)
      }
    }
  }
  
  
  
  ## DEG distribution across biotypes/chromosomes
  else if (graphic == "distr") {    
    
    if(!is.null(q)) {     # Computing DEG
      mySelection = rownames(degenes(output,q))
      detect = rownames(output@results[[1]]) %in% mySelection
    } else {
      stop("You must specify a valid value for q\n")
    }
    
    
    if ("Chrom" %in% colnames(output@results[[1]])) {
      
      numplot = 1
      
      infobio = output@results[[1]][,"Chrom"]
      
      genome <- 100*table(infobio)/sum(table(infobio))
      
      ordre <- order(genome, decreasing = TRUE)      
      
      perdet1 <- genome*table(infobio, detect)[names(genome),2] / table(infobio)[names(genome)]
      
      perdet2 <- 100*table(infobio, detect)[names(genome),2] / sum(table(infobio, detect)[,2])
      
      ceros <- rep(0, length(genome))
      
      biotable1 <- as.matrix(rbind(genome[ordre], perdet1[ordre], perdet2[ordre], ceros))
      rownames(biotable1) <- c("genome", "degVSgenome", "deg", "ceros")
      
      if (!is.null(chromosomes)) biotable1 = biotable1[,chromosomes]
      
      ymaxL1 <- ceiling(max(biotable1, na.rm = TRUE))      
      
    } else { numplot = 0 }
    
    
    
    if ("Biotype" %in% colnames(output@results[[1]])) {
      
      numplot = c(numplot, 1)
      
      infobio = output@results[[1]][,"Biotype"]
      
      genome <- 100*table(infobio)/sum(table(infobio))
      
      ordre <- order(genome, decreasing = TRUE)      
      
      perdet1 <- genome*table(infobio, detect)[names(genome),2] / table(infobio)[names(genome)]
      
      perdet2 <- 100*table(infobio, detect)[names(genome),2] / sum(table(infobio, detect)[,2])
      
      ceros <- rep(0, length(genome))
      
      biotable2 <- as.matrix(rbind(genome[ordre], perdet1[ordre], perdet2[ordre], ceros))
      rownames(biotable2) <- c("genome", "degVSgenome", "deg", "ceros")
      
      higher2 = which(biotable2[1,] > 2)
      lower2 = which(biotable2[1,] <= 2)
      
      if (length(higher2) > 0) {        
        ymaxL2 <- ceiling(max(biotable2[,higher2], na.rm = TRUE)) 
        if (length(lower2) > 0) {
          ymaxR2 <- ceiling(max(biotable2[,lower2], na.rm = TRUE))
          biotable2[,lower2] <- biotable2[,lower2]*ymaxL2/ymaxR2
        } else { ymaxR2 = ymaxL2 }
        
      } else { 
        ymaxR2 <- ceiling(max(biotable2[,lower2], na.rm = TRUE))
        ymaxL2 = ymaxR2               
      }     
      
      
      
    } else { numplot = c(numplot, 0) }
    
    
    
    
    # Plot
    if (sum(numplot) == 0) stop("Biotype or chromosome information is needed for this plot\n")
    
    if (sum(numplot) == 1) {  # 1 Plot 
      
      par(mar = c(10, 4, 2, 2))
      
      if (numplot[1] == 1) {
        
        barplot(biotable1[c(1,3),], main = "DEG distribution across chromosomes",
                xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
                beside = TRUE, col = c("grey", 2), las = 2,
                ylim = c(0, ymaxL1), border = c("grey", 2))
        
        barplot(biotable1[c(2,4),], main = "DEG distribution across chromosomes",
                xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
                beside = TRUE, col = c(2, 1), las = 2, density = 30,
                ylim = c(0, ymaxL1), border = 2, add = TRUE)
        
        legend(x = "topright", bty = "n", horiz = FALSE,
               fill = c("grey", 2, 2), density = c(NA,30,NA),
               border = c("grey", 2, 2),
               legend = c("%Chrom in genome", "%DEG in Chrom", "%Chrom in DEG"))
        
      }
      
      
      if (numplot[2] == 1) {
        
        barplot(biotable2[c(1,3),], main = "DEG distribution across biotypes",
                xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
                beside = TRUE, col = c("grey", 4), las = 2,
                ylim = c(0, ymaxL2), border = c("grey", 4))
        
        barplot(biotable2[c(2,4),], main = "DEG distribution across biotypes",
                xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
                beside = TRUE, col = c(4, 1), las = 2, density = 30,
                ylim = c(0, ymaxL2), border = 4, add = TRUE)
        
        axis(side=4, at = pretty(c(0,ymaxL2), n = 5), 
             labels = round(pretty(c(0,ymaxL2), n = 5)*ymaxR2/ymaxL2, 1))
        
        if (ymaxR2 != ymaxL2) {
          abline(v = 3*length(higher2) + 0.5, col = 3, lwd = 2, lty = 2)
        }    
        
        legend(x = "topright", bty = "n", horiz = FALSE, 
               fill = c("grey", 4, 4), density = c(NA,30,NA),
               border = c("grey", 4, 4),
               legend = c("%Biotype in genome", "%DEG in Biotype", "%Biotype in DEG"))        
      } 
    }
    
    
    if (sum(numplot) == 2) {  # 2 Plots 
      
      par(mar = c(10, 4, 2, 2), mfrow = c(1,2))
      
      # Chromosomes
      barplot(biotable1[c(1,3),], main = "DEG distribution across chromosomes",
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c("grey", 2), las = 2,
              ylim = c(0, ymaxL1), border = c("grey", 2))
      
      barplot(biotable1[c(2,4),], main = "DEG distribution across chromosomes",
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c(2, 1), las = 2, density = 30,
              ylim = c(0, ymaxL1), border = 2, add = TRUE)
      
      legend(x = "topright", bty = "n", horiz = FALSE,
             fill = c("grey", 2, 2), density = c(NA,30,NA), border = c("grey", 2, 2),
             legend = c("%Chrom in genome", "%DEG in Chrom", "%Chrom in DEG"))
      
      
      # Biotypes
      barplot(biotable2[c(1,3),], main = "DEG distribution across biotypes",
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c("grey", 4), las = 2,
              ylim = c(0, ymaxL2), border = c("grey", 4))
      
      barplot(biotable2[c(2,4),], main = "DEG distribution across biotypes",
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c(4, 1), las = 2, density = 30,
              ylim = c(0, ymaxL2), border = 4, add = TRUE)
      
      axis(side=4, at = pretty(c(0,ymaxL2), n = 5), 
           labels = round(pretty(c(0,ymaxL2), n = 5)*ymaxR2/ymaxL2, 1))
      
      if (ymaxR2 != ymaxL2) {
        abline(v = 3*length(higher2) + 0.5, col = 3, lwd = 2, lty = 2)
      }    
      
      legend(x = "topright", bty = "n", horiz = FALSE, 
             fill = c("grey", 4, 4), density = c(NA,30,NA),
             border = c("grey", 4, 4),
             legend = c("%Biotype in genome", "%DEG in Biotype", "%Biotype in DEG"))        
    }
  }

  par(mypar)

}




## Auxiliar function
mypretty <- function(x, n = 5) {
  
  mywidth <- diff(x)
  
  mybin <- ceiling(mywidth/(n-1))
  
  mylabels0 <- x[1] + mybin*(0:(n-1))
  
  ndig <- nchar(as.character(mylabels0))
  #print(ndig)
  
  mylabels <- mylabels0
  
  k <- 0
  
  for (i in 2:(n-1)) {
    
    mylabels[i] <- (round(mylabels0[i]/10^(ndig[i]-1),k))*10^(ndig[i]-1)
    
    while (mylabels[i] == mylabels[i-1]) {
      
      k <- k+1
      
      mylabels[i] <- (round(mylabels0[i]/10^(ndig[i]-1),k))*10^(ndig[i]-1)
      
    }
    
  }
  
  mylabels
}

##Counts for detected genes Plot according to BIOTYPES (boxplots)

countsbio.dat <- function (input, biotypes = NULL, factor = NULL, norm = FALSE)  {

  # input: input object
  # biotypes: List containing groups of biotypes to be studied
  # factor: If not NULL, it should contain the conditions to be studied and
  #         calculation will be done based on the mean of replicates of each condition.


  if (inherits(input,"eSet") == FALSE)
    stop("Error. You must give an eSet object\n")
  

  if (!is.null(assayData(input)$exprs))
    datos <- assayData(input)$exprs
  else
    datos <- assayData(input)$counts
  
  
  depth = round(colSums(datos)/10^6,1); names(depth) = colnames(datos)
  
  ceros = which(rowSums(datos) == 0)
  hayceros = (length(ceros) > 0)
    
  if (hayceros) {
    print(paste("Warning:", length(ceros), 
                "features with 0 counts in all samples are to be removed for this analysis."))
    datos0 = datos[-ceros,]
  } else { datos0 = datos}
    
  
  nsam <- NCOL(datos)
  
  if (nsam == 1) {
    datos <- as.matrix(datos)
    datos0 <- as.matrix(datos0)
  } 

  
  # Per condition
  if (is.null(factor)) {  # per sample  
    print("Count distributions are to be computed for:")
    print(colnames(datos))
    
  } else {  # per condition
    mifactor = as.factor(pData(input)[,factor])
    niveles = levels(mifactor)
    print("Counts per million distributions are to be computed for:")
    print(niveles)
    
    if (norm) {
      datos = sapply(niveles, 
                     function (k) { 
                       rowMeans(as.matrix(datos[, mifactor == k]))
                     })
      datos0 = sapply(niveles, 
                      function (k) { 
                        rowMeans(as.matrix(datos0[, mifactor == k]))
                      })
      
    } else {
      datos = sapply(niveles, 
                     function (k) { 
                       10^6 * rowMeans(t(t(datos[, mifactor == k])/colSums(as.matrix(datos[, mifactor == k]))))
                     })
      datos0 = sapply(niveles, 
                      function (k) { 
                        10^6 * rowMeans(t(t(datos0[, mifactor == k])/colSums(as.matrix(datos0[, mifactor == k]))))
                      })      
    }
    colnames(datos) = colnames(datos0) = niveles
    depth = sapply(niveles, function (k) paste(range(depth[mifactor == k]), collapse = "-"))
  }
  
  
  
  # Biotypes
  if (!is.null(featureData(input)$Biotype)) {  # read biotypes if they are provided
    if (hayceros) { 
      infobio0 <- as.character(featureData(input)$Biotype)[-ceros] 
    } else { infobio0 =  as.character(featureData(input)$Biotype) }
    infobio <- as.character(featureData(input)$Biotype)
  } else { infobio0 = NULL; infobio = NULL }  
  
  if (!is.null(infobio)) {    
    if(is.null(biotypes)) {
      biotypes <- unique(infobio)
      names(biotypes) <- biotypes    
    }   
    # which genes belong to each biotype
    biog <- lapply(biotypes, function(x) { which(is.element(infobio0, x)) })
    names(biog) = biotypes
    bionum <- c(NROW(datos0), sapply(biog, length))
    names(bionum) <- c("global", names(biotypes)) 
    bio0 = which(bionum == 0)
    if (length(bio0) > 0) bionum = bionum[-bio0]
        
  } else { biotypes = NULL; bionum = NULL }   
  
  
  # Create the summary matrix information
  if (is.null(bionum)) {
    resumen = vector("list", length = 1)
    names(resumen) = "global"
  } else {
    resumen = vector("list", length = length(bionum))
    names(resumen) = names(bionum)
  }
  
  cuentas = c(0,1,2,5,10)
  
  if (is.null(factor)) {
    if (norm) {
      datosCPM = datos
    } else {
      datosCPM = 10^6 * t(t(datos)/colSums(as.matrix(datos)))
    }
    
  } else { datosCPM = datos }
      
  for (i in 1:length(resumen)) { 
    
    if (i == 1) { 
      datosR = datosCPM       
    } else {       
      if(!is.null(infobio)) {
        datosR = datosCPM[which(infobio == names(resumen)[i]),,drop = FALSE]
        # if (class(datosR) !=  "matrix") { datosR = t(as.matrix(datosR)) } 
      }
    }
    
    
    nfeatures = nrow(datosR)
    datosR = datosR[which(rowSums(datosR) > 0),,drop = FALSE] 
    # if (class(datosR) !=  "matrix") { datosR = t(as.matrix(datosR)) }
    
    myglobal = NULL
    mypersample = NULL
    
    for (kk in 1:length(cuentas)) {
      mypersample = rbind(mypersample, apply(datosR, 2, function (x) { length(which(x > cuentas[kk])) }))
      myglobal = c(myglobal, sum(apply(datosR, 1, function (x) { max(x) > cuentas[kk] })))
    }    
    
    mypersample = round(100*mypersample/nfeatures, 1)
    mypersample = rbind(mypersample, depth)
    rownames(mypersample) = 1:nrow(mypersample)
    myglobal = c(round(100*myglobal/nfeatures, 1), nfeatures)
        
    resumen[[i]] = data.frame(c(paste("CPM >", cuentas), "depth"), mypersample, "total" = myglobal)
    colnames(resumen[[i]])[1] = names(resumen)[i]
    colnames(resumen[[i]])[2:(ncol(resumen[[i]])-1)] = colnames(datosR)
                              
  }  
  

  ## results
  cosas <- list("result" = datos0, 
                 "bionum" = bionum,
                 "biotypes" = infobio0,
                 "summary" = resumen)    
  
  cosas
}






#***************************************************************************#
#***************************************************************************#


## PLOT: Mean length for detected genes Plot according to BIOTYPES

countsbio.plot <- function (dat, samples = c(1,2), toplot = "global", 
                            plottype = c("barplot", "boxplot"), toreport = FALSE,...) {

  # dat: Data coming from countsbio.dat function
  # samples: Samples to be plotted. If NULL, all samples are plotted.
  # toplot: Name of biotype (including "global") to be plotted.

  mypar = par(no.readonly = TRUE)
  plottype = match.arg(plottype)
  
  ## Preparing data
  if (is.null(samples)) { 
    if (NCOL(dat$result) == 1) {
      samples = 1
    } else {
      samples <- 1:NCOL(dat$result) 
    }    
  }
  if(is.numeric(toplot)) toplot = names(dat$summary)[toplot]
  if (is.numeric(samples) && !is.null(colnames(dat$result))) samples = colnames(dat$result)[samples]
  
  if (plottype == "barplot") {
    
    if ((exists("ylab") && !is.character(ylab)) || !exists("ylab")) ylab = ""
    
    datos = dat$summary[[toplot]]
    mytotal = as.numeric(datos[,"total"])
    datos = as.matrix(datos[,samples])
    rownames(datos) = as.character(dat$summary[[toplot]][,1])
    
    
    par(mar = c(6,4,4,2))
    
    barplot(as.numeric(datos[1,]), col = miscolores[1], las = 2, main = "", ylab = "", density = 70,
            ylim = c(0,100), cex.axis = 0.8, names.arg = "",...)  
    for (i in 2:(length(mytotal)-2)) {
      barplot(as.numeric(datos[i,]), col = miscolores[i], las = 2, main = "", ylab = "", add = TRUE, 
              density = 70, ylim = c(0,100), cex.axis = 0.8, names.arg = "",...)    
    }
    bp = barplot(as.numeric(datos[(length(mytotal)-1),]), col = miscolores[(length(mytotal)-1)], las = 2, 
                 main = paste(toupper(toplot), " (", mytotal[length(mytotal)], ")", sep = ""), 
                 ylab = "Sensitivity (%)", add = TRUE, names.arg = colnames(datos), cex.axis = 0.8,
                 density = 70, ylim = c(0,100), cex.names = 0.8,...) 
    for (j in 1:(length(mytotal)-1)) abline(h = mytotal[j], col = miscolores[j], lwd = 2)
    if (length(samples) <= 10) {
      mtext(side = 3, text = datos["depth",], adj = 0.5, at = bp, cex = 0.8)
    } else {
      mtext(side = 3, text = datos["depth",], at = bp, cex = 0.7, las = 2)
    }    
    legend("top", rownames(datos)[-length(mytotal)], fill = miscolores, density = 70, bty = "n", ncol = 3)
      
    par(mar = c(5, 4, 4, 4) + 0.1) 
  }
  
  
  
  
  if (plottype == "boxplot") {
    
    conteos <- as.matrix(dat$result[,samples])
    if (is.numeric(samples)) colnames(conteos) = colnames(dat$result)[samples]
    else colnames(conteos) = samples
    num <- dat$bionum[toplot]
    if (is.null(num)) {
      if (toplot == "global") {
        num = nrow(conteos)
      } else {
        num = 0
      }
    } 
    infobio = dat$biotypes
    
    if (num == 0 && toplot != "global") stop("Error: No data available. Please, change toplot parameter.")
    
    #if (!exists("ylim")) ylim = range(na.omit(log2(1+conteos)))
    if ((exists("ylab") && !is.character(ylab)) || !exists("ylab")) ylab = "Expression values"
    
    ## Plots  
    
    if (length(samples) == 1) {  # only 1 sample is to be plotted (per biotypes if available)
      
      escala = logscaling(conteos, base = 2)
      
      if (is.null(infobio)) {
        boxplot(escala$data, col = miscolores[1], ylab = ylab, #ylim = ylim,
                main = "", yaxt = "n", ...)
      } else {
        par(mar = c(10, 4, 4, 2))  
        boxplot(escala$data ~ infobio, col = miscolores, ylab = ylab, #ylim = ylim,
                main = colnames(conteos), las = 2, cex.axis = 0.8, cex.lab = 0.9, yaxt = "n", ...)
        cuantos = dat$bionum[-1]
        cuantos = cuantos[sort(names(cuantos))]
        mtext(cuantos, 3, at = 1:length(cuantos), cex = 0.6, las = 2)
      }    
      
      
    } else {   # more than 1 sample is to be plotted
      if (toplot != "global") conteos = conteos[which(infobio == toplot),]          
      
      escala = logscaling(conteos, base = 2)
      main <- paste(toupper(toplot), " (", num, ")", sep = "")
      
      par(mar = c(6, 4, 2, 2))  
      boxplot(escala$data, col = miscolores, ylab = ylab, #ylim = ylim,
              main = main, las = 2, cex.lab = 0.9, cex.axis = 0.8, yaxt = "n", ...)       
    }
    
    axis(side = 2, at = escala$at, labels = escala$labels)
        
  }
  
  if (!toreport) par(mypar)  
  
}

degenes <-
function (object, q = 0.95, M = NULL) {
  # object = noiseq output object
  # M = "up" (up-regulated in condition 1), "down" (down-regulated in condition 1), NULL (all differentially expressed genes)
  # q = probability threshold (between 0 and 1)

  if (class(object) != "Output")
    stop("You must give the object returned by the noiseq function\n")

  x <- object@results[[1]]
  
  noiseqbio = "theta" %in% colnames(x)[1:4]
  
  if (noiseqbio) {
    y <- na.omit(x[c("theta","prob")])
    colnames(y)[1] = "M"
  } else {
    y <- na.omit(x[c("M","D","prob")])
  }
  

  if (is.null(M)) {

    losdeg <- y[y[,"prob"] > q,]
    print(paste(dim(losdeg)[1], "differentially expressed features"))

  } else if (M == "up") {

    estos <- y[y[,"M"] > 0,]
    losdeg <- estos[estos[,"prob"] > q,]
    print(paste(dim(losdeg)[1], "differentially expressed features (up in first condition)"))

  } else if (M == "down") {

    estos <- y[y[,"M"] < 0,]
    losdeg <- estos[estos[,"prob"] > q,]
    print(paste(dim(losdeg)[1], "differentially expressed features (down in first condition)"))

  } else {

    stop("ERROR! Value for parameter M is not valid. Please, choose among NULL, 'up' or 'down'")

  }

  # Restore the object with the same "results" structure
  losdeg = x[rownames(losdeg),]

  losdeg[order(losdeg[,"prob"], decreasing = TRUE),]

}

## Computing PCA 

PCA.dat <- function (input, norm = FALSE, logtransf = FALSE)  {

  # input: input object
  # norm: TRUE if data are already normalized, FALSE if not.
  # logtransf: TRUE if data are already log-transformed, FALSE if not.

  if (inherits(input,"eSet") == FALSE)
    stop("Error. You must give an eSet object\n")
  

  if (!is.null(assayData(input)$exprs))
    datos <- assayData(input)$exprs
  else
    datos <- assayData(input)$counts
  
  myfactors = pData(input)
  
  
  if (!norm) datos = rpkm(datos)
  
  if (!logtransf) datos = log2(datos+1)
  
  resultat = PCA.GENES(t(datos))
  

  ## results
  resultat <- list("result" = resultat,
                   "factors" = myfactors,                   
                   "norm" = norm,
                   "logtransf" = logtransf)    
  
  resultat
}






#***************************************************************************#
#***************************************************************************#


## PLOT: PCA plot (global or per biotype)

PCA.plot <- function (dat, samples = c(1,2), plottype = "scores", factor = NULL) {

  # dat: Data coming from PCA.dat function
  # samples: Principal components to be plotted. If NULL, PC1 and PC2 (default) will be plotted.
  # toplot: Name of biotype (including "global") to be plotted.
  # plottype: One of "scores" or "loadings"
  # factor: Name of the factor to be used to color the PCA score plot. If NULL, the first one is chosen.

  
  ## Preparing data
  if (is.null(samples)) samples = 1:2     

  
  if (plottype == "loadings") { 
    
    data2plot = dat$result
        
    rango = diff(range(data2plot$loadings[,samples]))
    
    plot(data2plot$loadings[,samples], col = 1, pch = ".",
         xlab = paste("PC", samples[1], round(data2plot$var.exp[samples[1],1]*100,0), "%"),
         ylab = paste("PC", samples[2], round(data2plot$var.exp[samples[2],1]*100,0), "%"),
         main = "Loadings",
         xlim = range(data2plot$loadings[,samples]) + 0.02*rango*c(-1,1),
         ylim = range(data2plot$loadings[,samples]) + 0.02*rango*c(-1,1))    
  }
  
    
  else if (plottype == "scores") {
    
    data2plot = dat$result
    
    if (is.null(factor)) factor = 1
    myfactor = as.character(dat$factors[,factor])
        
    condis = unique(myfactor)
    
    mypch = c(17:15, 18, 8, 1, 2)  # 7
    mycolors = colors()[c(554,89,111,512,17,586,132,428,601,568,86,390)] # 12
    parapintar = data.frame("col" = rep(mycolors, 7), "pch" = rep(mypch, 12), stringsAsFactors = FALSE)         
    parapintar = parapintar[1:length(condis),]
    rownames(parapintar) = condis
    
    pch = parapintar[myfactor,"pch"]
    col = parapintar[myfactor,"col"]       
        
    rango = diff(range(data2plot$scores[,samples]))
    
    plot(data2plot$scores[,samples], col = "white", 
         xlab = paste("PC", samples[1], round(data2plot$var.exp[samples[1],1]*100,0), "%"),
         ylab = paste("PC", samples[2], round(data2plot$var.exp[samples[2],1]*100,0), "%"),
         main = "Scores",
         xlim = range(data2plot$scores[,samples]) + 0.02*rango*c(-1,1),
         ylim = range(data2plot$scores[,samples]) + 0.02*rango*c(-1,1))    
    
    points(data2plot$scores[,samples[1]], data2plot$scores[,samples[2]],
           pch = pch, col = col, cex = 2.3) 
    
    legend("topleft", condis, pch = parapintar[,"pch"], col = parapintar[,"col"], bty = "n")        
  } 
  
  
}

##########################################################################################



##***********************************************************##

## Coefficient of Variation

CV = function(data) { 100 * sd(data, na.rm = TRUE) / mean(data, na.rm = TRUE) }


##***********************************************************##



## Filtering out genes with low counts

filtered.data = function(dataset, factor, norm = TRUE, depth = NULL, method = 1, cv.cutoff = 100, cpm = 1, p.adj = "fdr") {
  dataset0 = dataset[rowSums(dataset) > 0,]
  dataset = dataset0
  
  if ((method == 3) && (norm)) {
    if (is.null(depth)) {
      stop("ERROR: Sequencing depth for each column in dataset must be provided.\n")
    }      
    dataset = t(t(dataset0) / (colSums(dataset0)/depth))  # estimate counts from normalized data
  }         
  
  if ((method < 3) && (!norm)) {
    dataset = 10^6 * t(t(dataset0) / colSums(dataset0))
  }
    
  grupos = unique(factor)
  cumple = NULL
  
  cat("Filtering out low count features...\n")
  
  for (gg in grupos) {    
    datos = as.matrix(dataset[, factor == gg])
    
    if (method == 1) {
      
      if (ncol(datos) == 1) {
        cumplecond = (datos > cpm)
      } else {
        cumplecond = (apply(datos, 1, CV) < cv.cutoff)*(rowMeans(datos) > cpm)
        cumplecond[which(is.na(cumplecond) == TRUE)] = 0 
      }
      
      cumple = cbind(cumple, cumplecond)
    } 
    
    if (method == 2) {
      if (ncol(datos) == 1) stop("ERROR: At least 2 replicates per condition are required to apply this method.")
      mytest = apply(datos, 1, 
                     function (x) { 
                       suppressWarnings(wilcox.test(x, alternative = "greater", conf.int=FALSE, mu = 0))$"p.value" })
      mytest = p.adjust(mytest, method = p.adj)
      cumple = cbind(cumple, 1*(mytest < 0.05))  
    }    
    
    if (method == 3) {           
      p0 = cpm / 10^6                
      mytest = apply(datos, 1, 
                         function (x) suppressWarnings(prop.test(sum(x), n=sum(datos), p = p0,
                                                alternative = "greater"))$"p.value")       
      mytest = p.adjust(mytest, method = p.adj)
      cumple = cbind(cumple, 1*(mytest < 0.05))           
    }
  }
  
  cumple = which(rowSums(as.matrix(cumple)) >= 1) 
  
  cat(paste(length(cumple), "features are to be kept for differential expression analysis with filtering method", method)); cat("\n")
  
  dataset0[cumple,]
  
}




##***********************************************************##

noiseq <-
function (input, k = 0.5, norm = c("rpkm","uqua","tmm","n"),
          replicates = c("technical","biological","no"),
          factor=NULL, conditions = NULL, pnr = 0.2, nss = 5, v = 0.02, lc = 0)

# input: Object containing gene counts and as many columns as samples.

# k:      When counts = 0, 0 will be changed to k. By default, k = 0.5.
  
# norm:   Normalization method. It can be one of "rpkm" (default), "uqua"
#         (upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).

# factor: String with the factor to choose which conditions you are going
#         to compare.

# conditions: String with the conditions to compare in case one factor contains more
#         than 2 conditions.

# pnr:    Percentage of total reads (seq.depth) for each simulated sample.
#         Only needed when no replicates available. By default, pnr = 0.2.

# nss:    Number of simulated samples (>= 2). By default, nss = 5.
#         If nss = 0, real samples are used to compute noise.

# v:      Variability in sample total reads used to simulate samples.
#         By default, v = 0.02. Sample total reads is computed as a
#         random value from a uniform distribution in the interval
#         [(pnr-v)*sum(counts), (pnr+v)*sum(counts)]

# lc:     Length correction in done by dividing expression by length^lc.
#         By default, lc = 0. 

{
  
  if (inherits(input,"eSet") == FALSE)
    stop("Error. You must give an object generated by the readData function\n")

  if (is.null(factor))
    stop("Error. You must specify the factor to know which conditions you wish to compare.
         Please, give the argument 'factor'.\n")

  replicates <- match.arg(replicates)
  if (replicates == "biological") print("WARNING: Your experiment has biological replicates. You should consider using NOISeqBIO instead of NOISeq.")
  norm <- match.arg(norm)
  
  # (M,D) for signal and noise
  print("Computing (M,D) values...")
  miMD <- allMD(input, factor, k = k, replicates = replicates, 
                norm = norm, conditions = conditions,
                pnr = pnr, nss = nss, v = v, lc = lc)
 
    #------------------------------------------------------------------#
    
  
  
    ## Probability of differential expression

  print("Computing probability of differential expression...")
  prob.concounts <- probdeg(miMD$Ms, miMD$Ds, miMD$Mn, miMD$Dn)$prob

  if (!is.null(assayData(input)$exprs))
    todos <- rownames(as.matrix(assayData(input)$exprs))
  else
    todos <- rownames(as.matrix(assayData(input)$counts))
  
  prob <- prob.concounts[todos]
  names(prob) <- todos

    ## Results
  resultat <- data.frame("level_1" = miMD$Level1, "level_2" = miMD$Level2, 
                         "M" = miMD$Ms, "D" = miMD$Ds, "prob" = prob)
  rownames(resultat) <- todos

  # We change the name of the conditions to "name_mean"
  colnames(resultat)[1] <- paste(unlist(strsplit(miMD$comp," "))[1],"mean",sep="_")
  colnames(resultat)[2] <- paste(unlist(strsplit(miMD$comp," "))[3],"mean",sep="_")
  
  resultat <- data.frame(resultat, "ranking" = ranking(resultat)$statistic)
  if (!is.null(featureData(input)@data$Length))
    resultat <- data.frame(resultat, "Length" = as.numeric(as.character(featureData(input)@data[todos,"Length"])),
                           stringsAsFactors = FALSE)  
  if (!is.null(featureData(input)@data$GC))
    resultat <- data.frame(resultat, "GC" = as.numeric(as.character(featureData(input)@data[todos,"GC"])),
                           stringsAsFactors = FALSE)  
  if (!is.null(featureData(input)@data$Chromosome))
    resultat <- data.frame(resultat, "Chrom" = as.character(featureData(input)@data$Chromosome),
                           "GeneStart" = as.numeric(as.character(featureData(input)@data$GeneStart)),
                           "GeneEnd" = as.numeric(as.character(featureData(input)@data$GeneEnd)),
                           stringsAsFactors = FALSE)
  if (!is.null(featureData(input)@data$Biotype))
    resultat <- data.frame(resultat, "Biotype" = as.character(featureData(input)@data[todos,"Biotype"]),
                           stringsAsFactors = FALSE)
  #resultat[order(resultat[,5], decreasing = TRUE),]
  
  Output(data = list(resultat), method=norm,k=miMD$k,lc=lc,factor=factor,v=v,nss=nss,pnr=pnr,
         comparison=miMD$comp,replicates=replicates)
}

################################################################################################
################################################################################################


####### NOISeqBIO
#################


noiseqbio = function (input, k = 0.5, norm = c("rpkm","uqua","tmm","n"), nclust = 15, plot = FALSE,
                      factor=NULL, conditions = NULL, lc = 0, r = 50, adj = 1.5,
                      a0per = 0.9, random.seed = 12345, filter = 1, depth = NULL,
                      cv.cutoff = 500, cpm = 1) 
  
  # input: Object containing gene counts and as many columns as samples.
  
  # k:      When counts = 0, 0 will be changed to k. By default, k = 0.5.
  
  # norm:   Normalization method. It can be one of "rpkm" (default), "uqua"
  #         (upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).
  
  # factor: String with the factor to choose which conditions you are going
  #         to compare.
  
# conditions: String with the conditions to compare in case one factor contains more
#         than 2 conditions.

# lc:     Length correction in done by dividing expression by length^lc.
#         By default, lc = 0. 


# r:      Number of permutations to compute null distribution (r=10).
# a0per:  Percentile of S to compute a0. If NULL, a0 = 0. (a0per = 0.9)




{
  
  if (inherits(input,"eSet") == FALSE)
    stop("ERROR: You must give an object generated by the readData function\n")
  
  if (is.null(factor))
    stop("ERROR: You must specify the factor to know which conditions you wish to compare.
         Please, give the argument 'factor'.\n")
  
  if (min(table(input@phenoData@data[,factor])) < 2)
    stop("ERROR: To run NOISeqBIO at least two replicates per condition are needed.
         Please, run NOISeq if there are not enough replicates in your experiment.\n")  
  
  norm <- match.arg(norm)
  
  
  # Random seed
  if (!is.null(random.seed)) set.seed(random.seed)  


  
  # Z-scores for signal and noise
  cat("Computing Z values...\n")
  miMD <- allMDbio(input, factor, k = k, norm = norm, conditions = conditions, lc = lc, 
                   r = r, a0per = a0per, nclust = nclust, filter = filter, depth = depth,
                   cv.cutoff = cv.cutoff, cpm = cpm)
  
  
  #------------------------------------------------------------------#
  
  
  
  ##### Probability of differential expression
  
  cat("Computing probability of differential expression...\n")
  
  
  ## KDE estimators of f0 and f  
  desde <- min(c(miMD$Zs,miMD$Zn), na.rm = TRUE)
  hasta <- max(c(miMD$Zs,miMD$Zn), na.rm = TRUE)
  
  fdens <- density(miMD$Zs, adjust = adj, n = 5000, from = desde, to = hasta, na.rm = TRUE)
  f <- approxfun(fdens)
  
  f0dens <- density(miMD$Zn, adjust = adj, n = 5000, from = desde, to = hasta, na.rm = TRUE)
  f0 <- approxfun(f0dens)
  
  if (f0(0)/f(0) < 1) print("WARNING: f0(0)/f(0) < 1 => FP with Z~0 will be detected.")
  
  f0.f <- f0(miMD$Zs) / f(miMD$Zs)
  
  
  ## f0, f plot
  if (plot) {
    plot(f0dens, lwd = 2, col = 4, main = paste("r=", r, "; a0per=", a0per, sep = ""), xlab = "theta")
    lines(fdens, lwd = 2, col = 1)
    legend("topleft", c("f","f0"), col = c(1,4), lwd = 2)
  }
    
  
  ## ESTIMATION of p0
  p0 <- min(1/f0.f, na.rm = TRUE)
  p0 = min(p0,1)
  
  cat(paste("p0 =", p0)); cat("\n")
    
  
  ## PROBABILITY of DIFFERENTIAL EXPRESSION
  myprob <- 1 - p0*f0.f
  if(min(myprob, na.rm = TRUE) < 0) {
    print(summary(f0.f))
  } 
  names(myprob) <- names(miMD$Zs)
  
  cat("Probability\n")
  print(summary(myprob))
       
  
  if (!is.null(assayData(input)$exprs))
    todos <- rownames(as.matrix(assayData(input)$exprs))
  else
    todos <- rownames(as.matrix(assayData(input)$counts))
  
  myprob <- myprob[todos]
  names(myprob) <- todos
  
  ## Results
  resultat <- data.frame("level_1" = miMD$Level1, "level_2" = miMD$Level2, "theta" = miMD$Zs, "prob" = myprob,
                         "log2FC" = log2(miMD$Level1/miMD$Level2))
  rownames(resultat) <- todos
  
  colnames(resultat)[1] <- paste(unlist(strsplit(miMD$comp," "))[1],"mean",sep="_")
  colnames(resultat)[2] <- paste(unlist(strsplit(miMD$comp," "))[3],"mean",sep="_")
  
  #   resultat <- data.frame(resultat, "ranking" = ranking(resultat)$statistic)
  if (!is.null(featureData(input)@data$Length))
    resultat <- data.frame(resultat, "length" = as.numeric(as.character(featureData(input)@data[todos,"Length"])),
                           stringsAsFactors = FALSE)
  
  if (!is.null(featureData(input)@data$GC))
    resultat <- data.frame(resultat, "GC" = as.numeric(as.character(featureData(input)@data[todos,"GC"])),
                           stringsAsFactors = FALSE)
  
  if (!is.null(featureData(input)@data$Chromosome))
    resultat <- data.frame(resultat, "Chrom" = (as.character(featureData(input)@data$Chromosome)),
                           "GeneStart" = as.numeric(as.character(featureData(input)@data$GeneStart)),
                           "GeneEnd" = as.numeric(as.character(featureData(input)@data$GeneEnd)),
                           stringsAsFactors = FALSE)
  
  if (!is.null(featureData(input)@data$Biotype))
    resultat <- data.frame(resultat, "Biotype" = as.character(featureData(input)@data[todos,"Biotype"]),
                           stringsAsFactors = FALSE)
      
  Output(data = list(resultat), method=norm, k=miMD$k, lc=lc, factor=factor, comparison=miMD$comp,
         replicates="biological", v = 0, nss = 0, pnr = 0)
}

MD <-
function (dat = dat, selec = c(1:nrow(dat))) {
  pares <- as.matrix(combn(ncol(dat), 2))

  if (NCOL(pares) > 30) {  
    sub30 <- sample(1:NCOL(pares), size = 30, replace = FALSE)
    pares <- pares[,sub30]
  }
  
  mm <- NULL
  dd <- NULL
  for (i in 1:ncol(pares)) {
    a <- dat[selec,pares[1,i]]
    b <- dat[selec,pares[2,i]]
    mm <- cbind(mm, log(a/b, 2))
    dd <- cbind(dd, abs(a-b))
  }
  list("M" = mm, "D" = dd)
}



###########################################################################################
###########################################################################################



MDbio = function (dat = dat, selec = c(1:nrow(dat)), param = NULL, a0per = 0.9) {
  
  pares <- as.matrix(combn(ncol(dat), 2))
  
  mm <- NULL
  dd <- NULL
  for (i in 1:ncol(pares)) {
    a <- dat[selec,pares[1,i]]
    b <- dat[selec,pares[2,i]]
    mm <- cbind(mm, log(a/b, 2))
    dd <- cbind(dd, (a-b))  
  }
  
  
  ## Correcting (M,D) 
  sd.M = sqrt(param$sd[,1]^2 / (dat[,1]^2 * log(2)^2 * param$n[1]) + 
                param$sd[,2]^2 / (dat[,2]^2 * log(2)^2 * param$n[2]))
  
  sd.D = sqrt(param$sd[,1]^2/param$n[1] + param$sd[,2]^2/param$n[2])
  
  if(is.null(a0per)) {
    a0.M = a0.D = 0
  } else {
    
    if (a0per == "B") {
      B = 100
      a0.M <- B*max(sd.M, na.rm = TRUE)
      a0.D <- B*max(sd.D, na.rm = TRUE)
      
    } else {
      a0per = as.numeric(a0per)
      a0.M <- quantile(sd.M, probs = a0per, na.rm = TRUE)
      a0.D <- quantile(sd.D, probs = a0per, na.rm = TRUE)
    }
  }
  
  mm <- mm / (a0.M + sd.M)
  dd <- dd / (a0.D + sd.D)
  
  
  # Results
  list("M" = mm, "D" = dd)
}

biodetection.dat <- function(input, factor = NULL, k = 0) {

  if (inherits(input,"eSet") == FALSE)
    stop("Error. The input data must be an eSet object\n")

  if (any(!is.na(featureData(input)@data$Biotype)) == FALSE)
    stop ("No biological classification was provided.\nPlease run addData() function to add 
          this information\n")

  if (!is.null(assayData(input)$exprs)) {
    dat <- as.matrix(assayData(input)$exprs)
    mysamples = colnames(assayData(input)$exprs)
  } else {
    dat <- as.matrix(assayData(input)$counts)
    mysamples = colnames(assayData(input)$counts)
  }
  
  numgenes = nrow(dat)
  
  if (is.null(factor)) {  # per sample  
    cat("Biotypes detection is to be computed for:\n")
    print(colnames(dat))
    biotablas = vector("list", length = NCOL(dat))
    names(biotablas) = colnames(dat)
    
  } else {  # per condition
    mifactor = as.factor(pData(input)[,factor])
    niveles = levels(mifactor)
    cat("Biotypes detection is to be computed for:\n")
    print(niveles)
    biotablas = vector("list", length = length(niveles))
    names(biotablas) = niveles
    dat = sapply(niveles, function (k) rowSums(as.matrix(dat[, mifactor == k])))
  }
  
    
  infobio <- as.character(featureData(input)@data$Biotype)
  
  genome <- 100*table(infobio)/sum(table(infobio))
  ordre <- order(genome, decreasing = TRUE)
  
  for (i in 1:length(biotablas)) {
    
    detect <- dat[,i] > k
    
    perdet1 <- genome*table(infobio, detect)[names(genome),"TRUE"]/
      table(infobio)[names(genome)]
    
    perdet2 <- 100*table(infobio, detect)[names(genome),"TRUE"] /
      sum(table(infobio, detect)[,"TRUE"])    
    
    biotablas[[i]] <- as.matrix(rbind(perdet1[ordre], perdet2[ordre]))
    rownames(biotablas[[i]]) <- c("detectionVSgenome", "detectionVSsample")    
  }
  
  mybiotable = list("genome" = genome[ordre], "biotables" = biotablas, "genomesize" = numgenes)  
  mybiotable
}







#############################################################################################
#############################################################################################
#############################################################################################



biodetection.plot <- function(dat, samples = c(1,2), plottype = c("persample", "comparison"), 
                              toplot = "protein_coding", toreport = FALSE,...) {  
  
  mypar = par(no.readonly = TRUE)
  
  plottype = match.arg(plottype)

  if (length(samples) > 2) {
    stop("ERROR: This function cannot generate plots for more than 2 samples.\n 
         Please, use it as many times as needed to generate the plots for all your samples.\n")
  }
  
  if (is.numeric(samples)) samples = names(dat$biotables)[samples]
    
  biotable1 <- rbind(dat$genome, dat$biotables[[samples[1]]], rep(0, length(dat$genome)))
  
    
  
  # Computing ylim for left and right axis
     
  if (ncol(biotable1) >= 3) {
    ymaxL <- ceiling(max(biotable1[,1:3], na.rm = TRUE))
    ymaxR <- max(biotable1[,-c(1:3)], na.rm = TRUE)        
  } else {
    ymaxL <- ceiling(max(biotable1, na.rm = TRUE))
    ymaxR = 0 
  } 
  
  
  if (length(samples) == 2) {
    
    biotable2 <- rbind(dat$genome, dat$biotables[[samples[2]]], rep(0, length(dat$genome)))
    
    if (ncol(biotable2) >= 3) {
      ymax2 <- ceiling(max(biotable2[,1:3], na.rm = TRUE))
      ymax2sin <- max(biotable2[,-c(1:3)], na.rm = TRUE)
      ymaxR <- ceiling(max(ymaxR, ymax2sin))
    } else {
      ymax2 <- ceiling(max(biotable2, na.rm = TRUE))      
    }    
    
    ymaxL = max(ymaxL, ymax2)   
  }
  
  
  
  # Rescaling biotables (datos2)
  if (length(samples) == 2) {        
    if (ncol(biotable2) >= 3) biotable2[,-c(1:3)] <- biotable2[,-c(1:3)]*ymaxL/ymaxR    
  }  
  
  # Rescaling biotables (datos1)  
  if (ncol(biotable1) >= 3) biotable1[,-c(1:3)] <- biotable1[,-c(1:3)]*ymaxL/ymaxR
  
  
  ## PLOTS
    
  if (length(samples) == 1) {   # Plot (1 sample) - 2 scales    
    
    par(mar = c(11, 4, 2, 2))
    
    barplot(biotable1[c(1,3),], main = samples[1],
            xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
            beside = TRUE, col = c("grey", 2), las = 2,
            ylim = c(0, ymaxL), border = c("grey", 2))
    
    barplot(biotable1[c(2,4),], main = samples[1],
            xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
            beside = TRUE, col = c(2, 1), las = 2, density = 30,
            ylim = c(0, ymaxL), border = 2, add = TRUE)    
    
    if (ymaxR > 0) {  # if number of biotypes >= 3 so we have left and right axis
      axis(side=4, at = pretty(c(0,ymaxL), n = 5), 
           labels = round(pretty(c(0,ymaxL), n = 5)*ymaxR/ymaxL, 1))    
      abline(v = 9.5, col = 3, lwd = 2, lty = 2)        
    }    
    
    legend(x = "topright", bty = "n", horiz = FALSE,
           fill = c("grey", 2, 2), density = c(NA,30,NA),
           border = c("grey", 2, 2),
           legend = c("% in genome", "detected", "% in sample"))
    
    
  } else {   # Plot (2 samples)    
    
    par(mar = c(11, 4, 2, 2))    
    
    if (plottype == "persample") {    ### A plot for each sample separately
      
      # Datos1
      barplot(biotable1[c(1,3),], main = samples[1],
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c("grey", 2), las = 2,
              ylim = c(0, ymaxL), border = c("grey", 2))
      
      barplot(biotable1[c(2,4),], main = samples[1],
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c(2, 1), las = 2, density = 30,
              ylim = c(0, ymaxL), border = 2, add = TRUE)
      
      if (ymaxR > 0) {   # if number of biotypes >= 3 so we have left and right axis
        axis(side=4, at = pretty(c(0,ymaxL), n = 5), 
             labels = round(pretty(c(0,ymaxL), n = 5)*ymaxR/ymaxL, 1))      
        abline(v = 9.5, col = 3, lwd = 2, lty = 2)
      }  
      
      legend(x = "topright", bty = "n", horiz = FALSE,
             fill = c("grey", 2, 2), density = c(NA,30,NA), border = c("grey", 2, 2),
             legend = c("% in genome", "detected", "% in sample"))
      
      
      # Datos2        
      barplot(biotable2[c(1,3),], main = samples[2],
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c("grey", 4), las = 2,
              ylim = c(0, ymaxL), border = c("grey", 4))
      
      barplot(biotable2[c(2,4),], main = samples[2],
              xlab = NULL, ylab = "%features", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c(4, 1), las = 2, density = 30,
              ylim = c(0, ymaxL), border = 4, add = TRUE)
      
      if (ymaxR > 0) {   # if number of biotypes >= 3 so we have left and right axis
        axis(side=4, at = pretty(c(0,ymaxL), n = 5), 
             labels = round(pretty(c(0,ymaxL), n = 5)*ymaxR/ymaxL, 1))
        abline(v = 9.5, col = 3, lwd = 2, lty = 2)
      }      
      
      legend(x = "topright", bty = "n", horiz = FALSE, 
             fill = c("grey", 4, 4), density = c(NA,30,NA),
             border = c("grey", 4, 4),
             legend = c("% in genome", "detected", "% in sample"))
      
    }

    
    if (plottype == "comparison") {   ## A plot comparing two samples with regard to genome and for % in sample
      
      lefttable = rbind(100*dat$biotables[[samples[1]]][1,]/dat$genome, 100*dat$biotables[[samples[2]]][1,]/dat$genome)     
      
      righttable = rbind(dat$biotables[[samples[1]]][2,], dat$biotables[[samples[2]]][2,])
      
      if (length(toplot) > 1) {
        toplot = toplot[1]
        print("WARNING: More than one biotype was provided, the proportion test will only by applied to the first biotype.")
      }
      
      if ((toplot != 1) && (toplot != "global")) {         
        numgenes = dat$genomesize
        myx = round(righttable[,toplot]*numgenes/100, 0)
        mytest = prop.test(x = myx, n = rep(numgenes, 2), alternative = "two.sided")
        if (is.numeric(toplot)) toplot = colnames(righttable)[toplot]
      }
      
      asumar = colSums(righttable)
      asumar = which(asumar < 0.25)
      if (length(asumar) > 1) {
        righttable = cbind(righttable[,-asumar], rowSums(righttable[,asumar]))
        colnames(righttable)[ncol(righttable)] = "Others"
      }            
      
      # Detection in the genome
      bbb = barplot(lefttable, main = "Biotype detection over genome total",
                    xlab = NULL, ylab = "% detected features", axis.lty = 1, legend = FALSE, cex.names = 0.8,
                    beside = TRUE, col = c(2,4), las = 2, density = 80, border = c(2,4), ylim = c(0,100))    
      bbb = colSums(bbb)/2
      lines(bbb, dat$genome, pch = 20, type = "o", lwd = 2)
      
      
      # %detection in the sample        
      barplot(righttable, main = "Relative biotype abundance in sample",
              xlab = NULL, ylab = "Relative % biotypes", axis.lty = 1, legend = FALSE,
              beside = TRUE, col = c(2, 4), las = 2, border = c(2,4))
                  
      legend(x = "topright", bty = "n", horiz = FALSE, pch = c(15,15,20), lwd = c(NA,NA,1), legend = c(samples, "% in genome"),
             col = c(2,4,1))
      
      if ((toplot != 1) && (toplot != "global")) {        
        print(paste("Percentage of", toplot, "biotype in each sample:"))
        names(mytest$estimate) = samples
        print(round(mytest$estimate*100, 4))
        print(paste("Confidence interval at 95% for the difference of percentages:", samples[1], "-", samples[2]))
        print(round(mytest$conf.int[1:2]*100, 4))
        if (mytest$p.value < 0.05) {
          print(paste("The percentage of this biotype is significantly DIFFERENT for these two samples (p-value =",
                      signif(mytest$p.value, 4), ")."))
        } else {
          print(paste("The percentage of this biotype is NOT significantly different for these two samples (p-value =",
                      signif(mytest$p.value, 4), ")."))
        }       
      }      
    }
    
    
  }
  
  # Reset with the default values
  if (!toreport) par(mypar)
  
  
}

#################################################################

busca <-
function (x, S) {
  which(S[,1] == x[1] & S[,2] == x[2])
}


#################################################################



int.mult <- function(lista, todos = NULL) {
  
  if(is.null(todos)) {
    todos <- unlist(lista)
  }

  comunes <- todos

  for(i in 1:length(lista)) {
    comunes <- intersect(comunes, lista[[i]])
  }

  comunes
}



#################################################################


n.menor <-
function (x, S1, S2) {
 
  length(which(S1 <= x[1] &  S2 <= x[2]))

}



#################################################################



noceros <-
function (x, num = TRUE, k = 0) {
  
  nn <- length(which(x > k))
  
  if (num) {
    nn
    
  } else {
    if(nn > 0) { which(x > k) } else { NULL }
  }
}



#################################################################


sinceros <-
function (datos, k) {
  datos = as.matrix(datos)
  datos0 <- as.matrix(datos)

  if (is.null(k)) {

    mini0 <- min(datos[noceros(datos, num = FALSE, k = 0)])

    kc <- mini0/2

    datos0[datos0 == 0] <- kc

  } else {

    datos0[datos0 == 0] <- k

  }
  
  datos0
}



#################################################################


#### Simulating samples


sim.samples <-
  function(counts1, counts2 = NULL, pnr = 1, nss = 5, v = 0.02) {
    seqdep <- c(sum(counts1), sum(counts2))
    num.reads1 <- (pnr + c(-v,v))*seqdep[1]
    
    muestras <- vector("list")
    
    muestras$c1 <- NULL
    for (s in 1:nss) {
      tama <- round(runif(1, num.reads1[1], num.reads1[2]), 0)
      muestras$c1 <- cbind(muestras$c1,
                           rmultinom(1, size = tama, prob = counts1))
    }
    
    if(!is.null(counts2)) {
      num.reads2 <- (pnr + c(-v,v))*seqdep[2]
      muestras$c2 <- NULL
      for (s in 1:nss) {
        tama <- round(runif(1, num.reads2[1], num.reads2[2]), 0)
        muestras$c2 <- cbind(muestras$c2,
                             rmultinom(1, size = tama, prob = counts2))
      }
    }
    
    muestras
  }



#################################################################




ranking <-
  function(results) {
    
    M <- results$M
    
    D <- results$D
    
    prob <- results$prob
    
    ## Changing NA by 0
    M[is.na(M)] <- 0
    D[is.na(D)] <- 0
    prob[is.na(prob)] <- 0
    
    
    ## Ranking
    
    #   ranking1 <- M*prob
    # 
    #   ranking2 <- sign(M)*prob
    # 
    #   ranking3 <- M*D
    # 
    #   ranking4 <- M*D*prob
    
    ranking5 <- sqrt(M*M + D*D)*sign(M)
    
    
    ## Ranking results
    #list(ranking1, ranking2, ranking3, ranking4, ranking5)
    theranking <- data.frame(rownames(results), ranking5)
    rownames(theranking) <- NULL
    colnames(theranking) <- c("ID", "statistic")
    
    theranking
    
  }




#################################################################



#############################################################################
##############   Plot with 2 different Y axis (left and right)   ############
#############################################################################


# By Ajay Shah (taken from [R] Plot 2 time series with different y axes (left and right),
# in https://stat.ethz.ch/pipermail/r-help/2004-March/047775.html) 

# Modified by: Sonia Tarazona

### PARAMETERS (default):
# x: data to be drawn on X-axis
# yright: data to be drawn on Y right axis
# yleft: data to be drawn on Y left axis
# yrightlim (range(yright, na.rm = TRUE)): ylim for rigth Y-axis
# yleftlim (range(yleft, na.rm = TRUE)): ylim for left Y-axis
# xlab (NULL): Label for X-axis
# yylab (c("","")): Labels for right and left Y-axis
# pch (c(1,2)): Type of symbol for rigth and left data
# col (c(1,2)): Color for rigth and left data
# linky (TRUE): If TRUE, points are connected by lines.
# smooth (0): Friedman's super smoothing
# lwds (1): Line width for smoothed line
# length (10): Number of tick-marks to be drawn on axis
# ...: Other graphical parameters to be added by user (such as main, font, etc.)
###



plot.y2 <- function(x, yright, yleft, yrightlim = range(yright, na.rm = TRUE),
                    yleftlim = range(yleft, na.rm = TRUE),
                    xlim = range(x, na.rm = TRUE),
                    xlab = NULL, yylab = c("",""), lwd = c(2,2),
                    pch = c(1,2), col = c(1,2), type = c("o","o"),
                    linky = TRUE, smooth = 0, bg = c("white","white"),
                    lwds = 1, length = 10, ...,
                    x2 = NULL, yright2 = NULL, yleft2 = NULL, col2 = c(3,4))
{
  #par(mar = c(5,2,4,2), oma = c(0,3,0,3))

  ## Plotting RIGHT axis data

  plot(x, yright, axes = FALSE, ylab = "", xlab = xlab, ylim = yrightlim,
       xlim = xlim, pch = pch[1], type = type[1], lwd = lwd[1],
       col = col[1], ...)
  
  axis(4, pretty(yrightlim, length), col = 1, col.axis = 1)

  if (is.null(yright2) == FALSE) {
    points(x2, yright2, type = type[1], pch = pch[1], lwd = lwd[1], col = col2[1], ...)
  }
  
  #if (linky) lines(x, yright, col = col[1], ...)
  
  if (smooth != 0) lines(supsmu(x, yright, span = smooth), col = col[1], lwd = lwds, ...)
  
  if(yylab[1]=="") {
    mtext(deparse(substitute(yright)), side = 4, outer = FALSE, line = 2,
          col = 1,...)
  } else {
    mtext(yylab[1], side = 4, outer = FALSE, line = 2, col = 1, ...)
  }
  

  par(new = T)

  ## Plotting LEFT axis data
  
  plot(x, yleft, axes = FALSE, ylab = "" , xlab = xlab, ylim = yleftlim, 
       xlim = xlim, bg = bg[1],
       pch = pch[2], type = type[2], lwd = lwd[2], col = col[2], ...)
  
  box()
  
  axis(2, pretty(yleftlim, length), col = 1, col.axis = 1)

  if (is.null(yleft2) == FALSE) {
    points(x2, yleft2, type = type[2], pch = pch[2], bg = bg[2],
           lwd = lwd[2], col = col2[2], ...)
  }
  

  #if (linky) lines(x, yleft, col = col[2], ...)
  
  if (smooth != 0) lines(supsmu(x, yleft, span = smooth), col = col[2], lwd=lwds, ...)
  
  if(yylab[2] == "") {
    mtext(deparse(substitute(yleft)), side = 2, outer = FALSE, line = 2, col = 1, ...)
  } else {
    mtext(yylab[2], side = 2, outer = FALSE, line = 2, col = 1, ...)
  }
  
  
  ## X-axis
  axis(1, at = pretty(xlim, length))
  
   
}








#################################################################


## Log-scale for plots


logscaling = function (data, base = 2, k = 1) {
  
  # IDEA
  # plot(data,...,yaxt = "n")
  # axis(side = 2, at = donde, labels = etiquetas)
  
  logmaximo = round(max(data, na.rm = TRUE), 0)
  numceros = nchar(logmaximo)-1
    
  etiquetas = c(0, 10^(1:numceros))
  
  donde = log(etiquetas + k, base = base)
  
  data = log(data + k, base = base)
  
  list("data" = data, "at" = donde, "labels" = etiquetas)
    
}





##***************************************************************************##
##***************************************************************************##




miscolores <- colors()[c(554,89,111,512,17,586,132,428,601,568,86,390)]

allMD <- function (input, factor, conditions, k = 0.5, replicates, norm = "rpkm",
            pnr = 0.2, nss = 5, v = 0.02, lc = 0)

# input:  Set of data of type Input

# conditions: Levels of the factor to be compared (when the factor has more than 2 levels)
  
# k:      When counts = 0, 0 will be changed to k. By default, k = 0.5.
  
# norm:   Normalization method. It can be one of "rpkm" (default), "uqua"
#         (upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).

# pnr:    Percentage of total reads (seq.depth) for each simulated sample.
#         Only needed when noise = "simul". By default, pnr = 1.

# nss:    Number of simulated samples (>= 2). By default, nss = 5.
#         If nss = 0, real samples are used to compute noise.

# v:      Variability in sample total reads used to simulate samples.
#         By default, v = 0.02. Sample total reads is computed as a
#         random value from a uniform distribution in the interval
#         [(pnr-v)*sum(counts), (pnr+v)*sum(counts)]

# lc:     Length correction in done by dividing expression by length^lc.
#         By default, lc = 1. 

{

#   n1 <- ncol(as.matrix(datos1))
#   n2 <- ncol(as.matrix(datos2))
#   g.sinL <- names(which(is.na(long)))
  
  # Check if the factor introduced is already defined
  # If the factor introduced is defined and has more than 2 conditions, it will check if the conditions specified are defined too
  condition_fac = FALSE
  condition_lev = FALSE
  
  datos1 <- datos2 <- matrix()

  for (i in colnames(pData(input))) {
    if (factor == i) {
      condition_fac = TRUE  
      if (!is.factor(pData(input)[,i])) pData(input)[,i] = as.factor(pData(input)[,i])
      if (length(levels(pData(input)[,i])) == 2) {

        if (!is.null(assayData(input)$exprs)) {
          datos1 <- assayData(input)$exprs[,which(pData(input)[,i] ==levels(pData(input)[,i])[1]), drop = FALSE]
          datos2 <- assayData(input)$exprs[,which(pData(input)[,i] ==levels(pData(input)[,i])[2]), drop = FALSE]
        } else {
          datos1 <- assayData(input)$counts[,which(pData(input)[,i] ==levels(pData(input)[,i])[1]), drop = FALSE]
          datos2 <- assayData(input)$counts[,which(pData(input)[,i] ==levels(pData(input)[,i])[2]), drop = FALSE]
        }

        # Define the comparison string
        comparison <- paste(levels(pData(input)[,i])[1], levels(pData(input)[,i])[2], sep=" - ")

        condition_lev = TRUE
      }

      else {
        if (is.null(conditions))
          stop("Error. You must specify which conditions you wish to compare when the factor has two or more conditions.\n")
        if (length(conditions) != 2)
          stop("Error. The argument conditions must contain the 2 conditions you wish to compare.")
        
        l <- conditions %in% pData(input)[,i]
        # If they are defined, they will be TRUE
        if (l[1] == TRUE && l[2] == TRUE) {

          if (!is.null(assayData(input)$exprs)) {
            datos1 <- assayData(input)$exprs[,which(pData(input)[,i] == conditions[1]), drop = FALSE]
            datos2 <- assayData(input)$exprs[,which(pData(input)[,i] == conditions[2]), drop = FALSE]
          } else {
            datos1 <- assayData(input)$counts[,which(pData(input)[,i] == conditions[1]), drop = FALSE]
            datos2 <- assayData(input)$counts[,which(pData(input)[,i] == conditions[2]), drop = FALSE]
          }                    
          # Define the comparison string
          comparison <- paste(conditions[1],conditions[2], sep=" - ")
          
          condition_lev = TRUE
        }
      }        
    }
  }  
      
  if (condition_fac == FALSE)
    stop("The factor you have written does not correspond with any of the ones you have defined.")
  
  if (condition_lev == FALSE)
    stop("The conditions you have written don't exist in the factor specified.\n")
  
  # Correction to make it work when there are simulated samples
  if (replicates == "no")
    replicates = "technical"
#  if (description(input)@samples[[1]] == "no")
#    description(input)@samples[[1]] = "technical"
 
  n1 <- ncol(as.matrix(datos1))
  n2 <- ncol(as.matrix(datos2))
  
  if (norm == "n") {      # no normalization
    datos1 <- round(datos1, 100)
    datos2 <- round(datos2, 100)
  }


  if (is.null(k)) {
      m1 <- min(datos1[noceros(datos1, num = FALSE)], na.rm = TRUE)
      m2 <- min(datos2[noceros(datos2, num = FALSE)], na.rm = TRUE)
      mm <- min(m1, m2)
      k <- mm/2    
  } 

  
    # Total counts for each gene:
  suma1 <- rowSums(as.matrix(datos1))
  suma2 <- rowSums(as.matrix(datos2))
  

    # All genes
  todos <- rownames(as.matrix(datos1))

    # Genes with counts in any condition
  concounts <- names(which(suma1+suma2 > 0))

  long <- 1000
  g.sinL <- NULL
  
  if (!is.null(featureData(input)@data$Length)) {
    g.sinL <- names(which(is.na(featureData(input)@data$Length)))
    if (any(!is.na(featureData(input)@data$Length)) == TRUE) 
      long <- featureData(input)@data[concounts, "Length"]
  }
  
  
  if (replicates == "technical") {  ### technical replicates
    suma1 <- suma1[concounts]
    suma2 <- suma2[concounts]    

    #-------------------------------------------------------------------------#
    # Normalization of counts for each condition (aggregating replicates)

    if (norm == "rpkm") {      # RPKM
      suma1.norm <- rpkm(suma1, long = long, k = k, lc = lc)
      suma2.norm <- rpkm(suma2, long = long, k = k, lc = lc)
    }

    
    if (norm == "uqua") {
      suma.norm <- uqua(cbind(suma1, suma2), long = long, lc = lc, k = k)
      suma1.norm <- as.matrix(suma.norm[ ,1])
      suma2.norm <- as.matrix(suma.norm[ ,2])
    }

    
    if (norm == "tmm") {
      suma.norm <- tmm(as.matrix(cbind(suma1, suma2)), long = long,
                       lc = lc, k = k)      
      suma1.norm <- as.matrix(suma.norm[ ,1])
      suma2.norm <- as.matrix(suma.norm[ ,2])
    }
    
  }
   

    #-------------------------------------------------------------------------#

    ## Noise distribution

  if ((n1+n2)>2) {   # with real samples

    datitos <- cbind(datos1, datos2)
    datitos <- datitos[concounts,]

    gens.sin0 <- setdiff(concounts, g.sinL)

    if (norm == "n") {       # no normalization
      datitos.0 <- sinceros(datitos, k = k)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    if (norm == "rpkm") {      # RPKM
      datitos.0 <- rpkm(datitos, long = long, k = k, lc = lc)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    if (norm == "uqua") {      # Upper Quartile
      datitos.0 <- uqua(datitos, long = long, lc = lc, k = k)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    if (norm == "tmm") {
      datitos.0 <- tmm(datitos, long = long, lc = lc, k = k)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    datos1.norm <- datitos.norm[ ,1:n1]
    datos2.norm <- datitos.norm[ ,(n1+1):(n1+n2)]

    if (n1 > 1) {
      MD1 <- MD(dat = datos1.norm)
    } else { MD1 <- NULL }

    if (n2 > 1) {
      MD2 <- MD(dat = datos2.norm)
    } else { MD2 <- NULL }

    
  } else {                 # with simulated samples

    if (nss == 0) {
      nss <- 5
    }

    datos.sim <- sim.samples(counts1 = sinceros(suma1, k = k),
                             counts2 = sinceros(suma2, k = k),
                             pnr = pnr, nss = nss, v = v)

    nn <- sapply(datos.sim, ncol)

    dat.sim.norm <- vector("list", length = 2)

    datitos <- cbind(datos.sim[[1]], datos.sim[[2]])
    rownames(datitos) = names(suma1)

    sumita <- rowSums(datitos)
    g.sin0 <- names(which(sumita > 0))
    gens.sin0 <- setdiff(g.sin0, g.sinL)

    if (norm == "n") {       # no normalization
      datitos.0 <- sinceros(datitos, k = k)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    if (norm == "rpkm") {      # RPKM
      datitos.0 <- rpkm(datitos, long = long, k = k, lc = lc)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    if (norm == "uqua") {      # Upper Quartile
      datitos.0 <- uqua(datitos, long = long, lc = lc, k = k)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    if (norm == "tmm") {
      datitos.0 <- tmm(datitos, long = long, lc = lc, k = k)
      datitos.norm <- datitos.0[gens.sin0, ]
    }

    dat.sim.norm[[1]] <- datitos.norm[ ,1:nn[1]]
    dat.sim.norm[[2]] <- datitos.norm[ ,(nn[1]+1):sum(nn)]

    MD1 <- MD(dat = dat.sim.norm[[1]])
    MD2 <- MD(dat = dat.sim.norm[[2]])

  }

  Mr <- c(as.numeric(MD1$M), as.numeric(MD2$M))
  Dr <- c(as.numeric(MD1$D), as.numeric(MD2$D))

  
  
    #-------------------------------------------------------------------------#

    ## M and D for different experimental conditions

  if (replicates == "technical" & norm != "n") {
    
    MDs <- MD(dat = cbind(suma1.norm, suma2.norm))
    
    lev1 <- suma1.norm[,1]
    lev1 <- lev1[todos]
        
    lev2 <- suma2.norm[,1]
    lev2 <- lev2[todos]

  } else {

    if ((n1+n1) == 2) {
      datos1.norm <- sinceros(as.matrix(datos1)[concounts,], k = k)
      datos2.norm <- sinceros(as.matrix(datos2)[concounts,], k = k)      
    }  
    

    resum1.norm <- rowMeans(as.matrix(datos1.norm))
    resum2.norm <- rowMeans(as.matrix(datos2.norm))

    lev1 <- resum1.norm[todos]
    lev2 <- resum2.norm[todos]
    
    MDs <- MD(dat = cbind(resum1.norm, resum2.norm))

  }
  

      ## Completing M and D
  
  names(lev1) <- names(lev2) <- todos
  
  Ms <- as.numeric(MDs$M)
  names(Ms) <- rownames(MDs$M)
  Ms <- Ms[todos]
  names(Ms) <- todos

  Ds <- as.numeric(MDs$D)
  names(Ds) <- rownames(MDs$D)
  Ds <- Ds[todos]
  names(Ds) <- todos

  ## Results
  list("k" = k, "comp" = comparison, "Level1" = lev1, "Level2" = lev2, "Ms" = Ms, "Ds" = Ds, "Mn" = Mr, "Dn" = Dr)
 
}






#######################################################################
#######################################################################



allMDbio = function (input, factor, conditions, k = 0.5, norm = "rpkm", lc = 1,
                      r = 10, a0per = 0.9, nclust = 15, filter = 1, depth = NULL,
                     cv.cutoff = 0, cpm = 1)
  
  # input:  Set of data of type Input
  
  # conditions: Levels of the factor to be compared (when the factor has more than 2 levels)
  
  # k:      When counts = 0, 0 will be changed to k. By default, k = 0.5.
  
  # norm:   Normalization method. It can be one of "rpkm" (default), "uqua"
  #         (upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).

# lc:     Length correction in done by dividing expression by length^lc.
#         By default, lc = 1. 

# r:      Number of permutations to compute null distribution (r=10).
# a0per:  Percentile of S to compute a0. If NULL, a0 = 0. (a0per = 0.9)

{
  
  
  
  # Check if the factor introduced is already defined
  # If the factor introduced is defined and has more than 2 conditions, 
  # it will check if the conditions specified are defined too
  condition_fac = FALSE
  condition_lev = FALSE
  
  datos1 <- datos2 <- matrix()
  
  for (i in colnames(pData(input))) {
    if (factor == i) {
      condition_fac = TRUE
      if (!is.factor(pData(input)[,i])) pData(input)[,i] = as.factor(pData(input)[,i])
      if (length(levels(pData(input)[,i])) == 2) {
        
        if (!is.null(assayData(input)$exprs)) {
          datos1 <- assayData(input)$exprs[,which(pData(input)[,i] ==levels(pData(input)[,i])[1])]
          datos2 <- assayData(input)$exprs[,which(pData(input)[,i] ==levels(pData(input)[,i])[2])]
        } else {
          datos1 <- assayData(input)$counts[,which(pData(input)[,i] ==levels(pData(input)[,i])[1])]
          datos2 <- assayData(input)$counts[,which(pData(input)[,i] ==levels(pData(input)[,i])[2])]
        }
        
        # Define the comparison string
        comparison <- paste(levels(pData(input)[,i])[1], levels(pData(input)[,i])[2], sep=" - ")
        
        condition_lev = TRUE
        
        if (!((ncol(datos1) > 1) && (ncol(datos2) > 1))) 
          stop("Error. NOISeqBIO needs at least 2 biological replicates per condition.\n")
      }
      
      else {
        if (is.null(conditions))
          stop("Error. You must specify which conditions you wish to compare when the factor has two or more conditions.\n")
        if (length(conditions) != 2)
          stop("Error. The argument conditions must contain the 2 conditions you wish to compare.")
        
        l <- conditions %in% pData(input)[,i]
        # If they are defined, they will be TRUE
        if (l[1] == TRUE && l[2] == TRUE) {
          
          if (!is.null(assayData(input)$exprs)) {
            datos1 <- assayData(input)$exprs[,which(pData(input)[,i] == conditions[1])]
            datos2 <- assayData(input)$exprs[,which(pData(input)[,i] == conditions[2])]
          } else {
            datos1 <- assayData(input)$counts[,which(pData(input)[,i] == conditions[1])]
            datos2 <- assayData(input)$counts[,which(pData(input)[,i] == conditions[2])]
          }                    
          # Define the comparison string
          comparison <- paste(conditions[1],conditions[2], sep=" - ")
          
          condition_lev = TRUE
        }
      }        
    }
  }  
  
  if (condition_fac == FALSE)
    stop("The factor specified does not correspond with any of the ones you have defined.")
  
  if (condition_lev == FALSE)
    stop("The conditions specified don't exist for the factor specified.\n")
  
  
  ##-------------------------------------------------------------------------##
  
  
  # Number of observations within each condition 
  n1 <- ncol(as.matrix(datos1))
  n2 <- ncol(as.matrix(datos2))
  if (max(n1,n2) == 1) 
    stop("There is only one replicate per condition. Please, use NOISeq instead of NOISeqBIO.\n")
  
  # Rounding off data
  if (norm == "n") {      # no normalization
    datos1 <- round(datos1, 10)
    datos2 <- round(datos2, 10)
  }
  
  
  # Computing k
  if (is.null(k)) {
    m1 <- min(datos1[noceros(datos1, num = FALSE)], na.rm = TRUE)
    m2 <- min(datos2[noceros(datos2, num = FALSE)], na.rm = TRUE)
    k <- min(m1, m2)/2          
  } 
  
  
  # Total counts for each gene:
  suma1 <- rowSums(as.matrix(datos1))
  suma2 <- rowSums(as.matrix(datos2))
  
  # Genes with counts in any condition
  concounts <- names(which(suma1+suma2 > 0))
  
  # All genes
  todos <- rownames(as.matrix(datos1))
  
  
  # Gene length
  long <- 1000
  g.sinL <- NULL  # genes with no length defined
  
  if (!is.null(featureData(input)@data$Length)) {
    g.sinL <- names(which(is.na(featureData(input)@data$Length)))
    if (any(!is.na(featureData(input)@data$Length)) == TRUE) 
      long <- featureData(input)@data[concounts, "Length"]
  }
  
  
  # Genes with counts and with length
  gens.sin0 <- setdiff(concounts, g.sinL)
  
  
  # cond1 and cond2 in the same matrix
  datitos <- cbind(datos1, datos2)  
  datitos <- datitos[concounts,]  # selecting only genes with counts
  
  # Sequencing depth when filtering method = 3
  if (filter == 3 && is.null(depth)) depth = colSums(datitos)
  
  
  #-------------------------------------------------------------------------#
  #-------------------------------------------------------------------------#
  
  
  
  ## Normalization  
  
  if (norm == "n") {       # no normalization
    datitos.0 <- sinceros(datitos, k = k)
    datitos.norm <- datitos.0[gens.sin0, ]
  }
  
  if (norm == "rpkm") {      # RPKM
    datitos.0 <- rpkm(datitos, long = long, k = k, lc = lc)
    datitos.norm <- datitos.0[gens.sin0, ]
  }
  
  if (norm == "uqua") {      # Upper Quartile
    datitos.0 <- uqua(datitos, long = long, lc = lc, k = k)
    datitos.norm <- datitos.0[gens.sin0, ] 
  }
  
  if (norm == "tmm") {
    datitos.0 <- tmm(datitos, long = long, lc = lc, k = k)
    datitos.norm <- datitos.0[gens.sin0, ]       
  }
  
  
  
  #-------------------------------------------------------------------------#
  
  ## Filtering out low count features
  if (filter != 0) {
    datos.filt = filtered.data(dataset = datitos.norm, factor = c(rep("cond1", n1), rep("cond2", n2)), 
                               norm = TRUE, depth = depth, method = filter, cv.cutoff = cv.cutoff, cpm = cpm)    
  } else {
    datos.filt = datitos.norm
  }
  
  datos1.filt <- datos.filt[ ,1:n1]
  datos2.filt <- datos.filt[ ,(n1+1):(n1+n2)]   
  
  
  #-------------------------------------------------------------------------#
  
  
  
  ## Noise distribution
  
  Zr = NULL
  
  if (n1+n2 <= 9) {  # sharing information within clusters
    
    Zr = share.info(mydata = datos.filt, n1 = n1, n2 = n2, r = r, nclust = nclust)
    
  } else {   # r permutations
    
    for (i in 1:r) {
      
      print(paste("r =", i))
      
      mipermu = sample(1:(n1+n2))
            
      mipermu = datos.filt[,mipermu]
      
      mean1 = rowMeans(mipermu[,1:n1])
      mean2 = rowMeans(mipermu[,(n1+1):(n1+n2)])        
      
      sd1 = apply(mipermu[,1:n1], 1, sd)
      sd2 = apply(mipermu[,(n1+1):(n1+n2)], 1, sd)
      
      myparam = list("n" = c(n1,n2), "sd" = cbind(sd1,sd2))
      
      MDperm <- MDbio(dat = cbind(mean1, mean2), param = myparam, a0per = a0per)
      
      Zr = cbind(Zr, myDfunction(mydif = MDperm$D, myrat = MDperm$M, stat = 1, coef = 0.5))
       
    }
  }
  
  
  
  #-------------------------------------------------------------------------#
  
  
  ## Z-score for different experimental conditions (SIGNAL)
  
  mean1 = rowMeans(as.matrix(datos1.filt))
  mean2 = rowMeans(as.matrix(datos2.filt))        
  
  sd1 = apply(as.matrix(datos1.filt), 1, sd)
  sd2 = apply(as.matrix(datos2.filt), 1, sd)
  
  myparam = list("n" = c(n1,n2), "sd" = cbind(sd1,sd2))
  
  MDs <- MDbio(dat = cbind(mean1, mean2), param = myparam, a0per = a0per)
  
  
  Zs = myDfunction(mydif = MDs$D, myrat = MDs$M, stat = 1, coef = 0.5)
    
  
  
    
  
  #-------------------------------------------------------------------------#
  
  
  
  ## Completing M and D (in signal)
  
  lev1 <- mean1[todos]
  lev2 <- mean2[todos]
  names(lev1) <- names(lev2) <- todos
  
  Zs <- as.numeric(Zs)
  names(Zs) <- rownames(MDs$M)
  
  Zs <- Zs[todos]
  names(Zs) <- todos
  
  
  ## Computing Zn
  Zn = as.numeric(Zr)
  
  
  
  #-------------------------------------------------------------------------#
  
  ## Results
  list("k" = k, "comp" = comparison, "Level1" = lev1, "Level2" = lev2, "Zs" = Zs, "Zn" = Zn)
  
}




##############################################################################
##############################################################################




## Function to summarize difference and ratio information (D and D0)

myDfunction <- function (mydif, myrat, stat, coef) {
  
  if (stat == 1) { # linear combination of difference and ratio
    
    myDvalues = coef*mydif + (1-coef)*myrat
    
  }
  
  if (stat == 2) { # distance to origin from (ratio, difference)
    
    myDvalues = sign(mydif) * sqrt((mydif)^2 + (myrat)^2)  
    
  }
  
  myDvalues
  
}




#######################################################################

##################################################################################
##############      Quality Control Report on Expression Data       ##############
##################################################################################


## By Sonia Tarazona
## 25-June-2013
## Modified: 30-March-2015


### Generating data for QC report
##################################


data2report = function(input, factor = NULL, norm = FALSE) {  
    
  ## Biotype detection
  if (!is.null(featureData(input)$Biotype)) {  # BIOTYPES
    mybiotdet = biodetection.dat(input, factor = factor, k = 0); biot.avail = TRUE
    mycountsbio1 = countsbio.dat(input, factor = factor, norm = norm)
  } else {  # NO biotypes
    mybiotdet = NULL; mycountsbio1 = NULL; biot.avail = FALSE
  }
  
  ## Sequencing depth & Expression quantification
  mysat = saturation.dat(input, k = 0, ndepth = 6)
  mycountsbio2 = countsbio.dat(input, factor = factor, norm = norm)
  
  ## Bias detection
  if (!is.null(featureData(input)$Length)) {  # LENGTH
    mylength = length.dat(input, factor = factor, norm = norm); length.avail = TRUE
  } else { mylength = NULL; length.avail = FALSE }
  
  if (!is.null(featureData(input)$GC)) {  # GC
    myGC = GC.dat(input, factor = factor, norm = norm); GC.avail = TRUE
  } else { myGC = NULL; GC.avail = FALSE }
  
  myCD = cd.dat(input, norm = norm, refColumn = 1)
  
  
  ## PCA
  myPCA = PCA.dat(input, norm = norm, logtransf = FALSE)
  
  
  list("data" = list("biodet" = mybiotdet, "countsbiot" = mycountsbio1, "saturation" = mysat, 
                     "countsampl" = mycountsbio2, "length" = mylength, "GC" = myGC, "countdist" = myCD, "PCA" = myPCA),
       "parameters" = list("biotypes" = biot.avail, "length" = length.avail, "GC" = GC.avail))
  
}





##################################################################################


### Generating QC report
##################################



QCreport = function (input, file = NULL, samples = NULL, factor = NULL, norm = FALSE) {

  if (is.null(file))
	file <- paste("QCreport", format(Sys.time(), "_%Y%b%d_%H_%M_%S"), ".pdf", sep = "")  

  QCinfo = data2report(input = input, factor = factor, norm = norm)
  
  samples2 = colnames(QCinfo$data$countdist$data2plot)
    
  
  # NO factor
  if (is.null(factor)) {    
    if (length(samples) != 2) {
      stop("ERROR: Factor was not specified and the number of samples to be plotted is not equal to 2.\n
           Please, either indicate the factor or the two samples to be plotted.\n")
    } else { 
      niveles = NULL 
      if (is.numeric(samples)) { samples = colnames(QCinfo$data$countdist$data2plot)[samples] }
    }
    
    
  # FACTOR  
  } else {    
    myfactor = as.factor(pData(input)[,factor])
    niveles = as.character(unique(myfactor))
    
    if (length(niveles) > 2) {  # more than two levels
      if (length(samples) != 2) {
        stop("ERROR: The factor has more than two levels (conditions).\n
           Please, specify which two conditions are to be plotted.\n")
      } else {
        if (is.numeric(samples)) { samples = colnames(QCinfo$data$countsampl$result)[samples] }
        niveles = samples
      }
    } 
    
    if (length(niveles) == 2) {  # 2 samples
      samples = niveles
    }
  }  
  
    
  
  pdf(file, paper = "a4", width = 8.27, height = 11.69)
  
  
  # Page 0
  layout(matrix(c(1,2,3), nrow = 3, ncol = 1, byrow = TRUE), heights = c(25,15,60))
  
  par(mar = c(0,0,0,0))
  ## TITLE
  plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
  text(5,4, "Quality Control of Expression Data", adj = 0.5, cex = 3, col = "brown3", font = 2)
  text(5,1, paste("Generated by NOISeq on", format(Sys.time(), "%d %b %Y, %H:%M:%S")), 
       adj = 0.5, font = 3, cex = 1.5)
  
  ## SUBTITLE
  plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
  text(2,3, "Content", adj = 0.5, font = 2, cex = 2, col = "dodgerblue4")
  
  ## Content
  par(mar = c(0,3,0,3))
  lugares = c(1,3)
  plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
  
  text(lugares[1], 10, "Plot", adj = 0, font = 3, cex = 1)
  text(lugares[2], 10, "Description", adj = 0, font = 3, cex = 1)
  abline(h = 9.8, lty = 2, col = "grey")
  
  empiezo = 9.3
  bajo = 0.5
  text(lugares[1], empiezo, "Biotype detection", adj = 0, font = 2, cex = 1)
  if (QCinfo$parameters$biotypes) {
    text(lugares[2], empiezo, "Biotype abundance in the genome with %genes detected (counts > 0) in the sample/condition.", 
         adj = 0, font = 1, cex = 1)
    text(lugares[2], empiezo-bajo/2, "Biotype abundance within the sample/condition.", 
         adj = 0, font = 1, cex = 1)
  } else {
    text(lugares[2], empiezo, "Plot not available. Biotypes information was not provided.", 
         adj = 0, font = 1, cex = 1)
  }  
      
  empiezo = empiezo-bajo-0.3
  text(lugares[1], empiezo, "Biotype expression", adj = 0, font = 2, cex = 1)
  if (QCinfo$parameters$biotypes) {
    text(lugares[2], empiezo, "Distribution of gene counts per million per biotype in sample/condition (only genes with counts > 0).", 
         adj = 0, font = 1, cex = 1)
  } else {
    text(lugares[2], empiezo, "Plot not available. Biotypes information was not provided.", 
         adj = 0, font = 1, cex = 1)
  }    
                     
  empiezo = empiezo-bajo
  text(lugares[1], empiezo, "Saturation", adj = 0, font = 2, cex = 1)
  text(lugares[2], empiezo, "Number of detected genes (counts > 0) per sample across different sequencing depths", adj = 0, font = 1, cex = 1)
  
  empiezo = empiezo-bajo
  text(lugares[1], empiezo, "Expression boxplot", adj = 0, font = 2, cex = 1)
  text(lugares[2], empiezo, "Distribution of gene counts per million (all biotypes) in each sample/condition", adj = 0, font = 1, cex = 1)
  
  empiezo = empiezo-bajo
  text(lugares[1], empiezo, "Expression barplot", adj = 0, font = 2, cex = 1)
  text(lugares[2], empiezo, "Percentage of genes with >0, >1, >2, >5 or >10 counts per million in each sample/condition.", adj = 0, font = 1, cex = 1)
  
  empiezo = empiezo-bajo
  text(lugares[1], empiezo, "Length bias", adj = 0, font = 2, cex = 1)
  if (QCinfo$parameters$length) {
    text(lugares[2], empiezo, "Mean gene expression per each length bin. Fitted curve and diagnostic test.", 
         adj = 0, font = 1, cex = 1)
  } else {
    text(lugares[2], empiezo, "Plot not available. Gene length was not provided.", 
         adj = 0, font = 1, cex = 1)
  }
  
  empiezo = empiezo-bajo
  text(lugares[1], empiezo, "GC content bias", adj = 0, font = 2, cex = 1)
  if (QCinfo$parameters$GC) {
    text(lugares[2], empiezo, "Mean gene expression per each GC content bin. Fitted curve and diagnostic test.", 
         adj = 0, font = 1, cex = 1)
  } else {
    text(lugares[2], empiezo, "Plot not available. Gene GC content was not provided.", 
         adj = 0, font = 1, cex = 1)
  }
  
  empiezo = empiezo-bajo
  text(lugares[1], empiezo, "RNA composition bias", adj = 0, font = 2, cex = 1)
  text(lugares[2], empiezo, "Density plots of log fold changes (M) between pairs of samples.", adj = 0, font = 1, cex = 1)
  text(lugares[2], empiezo-bajo/2, "Confidence intervals for the median of M values.", adj = 0, font = 1, cex = 1)

  empiezo = empiezo-bajo-0.3
  text(lugares[1], empiezo, "Exploratory PCA", adj = 0, font = 2, cex = 1)
  text(lugares[2], empiezo, "Principal Component Analysis score plots for PC1 vs PC2, and PC1 vs PC3.", adj = 0, font = 1, cex = 1)

  
  
  
  # Page 1 (only if biotypes are provided)
  if (QCinfo$parameters$biotypes) {
    layout(matrix(c(1,1,2,3,4,5), nrow = 3, ncol = 2, byrow = TRUE), heights = c(10,45,45))
        
    ## SUBTITLE
    par(mar = c(0,0,0,0))
    plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
    text(5,6, "Biotype detection", adj = 0.5, font = 2, cex = 2, col = "dodgerblue4")
    
    ## BIODETECTION PLOTS
    biodetection.plot(QCinfo$data$biodet, samples = samples, plottype = "comparison", toreport = TRUE)
    countsbio.plot(QCinfo$data$countsbiot, toplot = "global", samples = samples[1], plottype = "boxplot",
                   ylim = range(log2(1+QCinfo$data$countsbiot$result)), toreport = TRUE)  
    countsbio.plot(QCinfo$data$countsbiot, toplot = "global", samples = samples[2], plottype = "boxplot",
                   ylim = range(log2(1+QCinfo$data$countsbiot$result)), toreport = TRUE)  
  }
  
  
  
  # Page 2
  #layout(matrix(c(1,2,3,8,4,9,1,5,6,8,7,9), nrow = 6, ncol = 2, byrow = FALSE), heights = c(10,35,10,5,35,5))
  layout(matrix(c(1,2,3,8,4,1,5,6,8,7), nrow = 5, ncol = 2, byrow = FALSE), heights = c(10,35,10,5,40))
  
  par(mar = c(0,0,0,0))
  ## SUBTITLE
  plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
  text(5,5, "Sequencing depth & Expression quantification", adj = 0.5, font = 2, cex = 2, col = "dodgerblue4")
  
  ## SEQUENCING DEPTH AND EXPRESSION QUANTIFICATION PLOTS 
  if (is.null(niveles)) {  # NO factor
    saturation.plot(QCinfo$data$saturation, samples = samples, toplot = 1, 
                    yleftlim = c(0,unlist(QCinfo$data$saturation$bionum[1])), toreport = TRUE)    
  } else {   # FACTOR
    par(mar = c(5.1,4.1,4.1,2.1))
    saturation.plot(QCinfo$data$saturation, samples = samples2[myfactor == niveles[1]], 
                    toplot = 1, toreport = TRUE, 
                    yleftlim = c(0,unlist(QCinfo$data$saturation$bionum[1])), )
    if (sum(myfactor == niveles[1]) > 2) {
      par(mar = c(0,0,0,0))
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
    }    
  } 
  
  par(mar = c(0,0,0,0))
  countsbio.plot(QCinfo$data$countsampl, toplot = "global", samples = samples, plottype = "boxplot",
                 toreport = TRUE)
  
  if (is.null(niveles)) {  # NO factor
    par(mar = c(0,0,0,0))
    plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
    plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
    
  } else {  # FACTOR
    saturation.plot(QCinfo$data$saturation, samples = samples2[myfactor == niveles[2]], 
                    toplot = 1, yleftlim = c(0,unlist(QCinfo$data$saturation$bionum[1])), toreport = TRUE)
    if (sum(myfactor == niveles[2]) > 2) {
      par(mar = c(0,0,0,0))
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
    }  
  }  
  
  countsbio.plot(QCinfo$data$countsampl, toplot = "global", samples = samples, plottype = "barplot",
                 toreport = TRUE)
                 
  
  
  
  #####  BIAS DETECTION
  QQ = 0.05
  
  # Page 3
  layout(matrix(c(1,1,2,2,3,4,5,5,6,7), nrow = 5, ncol = 2, byrow = TRUE), heights = c(10,10,35,10,35))
  
  par(mar = c(0,0,0,0))
  plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
  text(5,5, "Sequencing bias detection", adj = 0.5, font = 2, cex = 2, col = "dodgerblue4")
  
  if (QCinfo$parameters$length) {   ## LENGTH
    
    if (QCinfo$parameters$GC) {  ## LENGTH & GC      
      
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for feature length bias", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      laF = lapply(QCinfo$data$length$RegressionModels[samples],
                   function (x) summary(x)$"fstatistic")
      pvalores = sapply(laF, function (x) pf(x[1], df1 = x[2], df2 = x[3], lower.tail = FALSE))
      misR2 = sapply(QCinfo$data$length$RegressionModels[samples],
                     function (x) summary(x)$"r.squared")
      
      if (min(pvalores) < QQ) { 
        if (max(misR2) > 0.7) {
          text(5,5, "FAILED. At least one of the model p-values was lower than 0.05 and R2 > 70%.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting length bias is recommended.", adj = 0.5, font = 1, cex = 1)
        } else {
          text(5,5, "WARNING. At least one of the model p-values was lower than 0.05, but R2 < 70% for at least one condition.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting length bias could be advisable.",               
               adj = 0.5, font = 1, cex = 1)
          text(5,2, "Plese check in the plots below the strength of the relationship between length and expression.", 
               adj = 0.5, font = 1, cex = 1)
        }
        
      } else {
        text(5,4, "PASSED. No normalization for correcting length bias is required.", adj = 0.5, font = 1, cex = 1)
      }
      
      
      par(mar = c(5.1,4.1,4.1,2.1))
      
      length.plot(QCinfo$data$length, toreport = TRUE, samples = samples)
      
      par(mar = c(0,0,0,0))
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for GC content bias", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      laF = lapply(QCinfo$data$GC$RegressionModels[samples],
                   function (x) summary(x)$"fstatistic")
      pvalores = sapply(laF, function (x) pf(x[1], df1 = x[2], df2 = x[3], lower.tail = FALSE))
      misR2 = sapply(QCinfo$data$GC$RegressionModels[samples],
                     function (x) summary(x)$"r.squared")
      
      if (min(pvalores) < QQ) { 
        if (max(misR2) > 0.7) {
          text(5,5, "FAILED. At least one of the model p-values was lower than 0.05 and R2 > 70%.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting GC content bias is recommended.", adj = 0.5, font = 1, cex = 1)
        } else {
          text(5,5, "WARNING. At least one of the model p-values was lower than 0.05, but R2 < 70% for at least one condition.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting GC content bias could be advisable.",               
               adj = 0.5, font = 1, cex = 1)
          text(5,2, "Plese check in the plots below the strength of the relationship between GC content and expression.", 
               adj = 0.5, font = 1, cex = 1)
        }
        
      } else {
        text(5,4, "PASSED. No normalization for correcting GC content bias is required.", adj = 0.5, font = 1, cex = 1)
      }
      
      par(mar = c(5.1,4.1,4.1,2.1))
      GC.plot(QCinfo$data$GC, toreport = TRUE, samples = samples)      
      
      
      # Page 4
      layout(matrix(c(1,1,2,3,4,4), nrow = 3, ncol = 2, byrow = TRUE), heights = c(10,40,45))
      
      par(mar = c(0,0,0,0))
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for differences in RNA composition", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      if ("FAILED" %in% QCinfo$data$countdist$DiagnosticTest[,"Diagnostic Test"]) {
        text(5,5, "FAILED. There is a pair of samples with significantly different RNA composition", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is required.", adj = 0.5, font = 1, cex = 1)
      } else {
        text(5,5, "PASSED. The pairs of compared samples do not present significant differences in RNA composition.", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is NOT required.", adj = 0.5, font = 1, cex = 1)
      }
      
      
      par(mar = c(5.1,4.1,4.1,2.1))
      if (length(samples2) < 14) {
        cd.plot(QCinfo$data$countdist, samples = samples2) 
      } else {
        cd.plot(QCinfo$data$countdist, samples = setdiff(samples2, QCinfo$data$countdist$refColumn)[1:12]) 
      }
      
      
      par(mar = c(0,0,0,0))
      lugares = c(1,4.5,6.5,8.5)
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(lugares[1],10, "Confidence intervals for median of M values", adj = 0, font = 2, cex = 1.2)
      text(lugares[1], 9.4, "Sample", adj = 0, font = 1, cex = 1)
      abline(h = 9.2, lty = 2, col = "grey")
      for (j in 1:3) {
        text(lugares[j+1] , 9.4, colnames(QCinfo$data$countdist$DiagnosticTest)[j], adj = 0, font = 1, cex = 1)   
        
        for (i in 1:min(30,nrow(QCinfo$data$countdist$DiagnosticTest))) {
          
          if (j == 1) text(lugares[j], 9.2-i*0.3, rownames(QCinfo$data$countdist$DiagnosticTest)[i], 
                         adj = 0, font = 1, cex = 1)
          if (j < 3) text(lugares[j+1], 9.2-i*0.3, adj = 1, font = 1, cex = 1,
                          round(as.numeric(QCinfo$data$countdist$DiagnosticTest[i,j]),4))
          if (j == 3) text(lugares[j+1], 9.2-i*0.3, adj = 0, font = 1, cex = 1,
                           QCinfo$data$countdist$DiagnosticTest[i,j]) 
        }        
      }
      
      if (nrow(QCinfo$data$countdist$DiagnosticTest) > 30) {
        print("WARNING: In Diagnostic Test for RNA composition, the confidence intervals are shown for only 
              the first 30 samples.")
      }
      
      
      
          
      
    } else {  ## LENGTH & NO GC
      
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for feature length bias", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      laF = lapply(QCinfo$data$length$RegressionModels[samples],
                   function (x) summary(x)$"fstatistic")
      pvalores = sapply(laF, function (x) pf(x[1], df1 = x[2], df2 = x[3], lower.tail = FALSE))
      misR2 = sapply(QCinfo$data$length$RegressionModels[samples],
                     function (x) summary(x)$"r.squared")
      
      if (min(pvalores) < QQ) { 
        if (max(misR2) > 0.7) {
          text(5,5, "FAILED. At least one of the model p-values was lower than 0.05 and R2 > 70%.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting length bias is recommended.", adj = 0.5, font = 1, cex = 1)
        } else {
          text(5,5, "WARNING. At least one of the model p-values was lower than 0.05, but R2 < 70% for at least one condition.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting length bias could be advisable.",               
               adj = 0.5, font = 1, cex = 1)
          text(5,2, "Plese check in the plots below the strength of the relationship between length and expression.", 
               adj = 0.5, font = 1, cex = 1)
        }
        
      } else {
        text(5,4, "PASSED. No normalization for correcting length bias is required.", adj = 0.5, font = 1, cex = 1)
      }
      
      
      par(mar = c(5.1,4.1,4.1,2.1))
      
      length.plot(QCinfo$data$length, toreport = TRUE, samples = samples)
      
      
      par(mar = c(0,0,0,0))
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for differences in RNA composition", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      if ("FAILED" %in% QCinfo$data$countdist$DiagnosticTest[,"Diagnostic Test"]) {
        text(5,5, "FAILED. There is a pair of samples with significantly different RNA composition", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is required.", adj = 0.5, font = 1, cex = 1)
      } else {
        text(5,5, "PASSED. The pairs of compared samples do not present significant differences in RNA composition.", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is NOT required.", adj = 0.5, font = 1, cex = 1)
      }
      
      
      
      par(mar = c(5.1,4.1,4.1,2.1))
      if (length(samples2) < 14) {
        cd.plot(QCinfo$data$countdist, samples = samples2) 
      } else {
        cd.plot(QCinfo$data$countdist, samples = setdiff(samples2, QCinfo$data$countdist$refColumn)[1:12]) 
      }
      
      
      
      par(mar = c(0,0,0,0))
      lugares = c(1,4.5,6.5,8.5)
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(lugares[1],10, "Confidence intervals for median of M values", adj = 0, font = 2, cex = 1.2)
      text(lugares[1], 9.4, "Sample", adj = 0, font = 1, cex = 1)
      abline(h = 9.2, lty = 2, col = "grey")
      for (j in 1:3) {
        text(lugares[j+1] , 9.4, colnames(QCinfo$data$countdist$DiagnosticTest)[j], adj = 0, font = 1, cex = 1)   
        
        for (i in 1:min(30,nrow(QCinfo$data$countdist$DiagnosticTest))) {
          
          if (j == 1) text(lugares[j], 9.2-i*0.3, rownames(QCinfo$data$countdist$DiagnosticTest)[i], 
                           adj = 0, font = 1, cex = 1)
          if (j < 3) text(lugares[j+1], 9.2-i*0.3, adj = 1, font = 1, cex = 1,
                          round(as.numeric(QCinfo$data$countdist$DiagnosticTest[i,j]),4))
          if (j == 3) text(lugares[j+1], 9.2-i*0.3, adj = 0, font = 1, cex = 1,
                           QCinfo$data$countdist$DiagnosticTest[i,j]) 
        }        
      }
      
      if (nrow(QCinfo$data$countdist$DiagnosticTest) > 30) {
        print("WARNING: In Diagnostic Test for RNA composition, the confidence intervals are shown for only 
              the first 30 samples.")
      }
            
 
      
    }    
    
  } else {  ## NO LENGTH
    
    if (QCinfo$parameters$GC) {   ## NO LENGTH & GC
            
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for GC content bias", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      laF = lapply(QCinfo$data$GC$RegressionModels[samples],
                   function (x) summary(x)$"fstatistic")
      pvalores = sapply(laF, function (x) pf(x[1], df1 = x[2], df2 = x[3], lower.tail = FALSE))
      misR2 = sapply(QCinfo$data$GC$RegressionModels[samples],
                     function (x) summary(x)$"r.squared")
      
      if (min(pvalores) < QQ) { 
        if (max(misR2) > 0.7) {
          text(5,5, "FAILED. At least one of the model p-values was lower than 0.05 and R2 > 70%.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting GC content bias is recommended.", adj = 0.5, font = 1, cex = 1)
        } else {
          text(5,5, "WARNING. At least one of the model p-values was lower than 0.05, but R2 < 70% for at least one condition.", 
               adj = 0.5, font = 1, cex = 1)
          text(5,3, "Normalization for correcting GC content bias could be advisable.",               
               adj = 0.5, font = 1, cex = 1)
          text(5,2, "Plese check in the plots below the strength of the relationship between GC content and expression.", 
               adj = 0.5, font = 1, cex = 1)
        }
        
      } else {
        text(5,4, "PASSED. No normalization for correcting GC content bias is required.", adj = 0.5, font = 1, cex = 1)
      }
      
      par(mar = c(5.1,4.1,4.1,2.1))
      GC.plot(QCinfo$data$GC, toreport = TRUE, samples = samples)      
      
            
      par(mar = c(0,0,0,0))
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for differences in RNA composition", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      if ("FAILED" %in% QCinfo$data$countdist$DiagnosticTest[,"Diagnostic Test"]) {
        text(5,5, "FAILED. There is a pair of samples with significantly different RNA composition", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is required.", adj = 0.5, font = 1, cex = 1)
      } else {
        text(5,5, "PASSED. The pairs of compared samples do not present significant differences in RNA composition.", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is NOT required.", adj = 0.5, font = 1, cex = 1)
      }
      
      
      
      par(mar = c(5.1,4.1,4.1,2.1))
      if (length(samples2) < 14) {
        cd.plot(QCinfo$data$countdist, samples = samples2) 
      } else {
        cd.plot(QCinfo$data$countdist, samples = setdiff(samples2, QCinfo$data$countdist$refColumn)[1:12]) 
      }
      
      
      
      par(mar = c(0,0,0,0))
      lugares = c(1,4.5,6.5,8.5)
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(lugares[1],10, "Confidence intervals for median of M values", adj = 0, font = 2, cex = 1.2)
      text(lugares[1], 9.4, "Sample", adj = 0, font = 1, cex = 1)
      abline(h = 9.2, lty = 2, col = "grey")
      for (j in 1:3) {
        text(lugares[j+1] , 9.4, colnames(QCinfo$data$countdist$DiagnosticTest)[j], adj = 0, font = 1, cex = 1)   
        
        for (i in 1:min(30,nrow(QCinfo$data$countdist$DiagnosticTest))) {
          
          if (j == 1) text(lugares[j], 9.2-i*0.3, rownames(QCinfo$data$countdist$DiagnosticTest)[i], 
                           adj = 0, font = 1, cex = 1)
          if (j < 3) text(lugares[j+1], 9.2-i*0.3, adj = 1, font = 1, cex = 1,
                          round(as.numeric(QCinfo$data$countdist$DiagnosticTest[i,j]),4))
          if (j == 3) text(lugares[j+1], 9.2-i*0.3, adj = 0, font = 1, cex = 1,
                           QCinfo$data$countdist$DiagnosticTest[i,j]) 
        }        
      }
      
      if (nrow(QCinfo$data$countdist$DiagnosticTest) > 30) {
        print("WARNING: In Diagnostic Test for RNA composition, the confidence intervals are shown for only 
              the first 30 samples.")
      }
      
      
    } else {  ## NO LENGTH & NO GC
      
      par(mar = c(0,0,0,0))
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(5,8, "Diagnostic plot for differences in RNA composition", adj = 0.5, font = 4, cex = 1.5, col = "aquamarine4")
      
      if ("FAILED" %in% QCinfo$data$countdist$DiagnosticTest[,"Diagnostic Test"]) {
        text(5,5, "FAILED. There is a pair of samples with significantly different RNA composition", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is required.", adj = 0.5, font = 1, cex = 1)
      } else {
        text(5,5, "PASSED. The pairs of compared samples do not present significant differences in RNA composition.", 
             adj = 0.5, font = 1, cex = 1)
        text(5,3, "Normalization for correcting this bias is NOT required.", adj = 0.5, font = 1, cex = 1)
      }
      
      
      
      par(mar = c(5.1,4.1,4.1,2.1))
      if (length(samples2) < 14) {
        cd.plot(QCinfo$data$countdist, samples = samples2) 
      } else {
        cd.plot(QCinfo$data$countdist, samples = setdiff(samples2, QCinfo$data$countdist$refColumn)[1:12]) 
      }
      
      
      
      par(mar = c(0,0,0,0))
      lugares = c(1,4.5,6.5,8.5)
      plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
      text(lugares[1],10, "Confidence intervals for median of M values", adj = 0, font = 2, cex = 1.2)
      text(lugares[1], 9.4, "Sample", adj = 0, font = 1, cex = 1)
      abline(h = 9.2, lty = 2, col = "grey")
      for (j in 1:3) {
        text(lugares[j+1] , 9.4, colnames(QCinfo$data$countdist$DiagnosticTest)[j], adj = 0, font = 1, cex = 1)   
        
        for (i in 1:min(30,nrow(QCinfo$data$countdist$DiagnosticTest))) {
          
          if (j == 1) text(lugares[j], 9.2-i*0.3, rownames(QCinfo$data$countdist$DiagnosticTest)[i], 
                           adj = 0, font = 1, cex = 1)
          if (j < 3) text(lugares[j+1], 9.2-i*0.3, adj = 1, font = 1, cex = 1,
                          round(as.numeric(QCinfo$data$countdist$DiagnosticTest[i,j]),4))
          if (j == 3) text(lugares[j+1], 9.2-i*0.3, adj = 0, font = 1, cex = 1,
                           QCinfo$data$countdist$DiagnosticTest[i,j]) 
        }        
      }
      
      if (nrow(QCinfo$data$countdist$DiagnosticTest) > 30) {
        print("WARNING: In Diagnostic Test for RNA composition, the confidence intervals are shown for only 
              the first 30 samples.")
      }
      
      
    }
    
    
  }
  
  
  # Last page (for PCA)
  
  # Page 4
  layout(matrix(c(1,1,2,3,4,4), nrow = 3, ncol = 2, byrow = TRUE), heights = c(30,40,40))
  
  par(mar = c(0,0,0,0))
  plot(1:10, 1:10, type = "n", axes = FALSE, xlab = "", ylab = "")
  text(5,6, "Exploratory PCA", adj = 0.5, font = 2, cex = 2, col = "dodgerblue4")
  text(5,4, "Use this plot to see if samples are clustered according to the experimental design.", 
       adj = 0.5, font = 1, cex = 1)
  text(5,3, "Use ARSyNseq function to correct potential batch effects.", adj = 0.5, font = 1, cex = 1)
  
  if (is.null(factor)) factor = colnames(QCinfo$data$PCA$factors)[1]
    
  par(mar = c(5.1,4.1,4.1,2.1))
  PCA.plot(QCinfo$data$PCA, samples = 1:2, factor = factor) 
  
  par(mar = c(5.1,4.1,4.1,2.1))
  PCA.plot(QCinfo$data$PCA, samples = c(1,3), factor = factor) 

  #*****#
  
  dev.off()
  
}
  
  

ASCA.1f<-function(X = X,Designa = Designa,Designb=NULL,Designc=NULL,Fac=c(1,2),Join=NULL,Interaction=NULL)

{
#--------------------------------------------------------------------------------------
#  Dimensions of the matrices: 
#  X (p x n) contains expression values of n genes (in columns) and p conditions (in rows)
#  Designa (p x I) contains 0's and 1's for the TIME-POINTS in the experiment
#  Designres (p x H) INDIVIDUALS

#  Join = TRUE if the analyses of the model b and ab is studied jointly
#  Interaction = TRUE to consider interaction "ab" in the separated model

n<-ncol(X)
p<-nrow(X)
I<-ncol(Designa)

Faca=Fac[1] # number components Model a (time)
Facres=Fac[2] # number components Residues

#----------------------- Calculate Overall Mean --------------------------------------

offset<-apply(X,2,mean)
Xoff<-X-(cbind(matrix(1,nrow=p,ncol=1))%*%rbind(offset))


#-----------------------  PART I: Submodel a (TIME) -----------------------------------

	Model.a<-ASCAfun1(Xoff,Designa,Faca)
	Xres<-Xoff-Model.a$X


# ------------------------Collecting models ------------------------------------------

models <- ls(pattern="Model")
output <- vector(mode="list")
Xres <- Xoff
for (i in 1: length(models)) {
	mymodel <- get(models[i], envir=environment())
	output <- c(output, list(mymodel))
	Xres <- Xres - mymodel$X
	rm(mymodel)
	gc()
	
}
names(output) <- models

#------------------------- PART III: Submodel res -----------------------------------

Model.res<-ASCAfun.res(Xres,Facres)

Model.res<-list(Model.res)
names(Model.res)<-c("Model.res")
output<-c(output,Model.res)


#------------------------- Add Input data to the Output ----------------------------

Input<-list(X, Designa, Designb, Designc, Fac, Join,Interaction)
names(Input)<-c("X", "Designa", "Designb", "Designc", "Fac", "Join","Interaction")
Input<-list(Input)
names(Input)<-"Input"
output<-c(output,Input)

output 
}

readData <-
function (data = NULL, factors = NULL, length = NULL, biotype = NULL, chromosome = NULL, gc = NULL) {

  if (is.null(data))
    stop("Expression information must be provided to the readData function")

  if (is.null(factors))
    stop("Condition information must be provided to the readData funcion")

  if (is.null(length) == FALSE && is.vector(length) == FALSE && is.data.frame(length) == FALSE && is.matrix(length) == FALSE)
    stop( "The length info should be a vector or a data.frame/matrix.")

  if (is.null(gc) == FALSE && is.vector(gc) == FALSE && is.data.frame(gc) == FALSE && is.matrix(gc) == FALSE)
    stop( "The GC content info should be a vector or a data.frame/matrix.")
  
  if (is.null(chromosome) == FALSE && ncol(chromosome) != 3)
    stop( "The chromosome object should be a matrix or data.frame with 3 columns: chromosome, start position and end position.")

  if (is.null(biotype) == FALSE && is.vector(biotype) == FALSE && is.data.frame(biotype) == FALSE && is.matrix(biotype) == FALSE)
    stop( "The biotype info should be a vector or a data.frame/matrix.")

  countData <- as.matrix( data )

  rowNames <- rownames(countData)
  
  if (nrow(factors) == ncol(countData)) {
    rownames(factors) <- colnames(countData)
  } else {
    stop ("Number of rows in factors must be equal to number of columns in data.\n")
  }
  
     
  pheno <- AnnotatedDataFrame(data=as.data.frame(factors))

  input <- ExpressionSet(
               assayData = countData,
               phenoData = pheno)

  if (!is.null(length))
    input <- addData(data = input, length = length)

  if (!is.null(gc))
    input <- addData(data = input, gc = gc)

  if (!is.null(biotype))
    input <- addData(data = input, biotype = biotype)

  if (!is.null(chromosome))
    input <- addData(data = input, chromosome = chromosome)

  input    

}


######################################################
######################################################
######################################################


addData <- function(data, length = NULL, biotype = NULL, chromosome = NULL, factors = NULL, gc = NULL) {

  if (inherits(data,"eSet") == FALSE)
    stop("Error. You must give an eSet object.")
  
  if (is.null(length) == FALSE && is.vector(length) == FALSE && is.data.frame(length) == FALSE && is.matrix(length) == FALSE)
    stop( "The length info should be a vector or a data.frame/matrix.")  

  if (is.null(gc) == FALSE && is.vector(gc) == FALSE && is.data.frame(gc) == FALSE && is.matrix(gc) == FALSE)
    stop( "The GC content info should be a vector or a data.frame/matrix.")
  
  if (is.null(biotype) == FALSE && is.vector(biotype) == FALSE && is.data.frame(biotype) == FALSE && is.matrix(biotype) == FALSE)
    stop( "The biotype info should be a vector or a data.frame/matrix.")
  
  if (is.null(chromosome) == FALSE && ncol(chromosome) != 3)
    stop( "The chromosome object should be a matrix or data.frame with 3 columns: chromosome, start position and end position.")

  if (!is.null(assayData(data)$exprs))
    rowNames <- rownames(assayData(data)$exprs)
  else
    rowNames <- rownames(assayData(data)$counts)

  # If exists length
  if (!is.null(length)) {
    Length <- rep(NA,length(rowNames))
    names(Length) <- rowNames
    if (is.vector(length)) {
        Length[rowNames] <- as.numeric(as.character(length[rowNames]))
    } else if (is.data.frame(length) || is.matrix(length)) {
        if (ncol(length) == 2) {
            # We assume that the feature names are in the first column and the length in the second
            rownames(length) <- length[,1]
            Length[rowNames] <- as.numeric(as.character( length[rowNames,2]  ))
        } else if (ncol(length) == 1) {
            # We assume that the length are in the first column and the feature names in the rownames
            Length[rowNames] <- as.numeric(as.character( length[rowNames,1]  ))
        } else {
            stop( "The length matrix/data.frame contains more columns than expected.")
        }
    }
    
    featureData(data)@data <- cbind(featureData(data)@data, Length)
  }

  # If exists gc
  if (!is.null(gc)) {
    GC <- rep(NA,length(rowNames))
    names(GC) <- rowNames
    if (is.vector(gc)) {
        GC[rowNames] <- as.numeric(as.character(gc[rowNames]))
    } else if (is.data.frame(gc) || is.matrix(gc)) {
        if (ncol(gc) == 2) {
            # We assume that the feature names are in the first column and the GC content in the second
            rownames(gc) <- gc[,1]
            GC[rowNames] <- as.numeric(as.character( gc[rowNames,2]  ))
        } else if (ncol(gc) == 1) {
            # We assume that the GC contents are in the first column and the feature names in the rownames
            GC[rowNames] <- as.numeric(as.character( gc[rowNames,1]  ))
        } else {
            stop( "The GC matrix/data.frame contains more columns than expected.")
        }
    }
    
    featureData(data)@data <- cbind(featureData(data)@data, GC)
  }
  
  # If exists biotype
  if (!is.null(biotype)) {
    Biotype <- rep(NA,length(rowNames))
    names(Biotype) <- rowNames
    if (is.vector(biotype)) {
        Biotype[rowNames] <- as.character(biotype[rowNames])
    } else if (is.data.frame(biotype) || is.matrix(biotype)) {
        if (ncol(biotype) == 2) {
            # We assume that the feature names are in the first column and the biotypes in the second
            rownames(biotype) <- biotype[,1]
            Biotype[rowNames] <- as.character( biotype[rowNames,2]  )
        } else if (ncol(biotype) == 1) {
            # We assume that the biotypes are in the first column and the feature names in the rownames
            Biotype[rowNames] <- as.character( biotype[rowNames,1]  )
        } else {
            stop( "The biotype matrix/data.frame contains more columns than expected.")
        }
    }
    
    featureData(data)@data <- cbind(featureData(data)@data, Biotype)
    featureData(data)@data$Biotype <- as.character(featureData(data)@data$Biotype)
  } 
  
  # If exists chromosome
  if (!is.null(chromosome)) {

    Chromosome <- GeneStart <- GeneEnd <- rep(NA,length(rowNames))
    names(Chromosome) <- names(GeneStart) <- names(GeneEnd) <- rowNames
    
    Chromosome[rowNames] <- as.character(chromosome[rowNames,1])
    GeneStart[rowNames] <- as.numeric(as.character(chromosome[rowNames,2]))
    GeneEnd[rowNames] <- as.numeric(as.character(chromosome[rowNames,3]))
        
    featureData(data)@data <- cbind(featureData(data)@data, Chromosome, GeneStart, GeneEnd)
  }

  # If exists new factors
  if (!is.null(factors))
    phenoData(data)@data <- cbind(phenoData(data)@data, factors)
  
  data
  
}

ASCAfun2<-function (X,Desa,Desb,Fac) {

n <- ncol(X) # number of genes
I <- ncol(Desa) # number of levels in the factor TIME
J <- ncol(Desb) # number of levels in the other factor

XK1<-matrix(NA,nrow=I,ncol=n)

for (i in 1:I) {
     sub<-X[Desa[,i]==1,]
 if(is.null(nrow(sub))){ #when there isn't replicates
    XK1[i,] <- sub 
}else{
     XK1[i,]<-apply(sub,2,mean) }
}

XK2<-matrix(NA,nrow=J,ncol=n)

for (j in 1:J) {
     sub<-X[Desb[,j]==1,]
 if(is.null(nrow(sub))){ #when there isn't replicates
    XK2[j,] <- sub 
}else{
     XK2[j,]<-apply(sub,2,mean) }
}

NK<-matrix(NA,nrow=I,ncol=J)
XK<-matrix(NA,nrow=I*J,ncol=n)

k=1
for (j in 1:J){
  for (i in 1:I){
    sub<-X[(Desa[,i]+Desb[,j])==2,]
if(is.null(nrow(sub))){  #when there isn't replicates
    NK[i,j] <- 1
    XK[k,] <- sub-XK1[i,]-XK2[j,]
}else{
    NK[i,j]<-sqrt(nrow(sub))
    XK[k,]<-apply(sub,2,mean)-XK1[i,]-XK2[j,] }
    k=k+1
  }
}

XKw<-XK*(as.numeric(NK))

PCA<-PCA.GENES(XKw)
      scw<-PCA$scores[,1:Fac]
      ld<-PCA$loadings[,1:Fac]
      ssq<-PCA$var.exp
      if(Fac==1) {
      scw<-as.matrix(scw)
      ld<-as.matrix(ld) 
      }      
 if(Fac==0) {
      scw<-as.matrix(rep(0,I*J))
      ld<-as.matrix(rep(0,n))
	}           
# Re-weigth the scores
sc<-scw/(as.numeric(NK))
    
XKrec<-sc%*%t(ld)

Xab<-NULL
TPab<-NULL
for (i in 1:nrow(X)){
     position1<-which(Desa[i,]==1)
     position2<-which(Desb[i,]==1)
     Xab<-rbind(Xab,XK[I*(position2-1)+position1,])
     TPab<-rbind(TPab,XKrec[I*(position2-1)+position1,])
}
Eab<-Xab-TPab

 leverage<-apply(ld^2,1,sum)
 SPE<-apply(Eab^2,2,sum)

output<-list(XK,sc,ld,ssq,Xab,TPab,Eab,leverage,SPE)
names(output)<-c("data","scores","loadings","var.exp","X","TP","E","leverage","SPE")
output


}

setClass("Biodetection", representation(dat="list"))
setClass("CD", representation(dat="list"))
setClass("CountsBio", representation(dat="list"))
setClass("GCbias", representation(dat="list"))
setClass("lengthbias", representation(dat="list"))
setClass("Saturation", representation(dat="list"))
setClass("PCA", representation(dat="list"))

setGeneric("explo.plot", function(object, ...) standardGeneric("explo.plot"))

setMethod("explo.plot", "Biodetection", function(object, samples = c(1,2), plottype = c("persample", "comparison"), 
                                                 toplot = "protein_coding", ...) 
  biodetection.plot(object@dat, samples = samples, plottype = plottype, toplot = toplot, ...))

setMethod("explo.plot", "CD", function(object, samples = NULL, ...) cd.plot(object@dat, samples = samples, ...))


setMethod("explo.plot", "CountsBio", function(object, samples = c(1,2), toplot = "global", plottype = c("barplot", "boxplot"),...)
          countsbio.plot(object@dat, samples, toplot, plottype, ...))

setMethod("explo.plot", "GCbias", function(object, samples = NULL, toplot = "global", ...)
          GC.plot(object@dat, samples = samples, toplot = toplot, ...))

setMethod("explo.plot", "lengthbias", function(object, samples = NULL, toplot = "global", ...)
          length.plot(object@dat, samples = samples, toplot = toplot, ...))

setMethod("explo.plot", "Saturation",
          function(object, samples = NULL, toplot = 1, yleftlim = NULL, yrightlim = NULL, ...)
          saturation.plot(object@dat, samples = samples, toplot = toplot, yleftlim = yleftlim, yrightlim = yrightlim, ...))

setMethod("explo.plot", "PCA", function(object, samples = 1:2, plottype = "scores", factor = NULL)
  PCA.plot(object@dat, samples = samples, plottype = plottype, factor = factor))



# Show methods for exploration objects
setMethod("show", "Biodetection",
    function(object) {
      cat("\n Reference genome: \n==========\n")
      names(dimnames(object@dat$genome)) = NULL
      print(object@dat$genome)
      for (i in c(1:length(object@dat$biotables))) {
        cat("\n",names(object@dat$biotables)[i],"\n==========\n")
        print(object@dat$biotables[[i]])
      }
    })
           

setMethod("show", "CD",
    function(object) {      
      cat("\n Confidence intervals for median of M to compare each sample to reference:\n=======\n")
      print(object@dat$DiagnosticTest)
      cat("\n Reference sample is:\n=======\n")
      print(object@dat$refColumn)
    })

setMethod("show", "CountsBio",
          function(object) {            
              cat("\n Summary: \n============\n")              
              print(object@dat$summary[[1]])            
          })


setMethod("show","GCbias",
          function(object) {
            x <- object@dat$RegressionModels
            for (i in 1:length(x)) {
              print(names(x)[i])
              print(summary(x[[i]]))
            }            
          })


setMethod("show","lengthbias",
          function(object) {
            x <- object@dat$RegressionModels
            for (i in 1:length(x)) {
              print(names(x)[i])
              print(summary(x[[i]]))
            }            
          })

setMethod("show","Saturation",
          function(object) {
            x <- dat2save(object)
            cat("\n Number of detected features at each sequencing depth: \n============\n") 
            for (i in 1:length(x)) {
              print(names(x)[i])
              print(x[[i]])
            }            
          })


setMethod("show","PCA",
          function(object) {
            x <- object$result$var.exp
            x = round(x*100,4)
            colnames(x) = c("%Var", "Cum %Var")
            rownames(x) = paste("PC", 1:nrow(x))
            cat("\n Percentage of total variance explained by each component: \n============\n") 
            print(x)                        
          })



# Coercion methods for exploration objects

setGeneric("dat2save", function(object)  standardGeneric("dat2save"))

setMethod("dat2save","Biodetection", function(object) object@dat)

setMethod("dat2save","CD", function(object) object@dat)

setMethod("dat2save","CountsBio", function(object)  object@dat$summary)

setMethod("dat2save","GCbias", function(object) object@dat$data2plot)

setMethod("dat2save","lengthbias", function(object) object@dat$data2plot)

setMethod("dat2save","Saturation", function(object) {
  
  muestras = vector("list", length = length(object@dat$depth))
  names(muestras) = names(object@dat$depth)
  
  for (i in 1:length(muestras)) {
    
    muestras[[i]] = object@dat$depth[[i]]

    for (j in 1:length(object@dat$saturation)) {
      muestras[[i]] = cbind(muestras[[i]], object@dat$saturation[[j]][[i]])
    }

    colnames(muestras[[i]]) <- c("depth", names(object@dat$saturation))
  }

  muestras  
})


setMethod("dat2save","PCA", function(object) object@dat$result)


############################################################################
############################# OUTPUT OBJECT ################################
############################################################################

setClass("myInfo",representation(method="character", k="numeric", lc="numeric", factor="vector",
                  v="numeric",nss="numeric",pnr="numeric",comparison="vector",replicates="character"))
setClass("Output",representation(results="list"),contains="myInfo")

setValidity("Output",
    function(object) {
        if (!(is.character(object@method))) {
          return(paste("Method must be a string"))
        } else if (!(is.numeric(object@k))) {
          return(paste("k must be numeric"))
        } else if (!(is.numeric(object@lc))) {
          return(paste("lc must be numeric"))
        } else if (!(is.vector(object@factor))) {
          return(paste("Factor must be a vector of strings"))
        } else if (!(is.numeric(object@v))) {
          return(paste("v must be numeric"))
        } else if (!(is.numeric(object@nss))) {
          return(paste("nss must be numeric"))
        } else if (!(is.numeric(object@pnr))) {
          return(paste("pnr must be numeric"))
        } else if (!(is.vector(object@comparison))) {
          return(paste("Comparison must be a vector of strings"))
        } else if (!(is.list(object@results))) {
          return(paste("Results must be a list of data.frames"))
        } else {
          return(TRUE)
        }
    })

Output <-
function (data, method, k, lc, factor, v, nss, pnr, comparison, replicates) {
  
  new("Output",results=data, method = method, k = k, lc = lc, factor = factor, v = v, nss = nss, 
      pnr = pnr, comparison = comparison, replicates = replicates)
}

setMethod("show", "Output",
    function(object) {
      
      if (object@method == "n")
        object@method = "none"
      
      for (i in 1:length(object@results)) {
        cat("\nSummary",i,"\n=========\n")
        cat("\nYou are comparing",object@comparison[i],"from", object@factor[i], "\n\n")
        print(head(object@results[[i]][order(object@results[[i]][,5], decreasing = TRUE),]))
      }
      cat("\nNormalization\n")
      cat("\tmethod:", object@method, "\n")
      cat("\tk:", object@k, "\n")
      cat("\tlc:", object@lc, "\n")
      
      # Simulated samples
      if (object@replicates == "no") {
        cat("\nYou are working with simulated replicates:\n")
        cat("\tpnr:",object@pnr,"\n")
        cat("\tnss:",object@nss,"\n")
        cat("\tv:",object@v,"\n")
      } 
      # With biological or technical replicates
      else {
        cat("\nYou are working with",object@replicates, "replicates\n")
      }
           
})

\documentclass[10pt]{article}
\usepackage[a4paper,left=1.9cm,top=1.9cm,bottom=2.5cm,right=1.9cm,ignoreheadfoot]{geometry}
\usepackage{cite}
%\topmargin 0in
%\headheight 0in
%\headsep 0in
%\oddsidemargin 0in
%\evensidemargin 0in
%\textwidth 176mm
%\textheight 215mm
\usepackage[numbers]{natbib}
\usepackage{amsmath}
\usepackage{amssymb}
\usepackage{Sweave}
\SweaveOpts{keep.source=FALSE,eps=FALSE,pdf=TRUE,png=FALSE,include=FALSE,concordance=TRUE}
\usepackage{url}
\usepackage[utf8]{inputenc}
%\DeclareGraphicsRule{.tif}{png}{.png}{`convert #1 `dirname #1`/`basename #1 .tif`.png}
\newcommand{\noiseq}{\textsf{NOISeq}}
\newcommand{\noiseqbio}{\textsf{NOISeqBIO}}
\newcommand{\code}[1]{{\small\texttt{#1}}}
\newcommand{\R}{\textsf{R}}

\begin{document}

%\VignetteIndexEntry{NOISeq User's Guide}


\title{\noiseq: Differential Expression in \textsf{RNA-seq}}
\author{Sonia Tarazona (\texttt{starazona@cipf.es})\\Pedro Furi\'{o}-Tar\'{i} (\texttt{pfurio@cipf.es})\\
Mar\'{i}a Jos\'{e} Nueda (\texttt{mj.nueda@ua.es})\\Alberto Ferrer (\texttt{aferrer@eio.upv.es})\\Ana Conesa (\texttt{aconesa@cipf.es})}
% Please increment date when working on this document, so that
% date shows genuine change date, not merely date of compile.
\date{11 February 2016 \\(Version 2.14.1)}
\maketitle


\tableofcontents

\clearpage

<<options,results=hide,echo=FALSE>>=
options(digits=3, width=95)
@

\section{Introduction}

This document will guide you through to the use of the \R{} Bioconductor package \noiseq{},
for analyzing count data coming from next generation sequencing technologies. 
\noiseq{} package consists of three modules: (1) Quality control of count data; (2) Normalization and low-count filtering; and (3) Differential expression analysis.

First, we describe the input data format. Next, we illustrate the utilities to explore the quality of the count data: saturation, biases, 
contamination, etc. and show the normalization, filtering and batch correction methods included in the package. Finally, we explain how to compute differential 
expression between two experimental conditions. The differential expression method \noiseq{} and some of the plots included in the package were displayed in \cite{tarazona2011,tarazona2015}.The new version of \noiseq{} for biological replicates (\noiseqbio{}) is also implemented in the package.

The \noiseq{} and \noiseqbio{} methods are data-adaptive and nonparametric. 
Therefore, no distributional assumptions need to be done for the data
and differential expression analysis may be carried on for both raw counts or previously normalized or transformed datasets. 

We will use the ``reduced'' Marioni's dataset \cite{marioni2008} as an example throughout this document. In Marioni's experiment, 
human kidney and liver RNA-seq samples were sequenced. There are 5 technical replicates per tissue, 
and samples were sequenced in two different runs. 
We selected chromosomes I to IV from the original data and removed genes with 0 counts in all samples and with no length information available. 
Note that this reduced dataset is only used to decrease the computing time while testing the examples. We strongly recommend to use the whole set of features (e.g. the whole genome) in real analysis.

The example dataset can be obtained by typing:

<<data>>=
library(NOISeq)
data(Marioni)
@  



\vspace{1cm}

\section{Input data}

\noiseq{} requires two pieces of information to work that must be provided to the \code{readData} function: the expression data (\texttt{data}) 
and the factors defining the experimental groups to be studied or compared (\texttt{factors}). However, in order to perform the quality control of the data or normalize them, other additional annotations need to be provided such as the feature length, 
the GC content, the biological classification of the features (e.g. Ensembl biotypes), or the chromosome position of each feature.


\subsection{Expression data}

The expression data must be provided in a matrix or a data.frame R object, having as many rows as the number of features to be studied and as many columns as the number of
samples in the experiment. The following example shows part of the count data for Marioni's dataset:
<<>>=
head(mycounts)
@ 

The expression data can be both read counts or normalized expression data such as RPKM values, and also any other normalized expression values.


\subsection{Factors}
Factors are the variables indicating the experimental group for each sample. They must be given to the \code{readData} function in a data frame 
object. This data frame must have as many rows as samples (columns in data object) and as many columns or factors as different sample annotations the user wants to use. For instance, in Marioni's data, we have the factor ``Tissue'', but we can also define another factors (``Run'' or ``TissueRun''). The levels of the factor ``Tissue'' are ``Kidney'' and ``Liver''. The factor ``Run'' has two levels: ``R1'' and ``R2''. The factor ``TissueRun'' combines the sequencing run with the tissue and hence has four levels: ``Kidney\_1'', ``Liver\_1'', ``Kidney\_2'' and ``Liver\_2''. 

Be careful here, the order of the elements of the factor must coincide with the order of the samples (columns) in the expression data file provided.

<<factors>>=
myfactors = data.frame(Tissue=c("Kidney","Liver","Kidney","Liver","Liver","Kidney","Liver",
                                "Kidney","Liver","Kidney"),
                       TissueRun = c("Kidney_1","Liver_1","Kidney_1","Liver_1","Liver_1",
                                     "Kidney_1","Liver_1","Kidney_2","Liver_2","Kidney_2"),
                       Run = c(rep("R1", 7), rep("R2", 3)))
myfactors
@ 


\subsection{Additional biological annotation}

Some of the exploratory plots in \noiseq{} package require additional biological information such as feature length, GC content, biological 
classification of features, or chromosome position. You need to provide at least part of this information if you want to either generate 
the corresponding plots or apply a normalization method that corrects by length.

The following code show how the R objects containing such information should look like:

<<>>=
head(mylength)
head(mygc)
head(mybiotypes)
head(mychroms)
@ 

Please note, that these objects might contain a different number of features and in different order than the expression data. 
However, it is important to specify the names or IDs of the features in each case so the package can properly match all this information. 
The length, GC content or biological groups (e.g. biotypes), could be vectors, matrices or data.frames. 
If they are vectors, the names of the vector must be the feature names or
IDs. If they are matrices or data.frame objects, the feature names or IDs must be in the row names of the object. 
The same applies for chromosome position, which is also a matrix or data.frame.

Ensembl Biomart data base provides these annotations for a wide range of species: biotypes (the biological classification of the features),
GC content, or chromosome position. The latter can be used to estimate the length of the feature. However, it is more accurate computing the length
from the GTF or GFF annotation file so the introns are not considered.




\subsection{Converting data into a \noiseq{} object}

Once we have created in R the count data matrix, the data frame for the factors and the biological annotation objects (if needed), we have to pack all this information into a \noiseq{} object by using the \code{readData} function. An example on how it works is shown below: 

<<readData>>=
mydata <- readData(data=mycounts,length=mylength, gc=mygc, biotype=mybiotypes,
                   chromosome=mychroms, factors=myfactors)
mydata
@ 

The \code{readData} function returns an object of \emph{Biobase's eSet} class. To see which information is included in this object, type for instance:

<<results=hide>>=
str(mydata)
head(assayData(mydata)$exprs)
head(pData(mydata))
head(featureData(mydata)@data)
@ 

Note that the features to be used by all the methods in the package will be those in the data expression object. 
If any of this features has not been included in the additional biological annotation (when provided), the corresponding value will be NA.


It is possible to add information to an existing object. For instance, \code{noiseq} function accepts objects generated 
while using other packages such as \code{DESeq} package.
In that case, annotations may not be included in the object. The \code{addData} function allows the user to add annotation data to the object. 
For instance, if you generated the data object like this:
<<readData2>>=
mydata <- readData(data=mycounts,chromosome=mychroms, factors=myfactors)
@

And now you want to include the length and the biotypes, you have to use the \code{addData} function:

<<readData3>>=
mydata <- addData(mydata, length=mylength, biotype=mybiotypes, gc = mygc)
@


\textbf{IMPORTANT}: Some packages such as \emph{ShortRead} also use the \code{readData} function but with different input object and parameters. 
Therefore, some incompatibilities may occur that cause errors. To avoid this problem when loading simultaneously packages with functions with the same
name but different use, the following command can be used: \code{NOISeq::readData} instead of simply \code{readData}.



\vspace{1cm}



% \clearpage

\section{Quality control of count data}

Data processing and sequencing experiment design in RNA-seq are not straightforward. From the
biological samples to the expression quantification, there are many steps in which errors may be produced, despite of the many procedures 
developed to reduce noise at each one of these steps and to control the quality of the generated data.
Therefore, once the expression levels (read counts) have been obtained, it is absolutely
necessary to be able to detect potential biases or contamination before proceeding with further analysis (e.g. differential expression). 
The technology biases, such as the transcript length, GC content, PCR artifacts, uneven transcript read coverage, contamination by off-target transcripts or big differences in transcript distributions, are factors that interfere in the linear relationship between transcript
abundance and the number of mapped reads at a gene locus (counts).

In this section, we present a set of plots to explore the count data that may be helpful to detect these potential biases 
so an appropriate normalization procedure can be chosen. For instance, these plots will be useful for seeing which kind of features 
(e.g. genes) are being detected in our RNA-seq samples and with how many counts, which technical biases are present, etc. 

As it will be seen at the end of this section, it is also possible to generate a report in a PDF file including 
all these exploratory plots for the comparison of two samples or two experimental conditions.



\subsection{Generating data for exploratory plots}

There are several types of exploratory plots that can be obtained. They will be described in detail in the following sections. 
To generate any of these plots, first of all, \code{dat} function must be applied on the input data (\noiseq{} object) 
to obtain the information to be plotted. 
The user must specify the type of plot the data are to be computed for (argument \code{type}). 
Once the data for the plot have been generated with \code{dat} function, the plot will be drawn with the \emph{explo.plot} function. 
Therefore, for the quality control plots, we will always proceed like in the following example:

<<dat>>=
myexplodata <- dat(mydata, type = "biodetection")
explo.plot(myexplodata, plottype = "persample")
@


To save the data in a user-friendly format, the \code{dat2save} function can be used:

<<nicedata>>=
mynicedata <- dat2save(myexplodata)
@


We have grouped the exploratory plots in three categories according to the different questions that may arise during the quality control of the expression data:

\begin{itemize}
 \item \textbf{Biotype detection}: Which kind of features are being detected? Is there any abnormal contamination in the data? Did I choose an appropriate protocol?
 \item \textbf{Sequencing depth \& Expression Quantification}: Would it be better to increase the sequencing depth to detect more features? 
Are there too many features with low counts? Are the samples very different regarding the expression quantification?
 \item \textbf{Sequencing bias detection}: Should the expression values be corrected for the length or the GC content bias? 
Should a normalization procedure be applied to account for the differences among RNA composition among samples?
 \item \textbf{Batch effect exploration}: Are the samples clustered in concordance with the experimental design or with the batch in which they were processed? 
\end{itemize}




\subsection{Biotype detection}

When a biological classification of the features is provided (e.g. Ensembl biotypes), the following plots are useful to see 
which kind of features are being detected.
For instance, in RNA-seq, it is expected that most of the genes will be protein-coding so detecting an enrichment in the sample of any other biotype could point to a potential contamination or at least provide information on the sample composition to take decision on the type of analysis to be performed.

\subsubsection{Biodetection plot}

The example below shows how to use the \code{dat} and \code{explo.plot} functions to generate the data to be plotted and to draw a biodetection plot per sample. 

<<fig_biodetection,fig=TRUE,width=12>>=
mybiodetection <- dat(mydata, k = 0, type = "biodetection", factor = NULL)
par(mfrow = c(1,2))  # we need this instruction because two plots (one per sample) will be generated
explo.plot(mybiodetection, samples=c(1,2), plottype = "persample")
@ 


Fig. \ref{fig_biodetection} shows the ``biodetection" plot per sample. The gray bar corresponds to the percentage of each biotype in the genome 
(i.e. in the whole set of features provided), the stripped color bar is the proportion detected in our sample 
(with number of counts higher than \texttt{k}), and the solid color bar is the percentage of each biotype within the sample.
The vertical green line separates the most abundant biotypes (in the left-hand side, corresponding to the left axis scale) 
from the rest (in the right-hand side, corresponding to the right axis scale).

When \texttt{factor=NULL}, the data for the plot are computed separately for each sample. If \texttt{factor} is a string indicating the name of one of the columns in the factor object, the samples are aggregated within each of these experimental conditions and the data for the plot are computed per condition. In this example, samples in columns 1 and 2 from expression data are plotted and the features (genes) are considered to be detected if having a number of counts higher than \texttt{k=0}.


\begin{figure}[ht!]
\centering
\includegraphics[width=0.9\textwidth]{NOISeq-fig_biodetection}
\caption{Biodetection plot (per sample)}
\label{fig_biodetection}
\end{figure}


When two samples or conditions are to be compared, it can be more practical to represent both o them in the same plot. Then, two different plots can be generated: one representing the percentage of each biotype in the genome being detected in the sample, and other representing the relative abundance of each biotype within the sample. The following code can be used to obtain such plots:

<<fig_biodetection2,fig=TRUE,width=12>>=
par(mfrow = c(1,2))  # we need this instruction because two plots (one per sample) will be generated
explo.plot(mybiodetection, samples=c(1,2), toplot = "protein_coding", plottype = "comparison")
@ 


\begin{figure}[ht!]
\centering
\includegraphics[width=0.9\textwidth]{NOISeq-fig_biodetection2}
\caption{Biodetection plot (comparison of two samples)}
\label{fig_biodetection2}
\end{figure}

In addition, the ``biotype comparison'' plot also performs a proportion test for the chosen biotype (argument \texttt{toplot}) to test if the relative abundance of that biotype is different in the two samples or conditions compared.




\subsubsection{Count distribution per biotype}

The ``countsbio" plot (Fig. \ref{fig_boxplot1}) per biotype allows to see how the counts are distributed within each biological group. In the upper side of the plot, the number of detected features that will be represented in the boxplots is displayed. The values used for the boxplots are either the counts per million (if \texttt{norm = FALSE}) or the values provided by the use (if \texttt{norm = TRUE})
The following code was used to draw the figure. Again, data are computed per sample because no factor was specified 
(\texttt{factor=NULL}).
To obtain this plot using the \emph{explo.plot} function and the ``countsbio" data, we have to indicate the ``boxplot" type in the \texttt{plottype} argument, choose only one of the samples (\texttt{samples = 1}, in this case), and all the biotypes (by setting \code{toplot} parameter to 1 or "global"). 

<<fig_boxplot1,fig=TRUE,width=10>>=
mycountsbio = dat(mydata, factor = NULL, type = "countsbio")
explo.plot(mycountsbio, toplot = 1, samples = 1, plottype = "boxplot")
@ 

\begin{figure}[ht!]
\centering
\includegraphics[width=\textwidth]{NOISeq-fig_boxplot1}
\caption{Count distribution per biotype in one of the samples (for genes with more than 0 counts). At the upper part of the plot, the number of detected features within each biotype group is displayed.}
\label{fig_boxplot1}
\end{figure}




% \clearpage

\subsection{Sequencing depth \& Expression Quantification}

The plots in this section can be generated by only providing the expression data, since no other biological information is required. 
Their purpose is to assess if the sequencing depth of the samples is enough to detect the features of interest and to get a good 
quantification of their expression.


\subsubsection{Saturation plot}

The ``Saturation" plot shows the number of features in the genome detected with more than \texttt{k} counts with the sequencing depth of the sample, and with higher and lower simulated sequencing depths.  
This plot can be generated by considering either all the features or only the features included in a given biological group (biotype), if this information is available.
First, we have to generate the saturation data with the function \code{dat} and then we can use the resulting object to obtain, 
for instance, the plots in Fig. \ref{fig_sat1} and \ref{fig_sat2} by applying \code{explo.plot} function. The lines show how the number of detected features increases with depth. When the number of samples to plot is 1 or 2, 
bars indicating the number of new features detected when increasing the sequencing depth in one million of reads are also drawn. In that case, lines values are to be read in the left Y axis and bar values in the right Y axis. 
If more than 2 samples are to be plotted, it is difficult to visualize the ``newdetection bars'', 
so only the lines are shown in the plot.

<<fig_sat1,fig=TRUE>>=
mysaturation = dat(mydata, k = 0, ndepth = 7, type = "saturation")
explo.plot(mysaturation, toplot = 1, samples = 1:2, yleftlim = NULL, yrightlim = NULL)
@
<<fig_sat2,fig=TRUE>>=
explo.plot(mysaturation, toplot = "protein_coding", samples = 1:4)
@ 

The plot in Fig. \ref{fig_sat1} has been computed for all the features (without specifying a biotype) and for two of the samples. 
Left Y axis shows the number of detected genes with more than 0 counts at each sequencing depth, represented by the lines. 
The solid point in each line corresponds to the real available sequencing depth. The other sequencing depths are simulated 
from this total sequencing depth. The bars are associated to the right Y axis and show the number of new features detected per million of new sequenced reads
at each sequencing depth. The legend in the gray box also indicates the percentage of total features detected with more than $k=0$ counts 
at the real sequencing depth.

Up to twelve samples can be displayed in this plot. In Fig. \ref{fig_sat2}, four samples are compared and we can see, for instance, 
that in kidney samples the number of detected features is higher than in liver samples.  

\begin{figure}[ht!]
\centering
\includegraphics[width=0.5\textwidth]{NOISeq-fig_sat1}
\caption{Global saturation plot to compare two samples of kidney and liver, respectively.}
\label{fig_sat1}
\end{figure}


\begin{figure}[ht!]
\centering
\includegraphics[width=0.5\textwidth]{NOISeq-fig_sat2}
\caption{Saturation plot for protein-coding genes to compare 4 samples: 2 of kidney and 2 of liver.}
\label{fig_sat2}
\end{figure}


\subsubsection{Count distribution per sample}

It is also interesting to visualize the count distribution for all the samples, either for all the features or for the features belonging to a certain biological group (biotype). Fig. \ref{fig_boxplot2} shows this information for the biotype ``protein\_coding", which can be generated with the following code on the ``countsbio" object obtained in the previous section by setting the \texttt{samples} parameter to  \texttt{NULL}.
<<fig_boxplot2,fig=TRUE>>=
explo.plot(mycountsbio, toplot = "protein_coding", samples = NULL, plottype = "boxplot")
@ 

\begin{figure}[ht!]
\centering
\includegraphics[width=0.45\textwidth]{NOISeq-fig_boxplot2}
\caption{Distribution of counts for protein coding genes in all samples.}
\label{fig_boxplot2}
\end{figure}



\subsubsection{Sensitivity plot}

Features with low counts are, in general, less reliable and may introduce noise in the data that makes more difficult to extract the relevant information, 
for instance, the differentially expressed features. We have implemented some methods in the \noiseq{} package to filter out these low count features. The ``Sensitivity plot'' in Fig. \ref{fig_boxplot3} helps to decide the threshold to remove low-count features by indicating the proportion of such features that are present in our data. In this plot, the bars show the percentage of features within each sample having more than 0 counts per million (CPM), or more than 1, 2, 5 and 10 CPM. The horizontal lines are the corresponding percentage of features with those CPM in at least one of the samples (or experimental conditions if the \texttt{factor} parameter is not \texttt{NULL}). In the upper side of the plot, the sequencing depth of each sample (in million reads) is given. 
The following code can be used for drawing this figure.
 
<<fig_boxplot3,fig=TRUE>>=
explo.plot(mycountsbio, toplot = 1, samples = NULL, plottype = "barplot")
@ 

\begin{figure}[ht!]
\centering
\includegraphics[width=0.45\textwidth]{NOISeq-fig_boxplot3}
\caption{Number of features with low counts for each sample.}
\label{fig_boxplot3}
\end{figure}


% \clearpage


\subsection{Sequencing bias detection}

Prior to perform further analyses such as differential expression, it is essential to normalize data to make the samples comparable and 
remove the effect of technical biases from the expression estimation. The plots presented in this section are very useful for detecting the possible biases in
the data. In particular, the biases that can be studied are: the feature length effect, the GC content and the differences in RNA composition. 
In addition, these are diagnostic plots, which means that they are not only descriptive but an statistical test is also conducted to help the user to decide whether the bias is present and the data needs normalization.


\subsubsection{Length bias}

The ``lengthbias" plot describes the relationship between the feature length and the expression values. 
Hence, the feature length must be included in the input object created using the \code{readData} function. The data for this plot is generated as follows. 
The length is divided in intervals (bins) containing 200 features and the middle point of each bin is depicted in X axis. 
For each bin, the 5\% trimmed mean of the corresponding expression values (CPM if \texttt{norm=FALSE} or values provided if \texttt{norm=TRUE}) is computed and depicted in Y axis. If the number of samples or conditions 
to appear in the plot is 2 or less and no biotype is specified (toplot = ``global"), a diagnostic test is provided. 
A cubic spline regression model is fitted to explain the relationship between length and expression. 
Both the model p-value and the coefficient of determination (R2) are shown in the plot as well as the fitted regression curve. 
If the model p-value is significant and R2 value is high (more than 70\%), the expression depends on the feature length and the curve shows the type of dependence. 
 
Fig. \ref{fig_length} shows an example of this plot. In this case, the ``lengthbias" data were generated for each condition (kidney and liver) using the argument \texttt{factor}.

<<fig_length,results=hide,fig=TRUE>>=
mylengthbias = dat(mydata, factor = "Tissue", type = "lengthbias")
explo.plot(mylengthbias, samples = NULL, toplot = "global")
@ 

\begin{figure}[ht]
\centering
\includegraphics[width=\textwidth, height=0.5\textwidth]{NOISeq-fig_length}
\caption{Gene length versus expression.}
\label{fig_length}
\end{figure}

More details about the fitted spline regression models can be obtained by using the \code{show} function as per below:

<<showmodels>>=
show(mylengthbias)
@ 




\subsubsection{GC content bias}

The ``GCbias" plot describes the relationship between the feature GC content and the expression values. Hence, the feature GC
content must be included in the input object created using the \code{readData} function. 
The data for this plot is generated in an analogous way to the ``lengthbias" data. 
The GC content is divided in intervals (bins) containing 200 features. The middle point of each bin is
depicted in X axis. For each bin, the 5\% trimmed mean of the corresponding expression values is
computed and depicted in Y axis. If the number of samples or conditions to appear in the plot is 2 or less and no biotype is specified 
(toplot = ``global"), a diagnostic test is provided. A cubic spline regression model is fitted to explain the relationship between GC content and expression. 
Both the model p-value and the coefficient of determination (R2) are shown in the plot as well as the fitted regression curve. 
If the model p-value is significant and R2 value is high (more than 70\%), the expression will depend on the feature GC content 
and the curve will show the type of dependence. 
 
An example of this plot is in Fig. \ref{fig_GC}. In this case, the ``GCbias" data were also generated for each condition 
(kidney and liver) using the argument \texttt{factor}.

<<fig_GC,results=hide,fig=TRUE>>=
myGCbias = dat(mydata, factor = "Tissue", type = "GCbias")
explo.plot(myGCbias, samples = NULL, toplot = "global")
@ 

\begin{figure}[ht]
\centering
\includegraphics[width=\textwidth, height=0.5\textwidth]{NOISeq-fig_GC}
\caption{Gene GC content versus expression.}
\label{fig_GC}
\end{figure}




\subsubsection{RNA composition}

When two samples have different RNA composition, the distribution of sequencing reads across the features is different in such a way that although 
a feature had the same number of read counts in both samples, it would not mean that it was equally expressed in both. 
To check if this bias is present in the data, the ``cd" plot and the correponding diagnostic test can be used. In this case, each sample $s$ is compared to the reference sample $r$ (which can be arbitrarily chosen). To do that, M values are computed as $log2(counts_s=counts_r)$. 
If no bias is present, it should be expected that the median of M values for each comparison is 0. Otherwise, it would
be indicating that expression levels in one of the samples tend to be higher than in the other, and this could lead to false discoveries 
when computing differencial expression. Confidence intervals for the M median are also computed by bootstrapping. If value 0 does not fall inside the interval, 
it means that the deviation of the sample with regard to the reference sample is statistically significant. 
Therefore, a normalization procedure such as Upper Quartile, TMM or DESeq should be used to correct this effect and make the samples comparable before computing differential expression. Confidence intervals can be visualized by using \texttt{show} function. 

See below an usage example and the resulting plot in Fig. \ref{fig_countdistr}. It must be indicated if the data provided are already normalized (\texttt{norm=TRUE}) or not (\texttt{norm=FALSE}). The reference sample may be indicated with the refColumn parameter (by default, the first column is used). 
Additional plot parameters may also be used to modify some aspects of the plot.
<<fig_countdistr,fig=TRUE>>=
mycd = dat(mydata, type = "cd", norm = FALSE, refColumn = 1)
explo.plot(mycd)
@ 

\begin{figure}[ht]
\centering
\includegraphics[width=0.5\textwidth]{NOISeq-fig_countdistr}
\caption{RNA composition plot}
\label{fig_countdistr}
\end{figure}

In the plot can be seen that the $M$ median is deviated from 0 in most of the cases. This is corraborated by the confidence intervals for the $M$ median.

% \clearpage


\subsection{PCA exploration} \label{sec_PCA}

One of the techniques that can be used to visualize if the experimental samples are clustered according to the experimental design or if there is an unwanted source of noise in the data that hampers this clustering is the Principal Component Analysis (PCA).

PCA is a dimension reduction method that does not require any distributional assumption, but it usually works better if data distribution is not too skewed, as happens in RNA-seq data. This is why, NOISeq package log-tranforms the expression data when users indicate that they have not already been log-tranformed. 

NOISeq PCA function allows to plot the loading values, that is, the projection of the genes on the new principal components, or the scores, which are the projections of the samples (observations) on the space created by the new componets. 

To illustrate the utility of the PCA plots, we took Marioni's data and artificially added a batch effect to the first four samples that would belong then to bath 1. The rest of samples would belong to batch2, so we also create an additional factor to collect the batch information.

<<randomBatchEffect>>=
set.seed(123)
mycounts2 = mycounts
mycounts2[,1:4] = mycounts2[,1:4] + runif(nrow(mycounts2)*4, 3, 5)
myfactors = data.frame(myfactors, "batch" = c(rep(1,4), rep(2,6)))
mydata2 = readData(mycounts2, factors = myfactors)
@

Now we can run the following code to plot the samples scores for the two principal components of the PCA and color them by the factor ``Tissue'' (left hand plot) or by the factor ``batch'' (right hand plot): 

<<fig_PCA,fig=TRUE>>=
myPCA = dat(mydata2, type = "PCA")
par(mfrow = c(1,2))
explo.plot(myPCA, factor = "Tissue")
explo.plot(myPCA, factor = "batch")
@

\begin{figure}[ht]
\centering
\includegraphics[width=\textwidth, height=0.5\textwidth]{NOISeq-fig_PCA}
\caption{PCA plot colored by tissue (left) and by batch (right)}
\label{fig_PCA}
\end{figure}

We can appreciate in these plots that the two batches are quite separated so removing the batch effect should improve the clustering of the samples. More information on how to do that with \noiseq{} can be found in Section \ref{sec_batch}.



\subsection{Quality Control report}

The \code{QCreport} function allows the user to quickly generate a pdf report showing the exploratory plots described in this section 
to compare either two samples (if \texttt{factor=NULL}) or two experimental conditions (if \texttt{factor} is indicated). 
Depending on the biological information provided (biotypes, length or GC content), the number of plots included in the report may differ. 

<<QCreportExample,results=hide>>=
QCreport(mydata, samples = NULL, factor = "Tissue", norm = FALSE)
@ 

This report can be generated before normalizing the data (\texttt{norm = FALSE}) or after normalization to check if unwanted effects were corrected (\texttt{norm = TRUE}).

Please note that the data are log-transformed when computing Principal Component Analysis (PCA).



\vspace{1cm}


\section{Normalization, Low-count filtering \& Batch effect correction}

The normalization step is very important in order to make the samples comparable and to remove possibles biases in the data. 
It might also be useful to filter out low expression data prior to differential expression analysis, since they are less reliable 
and may introduce noise in the analysis.

Next sections explain how to use \noiseq{} package to normalize and filter data before performing any statistical analysis.


\subsection{Normalization} \label{sec_norm}

We strongly recommend to normalize the counts to correct, at least, sequencing depth bias. 
The normalization techniques implemented in \noiseq{} are RPKM \cite{Mortazavi2008},
Upper Quartile \cite{Bullard2010} and TMM, which stands for Trimmed Mean of M values \cite{Robinson2010}, but the package accepts data normalized with any other method as well as data previously transformed to remove batch effects or to reduce noise. 

The normalization functions (\code{rpkm}, \code{tmm} and \code{uqua}) can be applied to common R matrix and data frame objects. Please, find below some examples on how to apply them to data matrix extracted from \noiseq{} data objects:

<<normalization>>=
myRPKM = rpkm(assayData(mydata)$exprs, long = mylength, k = 0, lc = 1)
myUQUA = uqua(assayData(mydata)$exprs, long = mylength, lc = 0.5, k = 0)
myTMM = tmm(assayData(mydata)$exprs, long = 1000, lc = 0)
head(myRPKM[,1:4])
@ 

If the length of the features is provided to any of the normalization functions, the expression values are divided by 
$(length/1000)^{lc}$. Thus, although Upper Quartile and TMM methods themselves do not correct for the length of the features, 
\noiseq{} allows the users to combine these normalization procedures with an additional length correction whenever the length information is available. If $lc = 0$, no length correction is applied.
To obtain RPKM values, $lc = 1$ in \code{rpkm} function must be indicated. If $long = 1000$ in \code{rpkm} function, CPM values (counts per million) are returned.   

The $k$ parameter is used to replace the zero values in the expression matrix with other non-zero value in order to avoid indetermination in some calculations such as fold-change. If $k=NULL$, each 0 is replaced with the midpoint between 0 and the next non-zero value in the matrix. 



\subsection{Low-count filtering} \label{sec_filt}

Excluding features with low counts improves, in general, differential expression results, no matter the method being used,
since noise in the data is reduced. However, the best procedure
to filter these low count features has not been yet decided nor
implemented in the differential expression packages. \noiseq{} includes three methods to filter out features with low
counts:
\begin{itemize}
\item \textbf{CPM} (method 1): The user chooses a value for the parameter counts per million (CPM) in a sample under which a feature is
considered to have low counts. The cutoff for a condition with $s$ samples is
$CPM \times s$. Features with sum of expression values below the condition cutoff in all
conditions are removed. Also a cutoff for the coefficient of variation (in percentage) per condition
may be established to eliminate features with inconsistent expression values.
\item \textbf{Wilcoxon test} (method 2): For each feature and condition,  $H_0: m=0$ is tested versus $H_1: m>0$, where $m$ is the median of counts per condition. 
Features with p-value $> 0.05$ in all conditions are filtered out. P-values can be corrected for multiple testing using the \texttt{p.adj} option. This method is only recommended when the number of replicates per condition is at least 5.
\item \textbf{Proportion test} (method 3): Similar procedure to the Wilcoxon test but testing $H_0: p=p_0$ versus $H_1: p>p_0$, where $p$ is the 
feature relative expression and $p_0 = CPM/10^6$. Features with p-value $> 0.05$ in all conditions are filtered out. P-values can be corrected for multiple testing using the \texttt{p.adj} option.
\end{itemize}

This is an usage example of function \code{filtered.data} directly on count data with CPM method (method 1): 
<<filtering>>=
myfilt = filtered.data(mycounts, factor = myfactors$Tissue, norm = FALSE, depth = NULL, method = 1, cv.cutoff = 100, cpm = 1, p.adj = "fdr")
@ 

The ``Sensitivity plot'' described in previous section can help to take decisions on the CPM threshold to use in methods 1 and 3.




\subsection{Batch effect correction} \label{sec_batch}

When a batch effect is detected in the data or the samples are not properly clustered due to an unknown source of technical noise, it is usually appropriate to remove this batch effect or noise before proceeding with the differential expression analysis (or any other type of analysis).

\texttt{ARSyNseq} (ASCA Removal of Systematic Noise for sequencing data) is an R function implemented in \noiseq{} package that is designed for filtering the noise associated to identified or unidentified batch effects. The ARSyN method \cite{nueda2012} combines analysis
of variance (ANOVA) modeling and multivariate analysis of estimated effects (PCA) to identify the structured variation of either the effect of the batch (if the batch information is provided) or the ANOVA errors (if the batch information is unknown). Thus, ARSyNseq returns a filtered data set that is rich in the information of interest and includes only the random noise required for inferential analysis.

The main arguments of the \texttt{ARSyNseq} function are:
\begin{itemize}
 \item \texttt{data}: A Biobase's eSet object created with the \texttt{readData} function.
 \item \texttt{factor}: Name of the factor (as it was given to the \texttt{readData} function) to be used in the ARSyN model (e.g. the factor containing the batch information). When it is NULL, all the factors are considered.
  \item \texttt{batch}: TRUE to indicate that the \texttt{factor} argument indicates the batch information. In this case, the \texttt{factor} argument must be used to specify the names of the onlu factor containing the information of the batch.
  \item \texttt{norm}: Type of normalization to be used. One of ``rpkm'' (default), ``uqua'', ``tmm'' or ``n'' (if data are already normalized). If length was provided through the \texttt{readData} function, it will be considered for the normalization (except for ``n''). Please note that if a normalization method if used, the arguments \texttt{lc} and  \texttt{k} are set to 1 and 0 respectively.
  \item \texttt{logtransf}: If FALSE, a log-transformation will be applied on the data before computing ARSyN model 
  to improve the results of PCA on count data.
\end{itemize}

Therefore, we can differentiate two types of analysis:
\begin{enumerate}
\item When batch is identified with one of the factors described in the argument \texttt{factor} of the \texttt{data} object, \texttt{ARSyNseq}
  estimates this effect and removes it by estimating the main PCs of the ANOVA effects associated. In such case \texttt{factor} argument 
    will be the name of the batch and \texttt{batch=TRUE}.  
\item When batch is not identified, the model estimates the effects associated to each factor of interest and analyses if there exists 
  systematic noise in the residuals. If there is batch effect, it will be identified and removed by estimating the main PCs of these residuals.
  In such case \texttt{factor} argument can have several factors and \texttt{batch=FALSE}.  
\end{enumerate}


We will use the toy example generated in Section \ref{sec_PCA} to illustrate how \texttt{ARSyNseq} works. This is the code to use 
\texttt{ARSyNseq} batch effect correction when the user knows the batch in which the samples were processed, and to represent a PCA with the filtered data in order to see how the batch effect was corrected (Figure \ref{fig_knownBatch}:

<<fig_knownBatch,fig=TRUE>>=
mydata2corr1 = ARSyNseq(mydata2, factor = "batch", batch = TRUE, norm = "rpkm",  logtransf = FALSE)
myPCA = dat(mydata2corr1, type = "PCA")
par(mfrow = c(1,2))
explo.plot(myPCA, factor = "Tissue")
explo.plot(myPCA, factor = "batch")
@

\begin{figure}[ht!]
\centering
\includegraphics[width=\textwidth, height=0.5\textwidth]{NOISeq-fig_knownBatch}
\caption{PCA plot after correcting a known batch effect with \texttt{ARSyNseq}. The samples are colored by tissue (left) and by batch (right)}
\label{fig_knownBatch}
\end{figure}

Let us suppose now that we do not know the batch information. However, we can appreciate in the PCA plot of Section \ref{sec_PCA} that there is an unknown source of noise that prevents the samples from clustering well. In this case, we can run the following code to reduce 
the unidentified batch effect and to draw the PCA plots on the filtered data:

<<fig_unknownBatch,fig=TRUE>>=
mydata2corr2 = ARSyNseq(mydata2, factor = "Tissue", batch = FALSE, norm = "rpkm",  logtransf = FALSE)
myPCA = dat(mydata2corr2, type = "PCA")
par(mfrow = c(1,2))
explo.plot(myPCA, factor = "Tissue")
explo.plot(myPCA, factor = "batch")
@

\begin{figure}[ht!]
\centering
\includegraphics[width=\textwidth, height=0.5\textwidth]{NOISeq-fig_unknownBatch}
\caption{PCA plot after correcting an unidentified batch effect with \texttt{ARSyNseq}. The samples are colored by tissue (left) and by batch (right)}
\label{fig_unknownBatch}
\end{figure}


\vspace{1cm}




\section{Differential expression}

The \noiseq{} package computes differential expression between two experimental conditions given the expression level of the considered features. 
The package includes two non-parametric approaches for differential expression analysis: \noiseq{} \cite{tarazona2011} for technical replicates or no replication at all, and \noiseqbio{} \cite{tarazona2015}, which is optimized for the use of biological replicates. 
Both methods take read counts from RNA-seq as the expression values, in addition to previously normalized data and
read counts from other NGS technologies.

In the previous section, we described how to use normalization and filtering functions prior to perform differential expression analysis.
However, when using \noiseq{} or \noiseqbio{} to compute differential expression, it is not necessary to normalize or filter low counts before applying these methods because they include these options. Thus, normalization can be done automatically 
by choosing the corresponding value for the parameter \texttt{norm}. Furthermore, they also accept expression values normalized with other packages or procedures. If the data have been previously normalized, \texttt{norm} parameter must be set to ``n''. Regarding the low-count filtering, it is not necessary to filter in \noiseq{} method. In contrast, it is recommended to do it in \noiseqbio{}, which by default filters out low-count features with CPM method (\texttt{filter=1}).

The following sections describe in more detail the \noiseq{} and \noiseqbio{} methods.



\subsection{NOISeq} \label{sec_param1}

\noiseq{} method was designed to compute differential expression on data with technical replicates (NOISeq-real) or no replicates at all (NOISeq-sim). 
If there are technical replicates available, it summarizes them by summing up them. It is also possible to apply this method on biological replicates, that are averaged instead of summed. However, for biological replicates we strongly recommend \noiseqbio{}. 
\noiseq{} computes the following differential expression statistics for each feature: 
$M$ (which is the $log_2$-ratio of the two conditions) and $D$ (the value of the difference between conditions). 
Expression levels equal to 0 are replaced with the given constant $k>0$, in order to avoid 
infinite or undetermined $M$-values. If $k=NULL$, the 0 is replaced by the midpoint between 0 and the next non-zero value 
in the expression matrix.
A feature is considered to be differentially expressed if its corresponding $M$ and $D$ values are likely to be higher than in noise. Noise distribution is obtained by comparing all pairs of replicates within the same condition. 
The corresponding $M$ and $D$ values are pooled together to generate the distribution. 
Changes in expression between conditions with the same magnitude than changes in expression between replicates within the same condition 
should not be considered as differential expression. Thus, by comparing the $(M,D)$ values of a given feature against the noise
distribution, \noiseq{} obtains the ``probability of differential expression'' for this feature. 
If the odds Pr(differential expression)/Pr(non-differential expression) are higher
than a given threshold, the feature is considered to be differentially expressed between conditions. 
For instance, an odds value of 4:1 is equivalent to $q$ = Pr(differential expression) = 0.8 and it means that the feature is 4 times more likely to be differentially expressed than non-differentially expressed. The \noiseq{} algorithm compares replicates
within the same condition to estimate noise distribution (NOISeq-real). When no replicates are available, NOISeq-sim simulates technical replicates in order to estimate the differential expression probability. Please remember that to obtain a really reliable statistical results, you need biological replicates. NOISeq-sim simulates technical replicates from a multinomial distribution, 
so be careful with the interpretation of the results when having no replicates, since they are only an
approximation and are only showing which genes are presenting a higher change between conditions in your particular samples.

Table \ref{table:summary} summarizes all the input options and includes some recommendations for the values of the parameters when using \noiseq{}: 

\begin{table}[ht]
\caption{Possibilities for the values of the parameters} % title name of the table
\centering % centering table
\begin{tabular}{llllllll} % creating 10 columns
\hline\hline % inserting double-line
\textbf{Method} &\textbf{Replicates} & \textbf{Counts} &\textbf{norm} &\textbf{k} &\textbf{nss} &\textbf{pnr} &\textbf{v}
% &\multicolumn{7}{c}{Sum of Extracted Bits}
\\ [0.5ex]
\hline
% Entering 1st row
& &Raw &rpkm, uqua, tmm &0.5 \\[-1ex]
\raisebox{1.5ex}{NOISeq-real} & \raisebox{1.5ex}{Technical/Biological}
&Normalized &n &NULL &\raisebox{1.5ex}{0} &\raisebox{1.5ex}{-} &\raisebox{1.5ex}{-} \\[1ex]
\hline
% Entering 2nd row
& &Raw &rpkm, uqua, tmm &0.5 \\[-1ex]
\raisebox{1.5ex}{NOISeq-sim} & \raisebox{1.5ex}{None}
&Normalized &n &NULL &\raisebox{1.5ex}{$\geq5$} &\raisebox{1.5ex}{0.2} &\raisebox{1.5ex}{0.02} \\[1ex]

\hline % inserts single-line
\end{tabular}
\label{table:summary}
\end{table}


Please note that \texttt{norm = "n"} argument should be used in \texttt{noiseq} or \texttt{noiseqbio} whenever the data have been previously normalized or corrected for a batch effect.



\subsubsection{NOISeq-real: using available replicates}

NOISeq-real estimates the probability distribution for M and D in an empirical way, by computing M and D values for every pair 
of replicates within the same experimental condition and for every feature. Then, all these values are pooled together to 
generate the noise distribution. Two replicates in one of the experimental conditions are enough to run the algorithm. If 
the number of possible comparisons within a certain condition is higher than 30, in order to reduce computation time, 30 
pairwise comparisons are randomly chosen when estimating noise distribution.

It should be noted that biological replicates are necessary if the goal is to make any inference about the population. Deriving 
differential expression from technical replicates is useful for drawing conclusions about the specific samples being compared in the
 study but not for extending these conclusions to the whole population.

In RNA-seq or similar sequencing technologies, the counts from technical replicates (e.g. lanes) can be summed up. 
Thus, this is the way the algorithm summarizes the information from
technical replicates to compute M and D signal values (between different conditions). However, for biological replicates, 
other summary statistics such us the mean may be more meaningful. \noiseq{} calculates the mean of the biological replicates 
but we strongly recommend to use \noiseqbio{} when having biological replicates.

Here there is an example with technical replicates and count data normalized by \code{rpkm} method. Please note that, 
since the factor ``Tissue'' has two levels, we do not need to indicate which conditions are to be compared. 
<<results>>=
mynoiseq = noiseq(mydata, k = 0.5, norm = "rpkm", factor="Tissue", pnr = 0.2, 
                  nss = 5, v = 0.02, lc = 1, replicates = "technical")
head(mynoiseq@results[[1]])
@ 

NA values would be returned if the gene had 0 counts in all the samples. In that case, the gene would not be used to compute differential expression.

Now imagine you want to compare tissues within the same sequencing run. Then, see the following example on how to apply NOISeq 
on count data with technical replicates, TMM normalization, and no length correction. 
As ``TissueRun'' has more than two levels we have to indicate which levels (conditions) are to be compared: 
<<results=hide>>=
mynoiseq.tmm = noiseq(mydata, k = 0.5, norm = "tmm", factor="TissueRun", 
                      conditions = c("Kidney_1","Liver_1"), lc = 0, replicates = "technical")
@ 




\subsubsection{NOISeq-sim: no replicates available}

When there are no replicates available for any of the experimental conditions, \noiseq{} can simulate technical replicates. 
The simulation relies on the assumption that read counts follow a multinomial distribution, where 
probabilities for each class (feature) in the multinomial distribution are the probability of a read to 
map to that feature. These mapping probabilities are approximated by using counts in the only sample of the
 corresponding experimental condition. Counts equal to zero are replaced with $k$>0 to give all features
 some chance to appear.

Given the sequencing depth (total amount of reads) of the unique available sample, the size of the simulated samples is a percentage
 (parameter $pnr$) of this sequencing depth, allowing a small variability (given by the parameter $v$). The number of 
replicates to be simulated is provided by $nss$ parameter.

Our dataset do has replicates but, providing it had not, you would use NOISeq-sim as in the following example in which the simulation parameters 
have to be chosen ($pnr$, $nss$ and $v$): 

<<results=hide>>=
myresults <- noiseq(mydata, factor = "Tissue", k = NULL, norm="n", pnr = 0.2, 
                    nss = 5, v = 0.02, lc = 1, replicates = "no")
@



\subsubsection{NOISeqBIO} \label{sec_param2}

NOISeqBIO is optimized for the use on biological replicates (at least 2 per condition). It was developed by joining the philosophy of our previous work 
together with the ideas from Efron \emph{et al.} in \cite{Efron2001}. In our case, we defined the differential expression statistic 
$\theta$ as $(M+D)/2$, where $M$ and $D$ are the statistics defined in the previous section but
including a correction for the biological variability of the corresponding feature. 
The probability distribution of $\theta$ can be described as a mixture of two distributions: one for features changing between conditions and the other
for invariant features. Thus, the mixture distribution $f$ can be written as: $f(\theta) = p_{0}f_{0}(\theta)+p_{1}f_{1}(\theta)$, 
where $p_{0}$ is the probability for a feature to have the same expression in both conditions and
$p_{1} = 1-p_{0}$ is the probability for a feature to have different expression between conditions. 
$f_{0}$ and $f_{1}$ are, respectively, the densities of $\theta$ for features with no change in expression
between conditions and for differentially expressed features. If one of both distributions can be estimated, 
the probability of a feature to belong to one of the two groups can be calculated. Thus, the algorithm consists of the following steps:
\begin{enumerate}
 \item Computing $\theta$ values. \\
$M$ and $D$ are corrected for the biological variability: $M^* = \dfrac{M}{a_{0}+\hat \sigma_M}$ and $D^* = \dfrac{D_s}{a_{0}+\hat \sigma_D}$,
where $\hat \sigma^2_M$ and $\hat \sigma^2_D$  are the standard errors of $M_s$ and $D_s$ statistics, respectively, and $a_0$ is computed as a given percentile of all the values in
$\hat \sigma_M$ or $\hat \sigma_D$,  as in \cite{Efron2001} (the authors suggest the percentile 90th as the best option, which is the default option of the parameter ``a0per" that
may be changed by the user). To compute the $\theta$ statistic, the $M$ and $D$ statistics are combined:  $\theta = \dfrac{M^* + D^*}{2}$.

 \item Estimating the values of the $\theta$ statistic when there is no change in expression, i.e. the null statistic $\theta_{0}$. \\
In order to compute the null density $f_{0}$ afterwards, we first need to estimate the values of the $\theta$-scores for features with no change
between conditions. To do that, we permute $r$ times (parameter that may be set by the user) the labels of samples between conditions, compute $\theta$ values as above and pool
them to obtain $\theta_{0}$.

 \item Estimating the probability density functions $f$ and $f_{0}$. \\
We estimate $f$ and $f_{0}$ with a kernel density estimator (KDE) with Gaussian kernel and smoothing parameter ``adj" as indicated by the user.

 \item Computing the probability of differential expression given the ratio $f_{0}/f$ and an estimation $\hat{p}_{0}$ for $p_{0}$. If $\theta=z$
for a given feature, this probability of differential expression can be computed as $p_{1}(z)=1-\hat{p}_{0}f_{0}(z)/f(z)$.\\
To estimate $p_{0}$, the following upper bound is taken, as suggested in \cite{Efron2001}: $p_{0} \leq \min_{Z} \{f(Z)/f_{0}(Z) \}$.\\
Moreover, it is shown in \cite{Efron2001} that the FDR defined by Benjamini and Hochberg can be considered equivalent
to the \emph{a posteriori} probability $p_0(z) = 1 - p_1(z)$ we are calculating.
\end{enumerate}

When too few replicates are available for each condition, the null distribution is very poor since the number of different permutations is
low. For those cases (number of replicates in one of the conditions less than 5), it is convenient to borrow information across genes. Our proposal
consists of clustering all genes according to their
expression values across replicates using the k-means method. For each cluster  $k$ of genes, we consider the expression values of all the
genes in the cluster as observations within the corresponding condition (replicates) and then we shuffle this submatrix $r \times g_k$
times, where $g_k$ is the number of genes within cluster $k$. If $r \times g_k$ is higher than 1000, we compute 1000 permutations in that cluster.  For each permutation, we calculate $M$ and $D$ values and their
corresponding standard errors. In order to reduce the computing time, if $g_k \geq 1000$, we again subdivide cluster $k$ in subclusters
with k-means algorithm.

We will consider that Marioni's data have biological replicates for the following example. In this case, the method 2 (Wilcoxon test) to filter low counts
is used. Please, use \code{?noiseqbio} to know more about the parameters of the function.

<<results=hide>>=
mynoiseqbio = noiseqbio(mydata, k = 0.5, norm = "rpkm", factor="Tissue", lc = 1, r = 20, adj = 1.5, plot = FALSE,
                        a0per = 0.9, random.seed = 12345, filter = 2)
@ 




\subsection{Results}\label{sec_deg}

\subsubsection{NOISeq output object}

\noiseq{} returns an \code{Output} object containing the following elements:

\begin{itemize}

\item
  \texttt{comparison}: String indicating the two experimental conditions being compared and the sense of the comparison.
\item
  \texttt{factor}: String indicating the factor chosen to compute the differential expression.
\item
  \texttt{k}: Value to replace zeros in order to avoid indetermination when computing logarithms. 
\item
  \texttt{lc}: Correction factor for length normalization. Counts are divided by $length^{lc}$. 
\item
  \texttt{method}: Normalization method chosen. 
\item
  \texttt{replicates}: Type of replicates: ``technical" for technical replicates and ``biological" for biological ones.
\item
  \texttt{results}: R data frame containing the differential expression results, where each row corresponds to a feature. The columns are: Expression values for each condition to
be used by \code{NOISeq} or \code{NOISeqBIO} (the columns names are the levels of the factor); differential expression statistics (columns``M" and ``D" for \code{NOISeq} or
``theta" for \code{NOISeqBIO}); probability of differential expression (``prob"); ``ranking", which is a summary statistic of ``M" and ``D" values equal to $-sign(M) \times
\sqrt{M^2 + D^2}$, than can be used for instance in gene set enrichment analysis (only for \code{NOISeq}); ``Length" of each feature (if provided); ``GC" content of each feature
(if provided);
chromosome where the feature is (``Chrom"), if provided; start and end position of the feature within the chromosome (``GeneStart", ``GeneEnd"), if provided; feature biotype
(``Biotype"), if provided. 
\item
  \texttt{nss}: Number of samples to be simulated for each condition (only when there are not replicates available). 
\item
  \texttt{pnr}: Percentage of the total sequencing depth to be used in each simulated replicate (only when there are not replicates available). For instance, if pnr = 0.2 , each
simulated replicate will have 20\% of the total reads of the only available replicate in that condition.
\item
  \texttt{v}: Variability of the size of each simulated replicate (only used by NOISeq-sim).  
\end{itemize}

For example, you can use the following instruction to see the differential expression results for \code{NOISeq}:

<<>>=
head(mynoiseq@results[[1]])
@ 

The output \code{myresults@results[[1]]\$prob} gives the estimated probability of differential expression for each feature. 
Note that when using \noiseq{}, these probabilities are not equivalent to p-values. The higher the probability, the more likely that the difference in expression 
is due to the change in the experimental condition and not to chance. See Section \ref{sec_deg} to learn how to obtain the differentially expressed features.






\subsubsection{How to select the differentially expressed features} 

Once we have obtained the differential expression probability for each one of the features by using \code{NOISeq} or \code{NOISeqBIO} function, 
we may want to select the differentially expressed features for a given threshold $q$. This can be done with \code{degenes} function 
on the ``output" object using the parameter \code{q}. With the argument \code{M} we choose if we want all the differentially expressed features, 
only the differentially expressed features that are more expressed in condition 1 than in condition 2 (M = ``up") 
or only the differentially expressed features that are under-expressed in condition 1 with regard to condition 2 (M = ``down"):

<<>>=
mynoiseq.deg = degenes(mynoiseq, q = 0.8, M = NULL)
mynoiseq.deg1 = degenes(mynoiseq, q = 0.8, M = "up")
mynoiseq.deg2 = degenes(mynoiseq, q = 0.8, M = "down")
@ 

Please remember that, when using \code{NOISeq}, the probability of differential expression is not equivalent to $1-pvalue$. 
We recommend for $q$ to use values around $0.8$. If \code{NOISeq-sim} has been used because no replicates are available, 
then it is preferable to use a higher threshold such as $q=0.9$. However, when using \code{NOISeqBIO}, the
probability of differential expression would be equivalent to $1-FDR$, where $FDR$ can be considered as an adjusted p-value. 
Hence, in this case, it would be more convenient to use $q=0.95$.




\subsubsection{Plots on differential expression results}

\textbf{Expression plot}

Once differential expression has been computed, it is interesting to plot the average expression values of each condition and highlight the features declared as differentially expressed.
It can be done with the \code{DE.plot}. 

To plot the summary of the expression values in both conditions as in Fig. \ref{fig_summ_expr}, please write the following code 
(many graphical parameters can be adjusted, see the function help). Note that by giving $q=0.9$, differentially expressed features 
considering this threshold will be highlighted in red:
<<fig_summ_expr,fig=TRUE>>=
DE.plot(mynoiseq, q = 0.9, graphic = "expr", log.scale = TRUE)
@ 

\begin{figure}[ht!]
\centering
\includegraphics[width=0.6\textwidth]{NOISeq-fig_summ_expr}
\caption{Summary plot of the expression values for both conditions (black), where differentially expressed genes are highlighted (red).}
\label{fig_summ_expr}
\end{figure}


\textbf{MD plot}

Instead of plotting the expression values, it is also interesting to plot the log-fold change ($M$) and the absolute value of the difference in expression between conditions ($D$) as in Fig. \ref{fig_summ_MD}. 
This is an example of the code to get such a plot ($D$ values are displayed in log-scale) from \code{NOISeq} output (it is analogous for \code{NOISeqBIO} ouput). 
<<fig_summ_MD,fig=TRUE>>=
DE.plot(mynoiseq, q = 0.8, graphic = "MD")
@ 

\begin{figure}[ht!]
\centering
\includegraphics[width=0.6\textwidth]{NOISeq-fig_summ_MD}
\caption{Summary plot for (M,D) values (black) and the differentially expressed genes (red).}
\label{fig_summ_MD}
\end{figure}


\textbf{Manhattan plot}

The Manhattan plot can be used to display the expression of the genes across the chromosomes. 
The expression for both conditions under comparison is shown in the plot. 
The users may choose either plotting all the chromosomes or only some of them, 
and also if the chromosomes are depicted consecutively (useful for prokaryote organisms) 
or separately (one per line).
If a $q$ cutoff is provided, then differentially expressed features are highlighted in a different color.
The following code shows how to draw the Manhattan plot from the output object returned by \code{NOISeq} or \code{NOISeqBIO}. 
In this case, using Marioni's data, the expression (log-transformed) is represented for two chromosomes (see Fig. \ref{fig_manhattan}). 
Note that the chromosomes will be depicted in the same order that are given to ``chromosomes" parameter. 

Gene expression is represented in gray. Lines above 0 correspond to the first condition under comparison (kidney) 
and lines below 0 are for the second condition (liver). Genes up-regulated in the first condition are highlighted in red, 
while genes up-regulated in the second condition are highlighted in green. The blue lines on the horizontal axis (Y=0)
correspond to the annotated genes. X scale shows the location in the chromosome.
<<fig_manhattan,fig=TRUE,width=12,height=5>>=
DE.plot(mynoiseq, chromosomes = c(1,2), log.scale = TRUE,
        join = FALSE, q = 0.8, graphic = "chrom")
@ 


\begin{figure}[ht!]
\centering
\includegraphics[width=\textwidth]{NOISeq-fig_manhattan}
\caption{Manhattan plot for chromosomes 1 and 2}
\label{fig_manhattan}
\end{figure}

It is advisable, in this kind of plots, to save the figure in a file, for instance, a pdf file (as in the following code), 
in order to get a better visualization with the zoom.

\begin{Schunk}
\begin{Sinput}
pdf("manhattan.pdf", width = 12, height = 50)
DE.plot(mynoiseq, chromosomes = c(1,2), log.scale = TRUE,
        join = FALSE, q = 0.8)
dev.off()
\end{Sinput}
\end{Schunk}




\textbf{Distribution of differentially expressed features per chromosomes or biotypes}

This function creates a figure with two plots if both chromosomes and biotypes information is provided. 
Otherwise, only a plot is depicted with either the chromosomes or biotypes (if information of any of them is available). The $q$ cutoff must be provided.
Both plots are analogous. The chromosomes plot shows the percentage of features in each chromosome, 
the proportion of them that are differentially expressed (DEG) and the percentage of differentially expressed features in each chromosome. 
Users may choose plotting all the chromosomes or only some of them. The chromosomes are depicted according to the number of features they contain 
(from the greatest to the lowest). The plot for biotypes can be described similarly. The only difference is that this plot has a left axis scale
for the most abundant biotypes and a right axis scale for the rest of biotypes, which are separated by a green vertical line.

The following code shows how to draw the figure from the output object returned by \code{NOISeq} for the Marioni's example data. 

<<fig_distrDEG,fig=TRUE,width=12,height=7>>=
DE.plot(mynoiseq, chromosomes = NULL, q = 0.8, graphic = "distr")
@ 


\begin{figure}[ht!]
\centering
\includegraphics[width=\textwidth]{NOISeq-fig_distrDEG}
\caption{Distribution of DEG across chromosomes and biotypes for Marioni's example dataset.}
\label{fig_distrDEG}
\end{figure}




\vspace{1cm}

%\clearpage

\section{Setup}

This vignette was built on:
<<session>>=
sessionInfo()
@ 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\vspace{2cm}


\begin{thebibliography}{9}
% \providecommand{\natexlab}[1]{#1}
% \providecommand{\url}[1]{\texttt{#1}}
% \expandafter\ifx\csname urlstyle\endcsname\relax
%   \providecommand{\doi}[1]{doi: #1}\else
%   \providecommand{\doi}{doi: \begingroup \urlstyle{rm}\Url}\fi

\bibitem{tarazona2011}
S. Tarazona, F. Garc\'{\i}a-Alcalde, J. Dopazo, A. Ferrer, and A. Conesa.
\newblock {Differential expression in RNA-seq: A matter of depth}.
\newblock \emph{Genome Research}, 21: 2213 - 2223, 2011.

\bibitem{tarazona2015}
S. Tarazona, P. Furi\'{o}-Tar\'{i}, D. Turr\'{a}, A. Di Pietro, M.J. Nueda, A. Ferrer, and A. Conesa.
\newblock {Data quality aware analysis of differential expression in RNA-seq with NOISeq R/Bioc package}.
\newblock \emph{Nucleic Acids Research}, 43(21):e140, 2015.

\bibitem{marioni2008}
J.C. Marioni, C.E. Mason, S.M. Mane, M. Stephens, and Y. Gilad.
\newblock RNA-seq: an
assessment of technical reproducibility and comparison with gene
expression arrays.
\newblock \emph{Genome Research}, 18: 1509 - 517, 2008.


\bibitem{Mortazavi2008} 
A. Mortazavi, B.A. Williams, K. McCue, L. Schaeffer, and
B. Wold.  \newblock {Mapping and quantifying mammalian transcriptomes by RNA-Seq}.
  \newblock \emph{Nature Methods}, 5: 621 - 628, 2008.


\bibitem{Bullard2010}
J.H. Bullard, E.~Purdom, K.D. Hansen, and S.~Dudoit.
\newblock Evaluation of statistical methods for normalization and differential
  expression in {mRNA-Seq} experiments.
\newblock \emph{BMC bioinformatics}, 11\penalty0 (1):\penalty0 94, 2010.


\bibitem{Robinson2010}
M.D. Robinson, and A. Oshlack.
\newblock A scaling normalization method for differential expression analysis of {RNA-Seq} data.
\newblock \emph{Genome Biology}, 11: R25, 2010.


\bibitem{nueda2012}
M. Nueda, A. Conesa, and A. Ferrer.
\newblock {ARSyN: a method for the identification and removal
of systematic noise in multifactorial time-course microarray experiments}.
\newblock \emph{Biostatistics}, 13(3):553–566, 2012.

\bibitem{Efron2001}
B. Efron, R. Tibshirani, J.D. Storey, V. Tusher.
\newblock {Empirical Bayes Analysis of a Microarray Experiment}.
\newblock \emph{Journal of the American Statistical Association}, 2001.




\end{thebibliography}





\end{document}

\name{CD}
\alias{CD-class}
\alias{CD}
\alias{show,CD-method}
\alias{dat2save,CD-method}
\alias{explo.plot,CD-method}
\docType{class}
\title{CD class}

\description{
CD class generated from dat() function with type="cd". This object contains the distributions of log-fold changes (M values) between each of the samples and a reference sample as well as confidence intervals for the median of these distributions that are used to detect a potential RNA composition bias in the data.
}

\usage{
\S4method{explo.plot}{CD}(object, samples = NULL, ...)
\S4method{dat2save}{CD}(object)
}

\arguments{
  \item{object}{
    Object generated from \code{dat()} function.
  }
  \item{samples}{
    Samples or conditions to be plotted. If NULL, the twelve first samples are plotted because the plot for this object only admit a maximum of twelve samples.
  }
  \item{...}{
    Any argument from \code{par}.
  }
}

\section{Slots/List Components}{
Objects of this class contain (at least) the following list components:

  \code{dat}: List containing the following elements:

  \code{data2plot}: Data frame where each column contains the M values obtained as the log2-ratio of each sample against the reference sample.
  \code{refColumn}: Column number in input data that is taken as the reference sample.
  \code{DiagnosticTest}: Data frame that contains the lower and upper limits of the confidence intervals for the median of M values per each sample. The last column indicates if the diagnostic test for that sample has been passed or failed (so normalization has to be applied).
  
}

\section{Methods}{
This class has an specific \code{show} method in order to show the confidence intervals for the M median 
and a \code{dat2save} method to save the relevant information in the object in a user-friendly way. It also has an
\code{explo.plot} method to plot the data contained in the object.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{myCounts}
\alias{myCounts-class}
\alias{myCounts}
\docType{class}
\title{Class myCounts}

\description{
This is the main class which contains the information needed to do the different analyses.
}

\section{Extends}{
Class \code{eSet} (package 'Biobase').
}

\section{Quick View}{
This object will contain the expression data and further information needed to do the exploratory analysis or the normalization such as the length, GC content, biotypes, chromosomes and positions for each feature.

Internally, the data is stored as follows:

As \code{myCounts} derives from \code{eSet}, we have used the slot \code{assayData} to store all the expression data, \code{phenoData} to store the factors with the conditions, \code{featureData} which will contain the variables \code{Length}, \code{GCcontent}, \code{Biotype}, \code{Chromosome}, {Start Position}, \code{End Position} for each feature. It has been used the slot \code{experimentData} derived from \code{MIAME-class} which will contain the type of replicates (biological replicates, technical replicates or no replicates at all).
}
\seealso{

If you need further information to know the methods that can be used, see \code{eSet}, \code{AnnotatedDataFrame-class}, \code{MIAME-class}. 
}

\author{Sonia Tarazona}

\keyword{classes}

\name{Saturation}
\alias{Saturation-class}
\alias{Saturation}
\alias{saturation}
\alias{explo.plot,Saturation-method}
\alias{show,Saturation-method}
\alias{dat2save,Saturation-method}
\docType{class}
\title{Saturation class}

\description{
Saturation class generated from dat() function with type="saturation". This object contains the number of detected features per biotype at 
increasing sequencing depths and also the new detections per each million of new sequencing reads.
}

\usage{
\S4method{explo.plot}{Saturation}(object, samples = NULL, toplot = 1, yleftlim = NULL, yrightlim = NULL, ...)
\S4method{dat2save}{Saturation}(object)
}

\arguments{
  \item{object}{
    Object generated from \code{dat()} function.
  }
   \item{toplot}{
    This parameter indicates which biological group is to be plotted.
    It may be a number or a text with the name of the biological group. 
    If toplot=1 (or "global"), a global plot considering features from all the biological groups will be generated.
  }
  \item{samples}{
    The samples to be plotted. If NULL, all the samples are plotted for Saturation object.
  }

  \item{yleftlim}{
    Range for Y left-axis (on the left-hand side of the plot) when new detections are plotted (this occurs when the number of samples to be plotted is 1 or 2). 
If NULL (default), an appropriate range is computed.    
    } 
  \item{yrightlim}{
    Range for Y right-axis (on the right-hand side of the plot) when new detections are plotted (this occurs when the number of samples to be plotted is 1 or 2). 
If NULL (default), an appropriate range is computed.    
    }
  \item{...}{
    Any argument from \code{par}.
  }     
}

\section{Slots/List Components}{
Objects of this class contain (at least) the following list components:

  \code{dat}: List containing the information generated by dat() function. This list has the following elements:
  
  \code{saturation}: List containing for all the biological classes (and also a global class with all of them together) the saturation data 
		     to be plotted for each sample (in Y left axis).

  \code{bionum}: Vector containing for all the biological classes (and also a global class with all of them together) the number of 
		 features for that group.
  
  \code{depth}: List containing for each selected sample the increasing values of sequencing depth to be plotted.

  \code{newdet}: List containing for all the biological classes (and also a global class with all of them together) the new detection data 
		 to be plotted for each selected sample (in Y right axis). 

  \code{real}: List with as many elements as the number of biological classes (plus one for the global). Each element contains the real sequencing depth for each sample and the corresponding number of detected features at that sequencing depth. 
}

\section{Methods}{
This class has an specific \code{show} method in order to work and print
a summary of the elements which are contained and a \code{dat2save} method
to save the relevant information in an object cleanly. It also has an
\code{explo.plot} method to plot the data contained in the object.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{Data_Exploration}
\alias{dat}

\title{
Exploration of expression data.
}
\description{
Take the expression data and the feature annotations to generate the results that will be used for the exploratory plots (\code{explo.plot}) or saved by the user to perform other analyses.
}
\usage{
dat(input, type = c("biodetection","cd","countsbio","GCbias","lengthbias","saturation","PCA"), 
                    k = 0, ndepth = 6, factor = NULL, norm = FALSE, refColumn = 1, logtransf = FALSE)
}

\arguments{
  \item{input}{
Object of eSet class with expression data and optional annotation.
}
  \item{type}{
Type of plot for which the data are to be generated. It can be one of: "biodetection","cd","countsbio","GCbias","lengthbias","saturation".
}
  \item{k}{
A feature is considered to be detected if the corresponding number of read counts is > k. By default, k = 0. This parameter is used by types "biodetection" and "saturation".
}
  \item{ndepth}{
Number of different sequencing depths to be simulated and plotted apart from the real depth. By default, ndepth = 6. This parameter is only used by type "saturation".
}
  \item{factor}{
If factor = NULL (default), the calculations are done for each sample independently. When the factor is specified, the calculations are done for each experimental condition. 
Samples within the same condition are summed up ("biodetection") or averaged and normalized by sequencing depth ("countsbio", "GCbias" and "lengthbias").
}
  \item{norm}{
To indicate if provided data are already normalized (TRUE) or they are raw data (FALSE), which is the default. This parameter is used by types "cd", "lengthbias", "GCbias" and "countsbio".
}
  \item{refColumn}{
Column number in input data that is taken as the reference sample to compute M values. This parameter is only used by type "cd".
}
  \item{logtransf}{
To indicate if the data are already log-transformed (TRUE) or not (FALSE). If data are not log-transformed, a log-transformation will be applied before computing the Principal Component Analysis.
}

}


\value{
\code{dat()} function returns an S4 object to be used by \code{explo.plot()} or to be converted into a more friendly formatted object by the \code{dat2save()} function.
}


\author{
Sonia Tarazona
}


\seealso{
       \code{\link{Biodetection}},\code{\link{CD}},\code{\link{CountsBio}},\code{\link{GCbias}},\code{\link{lengthbias}},\code{\link{Saturation}},\code{\link{PCA}},\code{\link{readData}},\code{\link{addData}},\code{\link{dat2save}},\code{\link{explo.plot}}
     }

\examples{
## Load the input object with the expression data and the annotations
data(myCounts)

## Generating data for the plot "biodetection" and samples in columns 3 and 4 of expression data
mydata2plot = dat(mydata, type = "biodetection", k = 0)

## Generating the corresponding plot
explo.plot(mydata2plot, samples = c(3,4))

}

\name{Biodetection}
\alias{Biodetection-class}
\alias{Biodetection}
\alias{show,Biodetection-method}
\alias{explo.plot,Biodetection-method}
\alias{dat2save,Biodetection-method}
\docType{class}
\title{Biodetection class}

\description{
Biodetection class generated from dat() function with type="biodetection". This object contains the percentage of each biological 
class (e.g. biotype) in the genome (i.e. in the whole set of features provided), the corresponding percentage detected by the sample 
and the percentage of the biotype within the sample.
}

\usage{
\S4method{explo.plot}{Biodetection}(object, samples = c(1, 2), plottype = c("persample", "comparison"), toplot = "protein_coding", ...)
\S4method{dat2save}{Biodetection}(object)
}

\arguments{
  \item{object}{
    Object generated from \code{dat()} function.
  }
  \item{samples}{
    Samples or conditions to be plotted. If NULL, the two first samples are plotted because the plot for this object only admit a maximum of two samples.
  }
  \item{plottype}{
    If plottype="persample", each sample is plotted in a separate plot displaying abundance of byotype in genome, percentage of biotype detected by sample and abundance of biotype in sample.
    If plottype="comparison", two samples can be compared in the same plot. Two plots are generated, one for the percentage of biotype detected by each of the compared samples, and the other 
    for the abundance of the biotypes within the compared samples. 
  }
  \item{toplot}{
    If plottype="comparison" and a biotype is specified in this argument (by default toplot="protein_coding"), a proportion test is performed to test if the abundance of that biotype
    is significantly different for the two samples being compared.
  }
  \item{...}{
    Any argument from \code{par}.
  }
}

\section{Slots/List Components}{
An object of this class contains an element (dat) which is a list with the following components:

  \code{genome}: Vector containing the percentage of features per biotype in the genome.
  
  \code{biotables}: List with as many elements as samples or conditions. Each element of the list contains the percentage of features in the genome per biotype detected in that sample or condition features per biotype and the percentage of detected features in the sample or condition per biotype.  
}

\section{Methods}{
This class has an specific \code{show} method in order to work and print
a summary of the elements which are contained and a \code{dat2save} method
to save the relevant information in an object cleanly. It also has an
\code{explo.plot} method to plot the data contained in the object.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{GCbias}
\alias{GCbias-class}
\alias{GCbias}
\alias{explo.plot,GCbias-method}
\alias{show,GCbias-method}
\alias{dat2save,GCbias-method}
\docType{class}
\title{GCbias class}

\description{
GCbias class generated from dat() function with type="GCbias". 
This object contains the trimmed mean of expression for each GC content bin of 200 features per sample or condition and also per biotype (if available). 
It also includes the corresponding spline regression model fitted to explain the relationship between length and expression.
}

\usage{
\S4method{explo.plot}{GCbias}(object, samples = NULL, toplot = "global", ...)
\S4method{dat2save}{GCbias}(object)
}

\arguments{
  \item{object}{
    Object generated with \code{dat()} function.
  }
  \item{toplot}{
    Biological group to be plotted (features not belonging to that group are discarded).
    It may be a number or a text with the name of the biological group. 
    If toplot=1 or toplot="global", all features are used for the plot.
  }
  \item{samples}{
    Samples (or conditions) to be plotted. If NULL, all the samples are plotted.
    If samples > 2, only a descriptive plot will be generated. If not, diagnostic plots will be obtained showing both the R-squared and model p-value
from the spline regression model describing the relationship between the GC content and the expression. 
  }  
  \item{...}{
    Any argument from \code{par}.
  }
}

\section{Slots/List Components}{
Objects of this class contain (at least) the following list components:

  \code{dat}: List containing the information generated by dat() function. This list has the following elements:
  
  \code{data2plot}: A list with as many elements as biological groups (the first element correspond to all the features). 
Each element of the list is a matrix containing the GC content bins in the first column and an additional column 
for the trimmed mean expression per bin for each sample or condition. 
  \code{RegressionModels}: A list with as many elements as samples or conditions. 
Each element is an "lm" class object containing the spline regression model relating GC content and expression for that sample or condition (considering all the features).
  
}

\section{Methods}{
This class has an specific \code{show} method to print
a summary of spline regression models and a \code{dat2save} method
to save the GC content bin information. It also has an
\code{explo.plot} method to plot the data contained in the object.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{PCA.GENES}
\alias{PCA.GENES}

\title{
Principal Component Analysis
}
\description{
Computes a Principal Component Analysis on any data matrix.
}
\usage{
PCA.GENES(X)
}

\arguments{
  \item{X}{
		Matrix or data.frame with variables (e.g. genes) in columns and observations (e.g. samples) in rows.
	}
}

\examples{

	## Simulate data matrix with 500 variables and 10 observations
	datasim = matrix(sample(0:100, 5000, replace = TRUE), nrow = 10)

	## PCA
	myPCA = PCA.GENES(datasim)

	## Extracting the variance explained by each principal component
	myPCA$var.exp

}


\author{
Maria Jose Nueda
}

\name{Exploratory_Plots}
\alias{explo.plot}

\docType{methods}

\title{
Exploratory plots for expression data.
}

\description{
Standard generic function. Different types of plots showing the biological classification for detected features, the expression distribution across samples or biological groups, the detection of technical bias such as length, GCcontent or RNA composition, the dependence of expression on sequencing depth, etc.
}


\usage{
explo.plot(object, ...)
}

\arguments{
  \item{object}{
Object generated with \code{dat()} function.
}

  \item{...}{
    Any argument from \code{par}.
  }
}

\value{
The explo.plot() function takes the object generated by dat() and draws the corresponding plot.
}


\author{
Sonia Tarazona
}


\seealso{
       \code{\link{Biodetection}},\code{\link{CD}},\code{\link{CountsBio}},\code{\link{GCbias}},\code{\link{lengthbias}},\code{\link{Saturation}}, \code{\link{PCA}}, \code{\link{readData}}, \code{\link{addData}}, \code{\link{dat}}.
     }

\examples{
## Load the input object with the expression data and the annotations
data(myCounts)

## Generating data for the plot "biodetection" and samples in columns 3 and 4 of expression data
mydata2plot = dat(mydata, type = "biodetection", k = 0)

## Generating the corresponding plot
explo.plot(mydata2plot)

}

\name{example}

\docType{data}

\alias{mydata}
\alias{mynoiseq}

\title{Example of objects used and created by the NOISeq package}
\description{
  This is a quick view of the objects generated by the
  package. To take a look, see the usage information. These objects have
  been created from Marioni's reduce dataset (only chromosomes I to IV).
}
\usage{
# To load the object myCounts generated by the readData() function from R objects containing expression data, the factors describing the experimental conditions to be studied, the feature length, the feature biotypes, the chromosome and the position:
data(myCounts)

# To load the object generated after running the noiseq() function to compute differential expression:
data(noiseq)
}

\references{
    Marioni, J.C. and Mason, C.E. and Mane, S.M. and Stephens, M. and Gilad, Y. (2008) RNA-seq: an assessment of
technical reproducibility and comparison with gene expression arrays. \emph{Genome Research}, \bold{18}:
1509--1517.
}

\keyword{datasets}

\name{noiseq}
\alias{noiseq}

\title{
Differential expression method for technical replicates or no replicates at all
}
\description{
\code{noiseq} computes differential expression between two experimental conditions from read count data (e.g. RNA-seq).
}
\usage{
noiseq(input, k = 0.5, norm = c("rpkm","uqua","tmm","n"), 
replicates = c("technical","biological","no"), 
factor=NULL, conditions=NULL, pnr = 0.2, nss = 5, v = 0.02, lc = 0)
}

\arguments{
  \item{input}{
Object of eSet class coming from \code{\link{readData}} function or other R packages such as DESeq.
}
  \item{factor}{
A string indicating the name of factor whose levels are the conditions to be compared.
}
  \item{conditions}{
A vector containing the two conditions to be compared by the differential expression algorithm (needed when the \code{factor} contains more than 2 different conditions).
}
\item{replicates}{
  In this argument, the type of replicates to be used is defined: "technical", "biological" or "no" replicates. By default, "technical" replicates option is chosen.
}
  \item{k}{
Counts equal to 0 are replaced by k. By default, k = 0.5.
}
  \item{norm}{
Normalization method. It can be one of "rpkm" (default), "uqua" 
(upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).
}
  \item{lc}{
Length correction is done by dividing expression by length^lc.
By default, lc = 0.
}
  \item{pnr}{
Percentage of the total reads used to simulated each sample when no replicates are available. By default, pnr = 0.2.
}
  \item{nss}{
Number of samples to simulate for each condition (nss>= 2). By default, nss = 5.
}
  \item{v}{
Variability in the simulated sample total reads. 
By default, v = 0.02. Sample total reads is computed as a
random value from a uniform distribution in the interval
[(pnr-v)*sum(counts), (pnr+v)*sum(counts)]
}
}
\value{
The function returns an object of class \code{\link{Output}}
}

\author{
Sonia Tarazona
}

\seealso{
       \code{\link{readData}}.
     }

\examples{
## Load the input object from Marioni's data as returned by readData()
data(myCounts)

## Computing differential expression probability on RPKM-normalized data by NOISeq-real using factor "Tissue"
mynoiseq = noiseq(mydata, k = 0.5, norm = "rpkm", replicates = "technical", factor="Tissue", 
pnr = 0.2, nss = 5, v = 0.02, lc = 1)


## Computing differential expression probability on Upper Quartile normalized data by NOISeq-real 
## using factor "TissueRun" and comparing samples in Run 1 (levels "Kidney_1" and "Liver_1")
mynoiseq.uqua = noiseq(mydata, k = 0.5, norm = "uqua", replicates = "technical", factor="TissueRun", 
conditions = c("Kidney_1","Liver_1"), pnr = 0.2, nss = 5, v = 0.02, lc = 1)

}

\references{

Bullard J.H., Purdom E., Hansen K.D. and Dudoit S. (2010) Evaluation of statistical methods for 
normalization and differential expression in mRNA-seq experiments. \emph{BMC Bioinformatics} 11(1):94+.

Mortazavi A., Williams B.A., McCue K., Schaeer L. and Wold B. (2008) Mapping and quantifying 
mammalian transcriptomes by RNA-seq. \emph{Nature Methods} 5(7):621-628.

Robinson M.D. and Oshlack A. (2010) A scaling normalization method for differential expression
analysis of RNA-seq data. \emph{Genome Biology} 11(3):R25+.

Marioni, J.C. and Mason, C.E. and Mane, S.M. and Stephens, M. and Gilad, Y. (2008) RNA-seq: an assessment of
technical reproducibility and comparison with gene expression arrays. \emph{Genome Research}, \bold{18}:
1509--1517.

}

\name{Marioni}

\docType{data}

\alias{mybiotypes}
\alias{mycounts}
\alias{mychroms}
\alias{mylength}
\alias{mygc}
\alias{myfactors}

\title{Marioni's dataset}
\description{
  This is a reduced version for the RNA-seq count data from Marioni et al. (2008) along with additional annotation such as gene biotype, gene length, GC content, chromosome, start position and end position for genes in chromosomes I to IV. The expression data consists of 10 samples from kidney and liver tissues. There are five technical replicates (lanes) per tissue.
}
\usage{
data(Marioni)
}

\references{ 
    Marioni, J.C. and Mason, C.E. and Mane, S.M. and Stephens, M. and Gilad, Y. (2008) RNA-seq: an assessment of
technical reproducibility and comparison with gene expression arrays. \emph{Genome Research}, \bold{18}:
1509--1517.
}

\keyword{datasets}

\name{PCA}
\alias{PCA-class}
\alias{PCA}
\alias{show,PCA-method}
\alias{explo.plot,PCA-method}
\alias{dat2save,PCA-method}
\docType{class}
\title{PCA class}

\description{
PCA class generated from dat() function with type="PCA". This object contains the results of the PCA on the data matrix as well as the arguments used.
}

\usage{
\S4method{explo.plot}{PCA}(object, samples = 1:2, plottype = "scores", factor = NULL)
\S4method{dat2save}{PCA}(object)
}

\arguments{
  \item{object}{
    Object generated from \code{dat()} function.
  }
  \item{samples}{
    Principal components to be plotted. If NULL, the two first components are plotted.
  }
  \item{plottype}{
    If plottype="scores", the experimental samples are displayed in the plot and colored according to the values of the selected factor.
    If plottype="loadings", the genes are plotted. 
  }
  \item{factor}{
    The samples in the score plot will be colored according to the values of the selected factor. If NULL, the first factor is chosen.
  }
}

\section{Slots/List Components}{
An object of this class contains an element (dat) which is a list with the following components:

  \code{result}: List containing the output of PCA. It contains the following elements: "eigen" (eigenvalues and eigenvectors from the PCA decomposition), 
	  "var.exp" (variance explained by each Principal Component), "scores" (coefficients of samples in each PC), "loadings" (coefficients of genes in each PC).
  
  \code{factors}: Data.frame with factors inherited from object generated by readData() function.

  \code{norm}: Value provided for argument "norm". 

  \code{logtransf}: Value provided for argument "logtransf". 
}

\section{Methods}{
This class has an specific \code{show} method in order to work and print
a summary of the elements which are contained and a \code{dat2save} method
to save the relevant information in an object cleanly. It also has an
\code{explo.plot} method to plot the data contained in the object.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{noiseqbio}
\alias{noiseqbio}

\title{
Differential expression method for biological replicates
}
\description{
\code{noiseqbio} computes differential expression between two experimental conditions from read count data (e.g. RNA-seq).
}
\usage{
noiseqbio(input, k = 0.5, norm = c("rpkm","uqua","tmm","n"), nclust = 15, plot = FALSE,
                      factor=NULL, conditions = NULL, lc = 0, r = 50, adj = 1.5,
                      a0per = 0.9, random.seed = 12345, filter = 1, depth = NULL,
                      cv.cutoff = 500, cpm = 1)
}

\arguments{
  \item{input}{
Object of eSet class coming from \code{\link{readData}} function or other R packages such as DESeq.
}
  \item{k}{
Counts equal to 0 are replaced by k. By default, k = 0.5.
}
  \item{norm}{
Normalization method. It can be one of "rpkm" (default), "uqua" 
(upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).
}
  \item{factor}{
A string indicating the name of factor whose levels are the conditions to be compared.
}
  \item{conditions}{
A vector containing the two conditions to be compared by the differential expression algorithm (needed when the \code{factor} contains more than 2 different conditions).
}
  \item{lc}{
Length correction is done by dividing expression by length^lc. By default, lc = 0.
}
  \item{r}{
Number of permutations to generate noise distribution by resampling.
}
  \item{adj}{
Smoothing parameter for the Kernel Density Estimation of noise distribution. Higher values produce smoother curves.
}
  \item{nclust}{
Number of clusters for the K-means algorithm. Used when the number of replicates per condition is less than 5.
}
\item{plot}{
  If TRUE, a plot is generated showing the mixture distribution (f) and the noise distribution (f0) of theta values.
}
\item{a0per}{
  M and D values are corrected for the biological variability by being divided by S + a0, where S is the standard error of the corresponding statistic 
and a0 is determined by the value of a0per parameter. If a0per is NULL, a0 = 0. If a0per is a value between 0 and 1, 
a0 is the a0per percentile of S values for all features. If a0per = "B", a0 takes the highest value given by 100*max(S).
}
\item{random.seed}{
  Random seed. In order to get the same results in different runs of the method (otherwise the resampling procedure would produce different resulst), 
the random seed is set to this parameter value. 
}
\item{filter}{
  Method to filter out low count features before computing differential expression analysis. If filter=0, no filtering is performed. 
If 1, CPM method is applied. 
If 2, Wilcoxon test method (not recommended when the number of replicates per condition is less than 5),
If 3, proportion test method. Type \code{?filtered.data} for more details.
}
\item{depth}{
  Sequencing depth of each sample to be used by filtering method. It must be data provided when the data is already normalized 
and filtering method 3 is to be applied.
}
\item{cv.cutoff}{
  Cutoff for the coefficient of variation per condition to be used in filtering method 1.
}
\item{cpm}{
  Cutoff for the counts per million value to be used in filtering methods 1 and 3.
}
}
\value{
The function returns an object of class \code{\link{Output}}
}

\author{
Sonia Tarazona
}

\seealso{
       \code{\link{readData}}.
     }

\examples{
## Load the input object from Marioni's data as returned by readData()
data(myCounts)

## Computing differential expression probability by NOISeqBIO using factor "Tissue" (data will be RPKM-normalized)
mynoiseqbio = noiseqbio(mydata, k = 0.5, norm = "rpkm", factor="Tissue", lc = 1, r = 50, adj = 1.5, plot = FALSE,
                        a0per = 0.9, random.seed = 12345, filter = 1, cv.cutoff = 500, cpm = 1)
}

\references{

Bullard J.H., Purdom E., Hansen K.D. and Dudoit S. (2010) Evaluation of statistical methods for 
normalization and differential expression in mRNA-seq experiments. \emph{BMC Bioinformatics} 11(1):94+.

Mortazavi A., Williams B.A., McCue K., Schaeer L. and Wold B. (2008) Mapping and quantifying 
mammalian transcriptomes by RNA-seq. \emph{Nature Methods} 5(7):621-628.

Robinson M.D. and Oshlack A. (2010) A scaling normalization method for differential expression
analysis of RNA-seq data. \emph{Genome Biology} 11(3):R25+.

Marioni, J.C. and Mason, C.E. and Mane, S.M. and Stephens, M. and Gilad, Y. (2008) RNA-seq: an assessment of
technical reproducibility and comparison with gene expression arrays. \emph{Genome Research}, \bold{18}:
1509--1517.

}

\name{lengthbias}
\alias{lengthbias-class}
\alias{lengthbias}
\alias{explo.plot,lengthbias-method}
\alias{show,lengthbias-method}
\alias{dat2save,lengthbias-method}
\docType{class}
\title{lengthbias class}

\description{
lengthbias class generated from dat() function with type="lengthbias". 
This object contains the trimmed mean of expression for each length bin of 200 features per sample or condition and also per biotype (if available). 
It also includes the corresponding spline regression models fitted to explain the relationship between length and expression.
}

\usage{
\S4method{explo.plot}{lengthbias}(object, samples = NULL, toplot = "global", ...)
\S4method{dat2save}{lengthbias}(object)
}

\arguments{
  \item{object}{
    Object generated with \code{dat()} function.
  }
  \item{toplot}{
    Biological group to be plotted (features not belonging to that group are discarded).
    It may be a number or a text with the name of the biological group. 
    If toplot=1 or toplot="global", all features are used for the plot.
  }
  \item{samples}{
    Samples (or conditions) to be plotted. If NULL, all the samples are plotted.
    If samples > 2, only a descriptive plot will be generated. If not, diagnostic plots will be obtained showing both the R-squared and model p-value
from the spline regression model describing the relationship between the length and the expression. 
  }  
  \item{...}{
    Any argument from \code{par}.
  }
}

\section{Slots/List Components}{
Objects of this class contain (at least) the following list components:

  \code{dat}: List containing the information generated by dat() function. This list has the following elements:
  
  \code{data2plot}: A list with as many elements as biological groups (the first element correspond to all the features). 
Each element of the list is a matrix containing the length bins in the first column and an additional column 
for the trimmed mean expression per bin for each sample or condition. 
  \code{RegressionModels}: A list with as many elements as samples or conditions. 
Each element is an "lm" class object containing the spline regression model relating length and expression for that sample or condition (considering all the features).
  
}

\section{Methods}{
This class has an specific \code{show} method to print
a summary of spline regression models and a \code{dat2save} method
to save the length bin information. It also has an
\code{explo.plot} method to plot the data contained in the object.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{ARSyNseq}
\alias{ARSyNseq}
\alias{arsynseq}

\title{ASCA Removal of Systematic Noise on Seq data}

\description{
 \code{ARSyNseq} filters the noise associated to identified or not identified batch effects considering the experimental design 
and applying Principal Component Analysis (PCA) to the ANOVA parameters and residuals.
}
\usage{
ARSyNseq(data, factor = NULL, batch = FALSE, norm = "rpkm", logtransf = FALSE, Variability = 0.75, beta = 2)
}

\arguments{
  \item{data}{A Biobase's eSet object created with the \code{readData} function.}
  
  \item{factor}{Name of the factor (as it was given to the \emph{readData} function) to be used in the ARSyN model (e.g. the factor containing the batch information). When it is NULL, all the factors are considered.}
  
  \item{batch}{TRUE to indicate that the \emph{factor} argument indicates the batch information. In this case, the \emph{factor} argument must be used to specify the names of the only factor containing the information of the batch.}
  
  \item{norm}{Type of normalization to be used. One of ``rpkm'' (default), ``uqua'', ``tmm'' or ``n'' (if data are already normalized). If length was provided through the \emph{readData} function, it will be considered for the normalization (except for ``n''). Please note that if a normalization method if used, the arguments \emph{lc} and  \emph{k} are set to 1 and 0 respectively.}
  
  \item{logtransf}{If FALSE, a log-transformation will be applied on the data before computing ARSyN model 
  to improve the results of PCA on count data.}
  
  \item{Variability}{Parameter for Principal Componentents (PCs) selection of the ANOVA models effects. This is the desired proportion 
    of variability explained for the PC of the main effects (time and experimental group). Variability=0.75 by default.}
  
  \item{beta}{Parameter for PCs selection of the residual model. Components selected will be those that explain more than beta times 
		the average component variability computed as the total data variability divided by the rank of the matrix associated to the factor.
		Default beta=2. }
}

\details{
  
  When batch is identified with one of the factors described in the argument \code{factor} of the \code{data} object, \code{ARSyNseq}
	estimates this effect and removes it by estimating the main PCs of the ANOVA effects associated. Selected PCs will be those that explain 
  more than the variability proportion specified in \code{Variability}.
  
	When batch is not identified, the model estimates the effects associated to each factor of interest and analyses if there exists 
  systematic noise in the residuals. If there is batch effect, it will be identified with the main PCs of these residuals.
  Selected PCs will be those that explain more than \code{beta} times the average component variability.
   
}

\value{
The Biobase's eSet object created with the \code{readData} function that was given as input but replacing the expression data with the filtered expression data matrix.
}
\references{

Nueda, M.J.; Ferrer, A. and Conesa, A. (2012) ARSyN: a method for the identification and removal of
systematic noise in multifactorial time-course microarray experiments. \emph{Biostatistics} 13(3), 553-566.

}

\author{Maria Jose Nueda, \email{mj.nueda@ua.es} }

\examples{

# Generating an artificial batch effect from Marioni's data
data(Marioni)
set.seed(123)
mycounts2 = mycounts
mycounts2[,1:4] = mycounts2[,1:4] + runif(nrow(mycounts2)*4, 3, 5)
myfactors = data.frame(myfactors, "batch" = c(rep(1,4), rep(2,6)))
mydata2 = readData(mycounts2, factors = myfactors)

# Exploring batch effect with PCA
myPCA = dat(mydata2, type = "PCA")
par(mfrow = c(1,2))
explo.plot(myPCA, factor = "Tissue")
explo.plot(myPCA, factor = "batch")

# Removing batch effect when the batch is identified for each sample and exploring results with PCA
mydata2corr1 = ARSyNseq(mydata2, factor = "batch", batch = TRUE, norm = "rpkm",  logtransf = FALSE)
myPCA = dat(mydata2corr1, type = "PCA")
par(mfrow = c(1,2))
explo.plot(myPCA, factor = "Tissue")
explo.plot(myPCA, factor = "batch")

# If we consider that exist a batch but it is not identified (we do not know the batch information):
mydata2corr2 = ARSyNseq(mydata2, factor = "Tissue", batch = FALSE, norm = "rpkm",  logtransf = FALSE)
myPCA = dat(mydata2corr2, type = "PCA")
par(mfrow = c(1,2))
explo.plot(myPCA, factor = "Tissue")
explo.plot(myPCA, factor = "batch")
}

\keyword{ASCA, ANOVA, PCA, batch }

\name{QCreport}
\alias{QCreport}

\title{
Quality Control report for expression data
}
\description{
Generate a report with the exploratory plots for count data that can be generated from the biological information provided. This report is designed to compare two samples or two experimental conditions. 
}
\usage{
QCreport(input, file = NULL, samples = NULL, factor = NULL, norm = FALSE)
}

\arguments{
  \item{input}{
Object of eSet class coming from \code{\link{readData}} function or other R packages such as DESeq.
}
  \item{file}{
String indicating the name of the PDF file that will contain the report. It should be in this format: "filename.pdf". The default name is like this: "QCreport_2013Sep26_15:58:16.pdf".
}
  \item{samples}{
Vector with the two samples to be compared in the report when "factor" is NULL. If "factor" is not NULL and has more than two levels, samples has to indicate the two conditions to be compared. It can be numeric or character (when names of samples or conditions are provided).
}
  \item{factor}{
If NULL, individual samples indicated in "samples" are compared. Otherwise, it should be a string indicating the factor containing the experimental conditions to be compared in the report.
}
  \item{norm}{
TRUE to indicate that data are already normalized.
}
}
\value{
A pdf file.
}

\author{
Sonia Tarazona
}



\examples{
## Load the input object from Marioni's data as returned by readData()
data(myCounts)

## Generate the report
QCreport(mydata, samples = NULL, factor = "Tissue")

}

\references{
Marioni, J.C. and Mason, C.E. and Mane, S.M. and Stephens, M. and Gilad, Y. (2008) RNA-seq: an assessment of
technical reproducibility and comparison with gene expression arrays. \emph{Genome Research}, \bold{18}:
1509--1517.
}

\name{CountsBio}
\alias{CountsBio-class}
\alias{CountsBio}
\alias{show,CountsBio-method}
\alias{explo.plot,CountsBio-method}
\alias{dat2save,CountsBio-method}
\docType{class}
\title{CountsBio class}

\description{
CountsBio class generated from dat() function with
type="countsbio". This object contains the count distribution for each biological group 
and also the percentage of features with counts per million higher than 0, 1, 2, 5 or 10,
per each sample independently and in at least one of the samples (total).
}

\usage{
\S4method{explo.plot}{CountsBio}(object, samples = c(1,2), toplot = "global", plottype = c("barplot", "boxplot"),...)
\S4method{dat2save}{CountsBio}(object)
}

\arguments{
  \item{object}{
    Object generated with \code{dat()} function.
  }
  \item{toplot}{
    This parameter indicates which biological group is to be plotted.
    It may be a number or a text with the name of the biological group. 
    If toplot=1 (or "global"), a global plot with all the biological groups will be generated.

  }
  \item{samples}{
    Samples or conditions to be plotted. If NULL, the two first samples are plotted because the plot for this object only admit a maximum of two samples.
  }
  \item{plottype}{
    Type of plot to be generated for "countsbio" data. If "barplot", the plot indicates the percentage of features with counts per millior higher than 0, 1, 2, 5 or 10 counts or less. Above each bar, the sequencing depth (million reads) is shown. 
If "boxplot", a boxplot is drawn per sample or condition showing the count distribution for features with more than 0 counts. 
Both types of plot can be obtained for all features ("global") or for a specified biotype (when biotypes are available).
  }
  \item{...}{
    Any argument from \code{par}.
  }
}

\section{Slots/List Components}{
Objects of this class contain a list (\code{dat}) with the following components:
  
  \code{result}: Matrix containing the expression data for all the detected features and all samples or conditions.

  \code{bionum}: Vector containing the number of detected features per biological group (global indicates the total).
  
  \code{biotypes}: Vector containing the biological group (biotype) for each detected feature.

  \code{summary}: List with as many elements as number of biotypes and an additional element with the global information (for all features).
Each element is a data frame containing for each sample or condition the number of features with 0 counts, 1 count or less, 
2 counts or less, 5 counts or less and 10 counts or less, more than 10 counts, the total number of features and the sequencing depth.  
}

\section{Methods}{
This class has an specific \code{show} method in order to work and print
a summary of the elements which are contained and a \code{dat2save} method
to save the relevant information in an object cleanly. It also has an
\code{explo.plot} method to plot the data contained in the object.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{Normalization}
\alias{uqua}
\alias{rpkm}
\alias{tmm}

\title{
Normalization methods
}
\description{
Normalization procedures such as RPKM (Mortazavi et al., 2008),
Upper Quartile (Bullard et al., 2010) and TMM (Trimmed Mean of M) (Robinson and Oshlack, 2010). 
These normalization functions are used within the \code{noiseq} or \code{noiseqbio} functions but may be also used by themselves to normalize a dataset.
}
\usage{
uqua(datos, long = 1000, lc = 0, k = 0)
rpkm(datos, long = 1000, lc = 1, k = 0)
tmm(datos, long = 1000, lc = 0, k = 0, refColumn = 1, logratioTrim = 0.3, sumTrim = 0.05, doWeighting = TRUE, Acutoff = -1e+10)
}

\arguments{
  \item{datos}{
Matrix containing the read counts for each sample.
}
  \item{long}{
Numeric vector containing the length of the features. If long == 1000, no length correction is applied (no matter the value of parameter lc).
}
  \item{lc}{
Correction factor for length normalization. This correction is done by dividing the counts 
vector by (length/1000)^lc. If lc = 0, no length correction is applied. By default, lc = 1 for RPKM and lc = 0 for the other methods.
}
  \item{k}{
Counts equal to 0 are changed to k in order to avoid indeterminations when applying logarithms, for instance. By default, k = 0.
}
\item{refColumn}{
Column to use as reference (only needed for \code{tmm} function).
}
\item{logratioTrim}{
Amount of trim to use on log-ratios ("M" values) (only needed for \code{tmm} function).
}
\item{sumTrim}{
Amount of trim to use on the combined absolute levels ("A" values) (only needed for \code{tmm} function).
}
\item{doWeighting}{
Logical, whether to compute (asymptotic binomial precision) weights (only needed for \code{tmm} function).
}
\item{Acutoff}{
Cutoff on "A" values to use before trimming (only needed for \code{tmm} function).
}
}
\details{
\code{tmm} normalization method was taken from \emph{edgeR} package (Robinson et al., 2010). 

Although \code{Upper Quartile} and \code{TMM} methods themselves do not correct for the length of the features, these functions in
\code{NOISeq} allow users to combine the normalization procedures with an additional length correction whenever 
the length information is available.
}


\examples{

## Simulate some count data and the features length
datasim = matrix(sample(0:100, 2000, replace = TRUE), ncol = 4)
lengthsim = sample(100:1000, 500)

## RPKM normalization
myrpkm = rpkm(datasim, long = lengthsim, lc = 1, k = 0)

## Upper Quartile normalization, dividing normalized data by the square root of the features length and replacing counts=0 by k=1
myuqua = uqua(datasim, long = lengthsim, lc = 0.5, k = 1)

## TMM normalization with no length correction
mytmm = tmm(datasim, long = 1000, lc = 0, k = 0)

}


\references{

Bullard J.H., Purdom E., Hansen K.D. and Dudoit S. (2010) Evaluation of statistical methods for 
normalization and differential expression in mRNA-seq experiments. \emph{BMC Bioinformatics} 11(1):94+.

Mortazavi A., Williams B.A., McCue K., Schaeer L. and Wold B. (2008) Mapping and quantifying 
mammalian transcriptomes by RNA-seq. \emph{Nature Methods} 5(7):621-628.

Robinson M.D. and Oshlack A. (2010) A scaling normalization method for differential expression
analysis of RNA-seq data. \emph{Genome Biology} 11(3):R25+.

Robinson M.D., McCarthy D.J. and Smyth G.K. (2010) edgeR: a Bioconductor package for differential 
expression analysis of digital gene expression data. \emph{Bioinformatics} 26(1):139-140.

}
\author{
Sonia Tarazona
}

\name{Output}
\alias{Output-class}
\alias{Output}
\alias{show,Output-method}
\docType{class}
\title{Output class of NOISeq}

\description{
Output object containing the results from differential expression analysis by \code{noiseq} or \code{noiseqbio}.
}

\section{Slots/List Components}{
Objects of this class contain (at least) the following list components:
  
  \code{comparison}: String indicating the two experimental conditions being compared and the sense of the comparison.

  \code{factor}: String indicating the factor chosen to compute the differential expression.
  
  \code{k}: Value to replace zeroes in orden to avoid inderminations when computing logarithms. 
  
  \code{lc}: Correction factor for length normalization. Counts are divided by length^lc.
  
  \code{method}: Normalization method chosen. It can be one of "rpkm" (default), "uqua" (Upper Quartile), "tmm" 
		 (Trimmed Mean of M) or "n" (no normalization).

  \code{replicates}: Type of replicates: "technical" for technical replicates and "biological" for biological ones.

  \code{results}: R data frame containing the differential expression results, where each row corresponds to a feature. The columns are: Expression values for each condition to be used by \code{noiseq} 
or \code{noiseqbio} (the columns names are the levels of the factor); differential expression statistics (columns "M" and "D" for \code{noiseq} or "theta" for \code{noiseqbio}); probability of differential expression ("prob"); "ranking", which is a summary statistic of "M" and "D" values equal to -sign(M)*sqrt(M^2 + D^2), than can be used for instance in gene set enrichment analysis (only when \code{noiseq} is used); "length" and "GC" of each feature (if provided); chromosome where the feature is ("Chrom"), if provided; start and end position of the feature within the chromosome ("GeneStart", "GeneEnd"), if provided.  

  \code{nss}: Number of samples to be simulated for each condition (only when there are not replicates available). 

  \code{pnr}: Percentage of the total sequencing depth to be used in each simulated replicate (only when there are not replicates available). If, for instance, pnr = 0.2 , each simulated replicate will have 20\% of the total reads of the only available replicate in that condition.

  \code{v}: Variability of the size of each simulated replicate (only used by NOISeq-sim).

}

\section{Methods}{
This class has an specific \code{show} method in order to work and print a summary of the elements which are contained.
}

\author{Sonia Tarazona}

\keyword{classes}

\name{FilterLowCounts}
\alias{filtered.data}

\title{
Methods to filter out low count features
}
\description{
Function to filter out the low count features according to three different methods.
}
\usage{
filtered.data(dataset, factor, norm = TRUE, depth = NULL, method = 1, cv.cutoff = 100, cpm = 1, p.adj = "fdr")
}

\arguments{
  \item{dataset}{
Matrix or data.frame containing the expression values for each sample (columns) and feature (rows).
}
  \item{factor}{
Vector or factor indicating which condition each sample (column) in dataset belongs to.
}
  \item{norm}{
Logical value indicating whether the data are already normalized (TRUE) or not (FALSE).
}
  \item{depth}{
Sequencing depth of samples (column totals before normalizing the data). Depth only needs to be provided when method = 3  and norm = TRUE. 
}
\item{method}{
Method must be one of 1,2 or 3. 
Method 1 (CPM) removes those features that have an average expression per condition less than cpm value  
and a coefficient of variation per condition higher than cv.cutoff (in percentage) in all the conditions.
Method 2 (Wilcoxon) performs a Wilcoxon test per condition and feature where in the null hypothesis the median expression is 0 and
in the alternative the median is higher than 0. Those features with p-value greater than 0.05 in all the conditions are removed.
Method 3 (Proportion test) performs a proportion test on the counts per condition and feature (or pseudo-counts if data were normalized) 
where null hypothesis is that the feature relative expression (count proportion) is equal to cpm/10^6 and higher than cpm/10^6 for the alternative.
Those features with p-value greater than 0.05 in all the conditions are removed.
}
\item{cv.cutoff}{
Cutoff for the coefficient of variation per condition to be used in method 1 (in percentage).
}
\item{cpm}{
Cutoff for the counts per million value to be used in methods 1 and 3.
}
\item{p.adj}{
Method for the multiple testing correction. The same methods as in the p.adjust function in stats package can be chosen: "holm", "hochberg", "hommel", "bonferroni", "BH", "BY",
"fdr", "none". 
}
}

\examples{

## Simulate some count data
datasim = matrix(sample(0:100, 2000, replace = TRUE), ncol = 4)

## Filtering low counts (method 1)
myfilt1 = filtered.data(datasim, factor = c("cond1", "cond1", "cond2", "cond2"), norm = FALSE, depth = NULL, method = 1, cv.cutoff = 100, cpm = 1)

## Filtering low counts (method 2)
myfilt2 = filtered.data(datasim, factor = c("cond1", "cond1", "cond2", "cond2"), norm = FALSE, method = 2)

## Filtering low counts (method 3)
myfilt3 = filtered.data(datasim, factor = c("cond1", "cond1", "cond2", "cond2"), norm = FALSE, method = 3, cpm = 1)

}



\author{
Sonia Tarazona
}

\name{Differential expression plots}
\alias{DE.plot}

\title{
Plotting differential expression results
}
\description{
Function to generate plots showing different aspects of differential expression results. 
Expression plot is to compare the expression values in each condition for all features. Differentially expressed features can be highlighted.
Manhattan plot is to compare the expression values in each condition across all the chromosome positions. Differentially expressed features can also be highlighted.
MD plot shows the values for (M,D) statistics. Differentially expressed features can also be highlighted.
Distribution plot displays the percentage of differentially expressed features per chromosome and biotype (if this information is provided by the user).
}
\usage{
DE.plot(output, q = NULL, graphic = c("MD","expr","chrom","distr"), pch = 20, cex = 0.5, col = 1, pch.sel = 1, cex.sel = 0.6, col.sel = 2,log.scale = TRUE, 
chromosomes = NULL, join = FALSE,...)
}

\arguments{

  \item{output}{
Object of class \code{\link{Output}}.
}
  \item{q}{
Probability of differential expression threshold to determine differentially expressed features.
}
  \item{graphic}{
String indicating which kind of plot is to be generated. If "expr", the feature expression values are depicted. 
If "MD", the values for the (M,D) statistics when comparing both conditions are used. 
If "chrom", the feature expression values are depicted across their positions in the chromosomes (if chromosome information has been provided).
If "distr", two plots showing the percentage of differentially expressed features per both chromosome and biotype are generated 
(only if this information is available).
}
  \item{pch, cex, col,...}{
Graphical parameters as in any other R plot. See \code{\link{par}}. They do not apply for graphic="chrom".
}
  \item{pch.sel, cex.sel, col.sel}{
\code{pch}, \code{cex} and \code{col}, respectively, to represent differentially expressed features. They do not apply for graphic="chrom".
}
  \item{log.scale}{
If TRUE, log2(data+K) values are depicted instead of the expression data in the \code{\link{Output}} object. K is an appropriate constant to avoid negative values. 
It does not apply for graphic="MD" and graphic="distr".
}
  \item{chromosomes}{
Character vector indicating the chromosomes to be plotted. If NULL, all chromosomes are plotted. It only applies for graphic="chrom" and graphic="distr". For graphic="chrom", the chromosomes are plotted in the given order. In some cases (e.g. chromosome names are character strings), it is very convenient to specify the order although all chromosomes are being plotted. For graphic="distr", the chromosomes are plotted according to the number of features they contain (from the highest number to the lowest).
}
  \item{join}{
If FALSE, each chromosome is depicted in a separate line. If TRUE, all the chromosomes are depicted in the same line, consecutively (useful for prokaryote organisms). It only applies for graphic="chrom".
}
}

\author{
Sonia Tarazona
}



\seealso{
       \code{\link{readData}}, \code{\link{noiseq}}, \code{\link{degenes}}.
     }

\examples{
## We load the object generated after running noiseq on Marioni's data
data(noiseq)

## Third, plot the expression values for all genes and highlighting the differentially expressed genes
DE.plot(mynoiseq, q = 0.8, graphic = "expr", log.scale = TRUE)
DE.plot(mynoiseq, q = 0.8, graphic = "MD")
DE.plot(mynoiseq, chromosomes = c(1,2), log.scale = TRUE,join = FALSE, q = 0.8, graphic = "chrom")
DE.plot(mynoiseq, chromosomes = NULL, q = 0.8, graphic = "distr")
}

\name{degenes}
\alias{degenes}

\title{
Recovering differencially expressed features.
}
\description{
Recovering differencially expressed features for a given threshold from \code{noiseq} or \code{noiseqbio} output objects.
}
\usage{
degenes(object, q = 0.95, M = NULL)
}

\arguments{
  \item{object}{
Object of class \code{\link{Output}}.
}
  \item{q}{
Value for the probability threshold (by default, 0.95).
}
  \item{M}{
String indicating if all differentially expressed features are to be returned or only up or down-regulated features. The possible values are:
"up" (up-regulated in condition 1), "down" (down-regulated in condition 1), or NULL (all differentially expressed features).
}
}
\value{
A matrix containing the differencially expressed features, the statistics and the probability of differential expression.
}
\author{
Sonia Tarazona
}


\seealso{
       \code{\link{readData}}, \code{\link{noiseq}}, \code{\link{noiseqbio}}.
     }

\examples{
## Load the object mynoiseq generated by computing differential expression probability with noiseq() on Marioni's data:
data(noiseq)

## Third, use degenes() function to extract differentially expressed features:
mynoiseq.deg = degenes(mynoiseq, q = 0.8, M = NULL)

}

\references{
Marioni, J.C. and Mason, C.E. and Mane, S.M. and Stephens, M. and Gilad, Y. (2008) RNA-seq: an assessment of
technical reproducibility and comparison with gene expression arrays. \emph{Genome Research}, \bold{18}:
1509--1517.
}

\name{Data2Save}
\alias{dat2save}

\docType{methods}

\title{
Saving data generated for exploratory plots.
}

\description{
This function is to save the data generated to draw the exploratory plots in a user-friendly format.
}

\value{
The dat2save() function takes the object generated by dat() and creates
a new one with the most relevant information.
}


\author{
Sonia Tarazona
}


\seealso{
       \code{\link{readData}}, \code{\link{addData}}, \code{\link{dat}},
       \code{\link{explo.plot}}.
     }

\examples{
## Load the input object with the expression data and the annotations
data(myCounts)

## Generating data for the plot "biodetection" and samples in columns 3 and 4 of expression data
mydata2plot = dat(mydata, type = "biodetection", k = 0)

## Save the relevant information cleanly
mydata2save = dat2save(mydata2plot)

}

\name{readData}
\alias{readData}
\alias{addData}

\title{
Creating an object of eSet class
}
\description{
This function is to create an object of eSet class to be used by NOISeq functions from matrix or data.frame R objects.
}
\usage{
readData(data, factors, length = NULL, biotype = NULL, chromosome = NULL, gc = NULL)
addData(data, length = NULL, biotype = NULL, chromosome = NULL, factors = NULL, gc = NULL)
}

\arguments{
  \item{data}{
Matrix or data.frame containing the counts (or expression data) for each feature and sample. Features must be in rows and samples must be in columns.
}

  \item{factors}{
A data.frame containing the experimental condition or group for each sample (columns in the \code{data} object).
}
  \item{biotype}{
    Optional argument.Vector, matrix or data.frame containing the biological group (biotype)
  for each feature. In case of giving a vector, the names of the
vector must be the feature names or ids  with the same type of
identifier used in \code{data}. If a matrix or a data.frame is provided, 
and it has two columns, it is expected that the feature names or ids are
in the first column and the biotypes of the features in the
second. If it only has one column containing the biotypes, the rownames of the
object must be the feature names or ids.
}
  \item{chromosome}{
Optional argument. A matrix or data.frame containing the chromosome, start position and end position of each feature. 
The rownames must be the feature names or ids with the same type of identifier used in \code{data}.
}

\item{gc}{
Optional argument.Vector, matrix or data.frame containing the GC content
  of each feature. In case of giving a vector, the names of the
vector must be the feature names or ids  with the same type of
identifier used in \code{data}. If a matrix or a data.frame is provided, 
and it has two columns, it is expected that the feature names or ids are
in the first column and the GC content of the features in the
second. If it only has one column containing the GC content, the rownames of the
object must be the feature names or ids.
}

  \item{length}{
    Optional argument.Vector, matrix or data.frame containing the length
  of each feature. In case of giving a vector, the names of the
vector must be the feature names or ids  with the same type of
identifier used in \code{data}. If a matrix or a data.frame is provided, 
and it has two columns, it is expected that the feature names or ids are
in the first column and the length of the features in the
second. If it only has one column containing the length, the rownames of the
object must be the feature names or ids.
}
}

\value{
It returns an object of eSet class \code{\link{myCounts}} with all the information defined and ready to be used.
}

\author{
Sonia Tarazona
}

\examples{
# Load an object containing the information explained above
data(Marioni)

# Create the object with the data        
mydata <- readData(data=mycounts, biotype=mybiotypes, chromosome=mychroms, factors=myfactors)

# Add length annotation to the existing data object
mydata <- addData(mydata, length=mylength)
}

\references{ 
    Marioni, J.C. and Mason, C.E. and Mane, S.M. and Stephens, M. and Gilad, Y. (2008) RNA-seq: an assessment of
technical reproducibility and comparison with gene expression arrays. \emph{Genome Research}, \bold{18}:
1509--1517.
}
